# -*- coding: utf-8 -*-
import sys
import json

import numpy as np

import cartopy.crs as ccrs
import cartopy.feature as cfeature
import matplotlib as mpl
import matplotlib.pyplot as plt

import netCDF4

from cartopy.mpl.ticker import LongitudeFormatter, LatitudeFormatter
from cartopy.util import add_cyclic_point
from matplotlib.colors import ListedColormap

#import py3grads
#from pathlib import Path

#----------------------------------------------
colors = [ "whitesmoke", "black"]
fill_cmap = ListedColormap(colors,"fillcmap")
mpl.colormaps.register(fill_cmap)
#----------------------------------------------
# basin mask

path_mask = '../data/np10km/mask'
file_mask = path_mask + '/' + 'pacific_mask3_np.gd'

nxm=2049
nym=784

fl = open(file_mask,'rb')
mask_tmp = np.fromfile(fl, dtype = '>f', count= nxm*nym)
mask=mask_tmp.astype(np.float64).reshape(nym,nxm).copy()
maskbnd=mask_tmp.astype(np.float64).reshape(nym,nxm).copy()

latb = np.array(np.empty((nym)),dtype=np.float64)
for j in range(nym):
    latb[j] = -15.0 + 1/10 * (j - 1.5)

lonb = np.array(np.empty((nxm)),dtype=np.float64)
for i in range(nxm):
    lonb[i] = 99.0 + 1/11 * (i - 1.5)

#path_mask = '.'
#file_mask_ctl = path_mask + '/' + 'pacific_mask_np.ctl'
#file_mask_ctl = path_mask + '/' + 'pacific_mask2_np.ctl'
#file_mask_ctl = path_mask + '/' + 'pacific_mask3_np.ctl'
#file_mask_ctl = path_mask + '/' + 'np_24n165e.ctl'

#outfile = 'pac_region_mask_np.png'
#outfile = 'pac_region_mask2_np.png'
outpng = './fig/pac_region_mask3_np.png'
outpdf = './fig/pac_region_mask3_np.pdf'
#outfile = '24n165e_np.png'

#ga = py3grads.Grads(verbose=False)
#print(file_mask_ctl)
#ga('open '+str(file_mask_ctl))

#ga('set t 1')
#ga('set y 1')
#ga('set x 1 2049')
#lons = ga.exp('lon')
#print(lons.shape)
#ga('set x 1')
#ga('set y 1 784')
#lats = ga.exp('lat')
#print(lats.shape)
#
#ga('set x 1 2049')
#ga('set y 1 784')
#
#mask = ga.exp('index')
#print(mask.shape)
#
#nx=len(lons)
#lonb=np.array(np.empty((nx+1)),dtype=np.float64)
#lonb[0] = lons[0] - 0.5 * (lons[1]-lons[0])
#for i in range(1,nx):
#   lonb[i] = 0.5 * (lons[i-1]+lons[i])
#
#lonb[nx] = lons[nx-1] + 0.5 * (lons[nx-1]-lons[nx-2])
#
#print(lonb)
#
#ny=len(lats)
#latb=np.array(np.empty((ny+1)),dtype=np.float64)
#latb[0] = lats[0] - 0.5 * (lats[1]-lats[0])
#for i in range(1,ny):
#   latb[i] = 0.5 * (lats[i-1]+lats[i])
#
#latb[ny] = lats[ny-1] + 0.5 * (lats[ny-1]-lats[ny-2])

#print(latb)

#mask = np.where(mask==21,10,mask)
#mask = np.where(mask==22,9,mask)
#mask = np.where(mask==23,8,mask)
#mask = np.where(mask==24,0,mask)
#mask = np.where(mask==25,6,mask)

mask = np.where(mask==-9,np.nan,mask)
mask = np.where(mask>10,20.0,mask)

maskbnd[:,:] = mask[:,:]
for j in range(nym):
   for i in range(2,nxm-2):
      if ( not np.isnan(mask[j,i-2]) and mask[j,i]==20):
         maskbnd[j,i-2]=20
      if ( not np.isnan(mask[j,i-1]) and mask[j,i]==20):
         maskbnd[j,i-1]=20
      if ( not np.isnan(mask[j,i+1]) and mask[j,i]==20):
         maskbnd[j,i+1]=20
      if ( not np.isnan(mask[j,i+2]) and mask[j,i]==20):
         maskbnd[j,i+2]=20

for j in range(1,nym-1):
   for i in range(nxm):
      if ( not np.isnan(mask[j-1,i]) and mask[j,i]==20):
         maskbnd[j-1,i]=20
      if ( not np.isnan(mask[j+1,i]) and mask[j,i]==20):
         maskbnd[j+1,i]=20

maskbnd = np.where(maskbnd==0,np.nan,maskbnd)
maskbnd = np.where(maskbnd<10,-10,maskbnd)

#-------------------------------------
# Draw Figures

title = ' Pacific region mask for NP model '

ct = np.arange(1,20.1,2)

fig = plt.figure(figsize=(11,8.5))
#fig.suptitle( suptitle, fontsize=20 )

proj = ccrs.PlateCarree(central_longitude=-160.)
lon_formatter = LongitudeFormatter(zero_direction_label=True)
lat_formatter = LatitudeFormatter()

ax = fig.add_subplot(1,1,1,projection=proj)


ca=ax.pcolormesh(lonb, latb, maskbnd, cmap='fillcmap', transform=ccrs.PlateCarree())
#ca=ax.pcolormesh(lonb, latb, mask, cmap='gray', vmin=15, transform=ccrs.PlateCarree())
#ca=ax.pcolormesh(lonb, latb, mask, color='black', vmin=15, transform=ccrs.PlateCarree())
ax.coastlines()
ax.set_xticks(np.arange(120,285.1,30),crs=ccrs.PlateCarree())
ax.set_yticks(np.arange(-10,60.1,10),crs=ccrs.PlateCarree())
ax.set_extent([97,288,-18,66],crs=ccrs.PlateCarree())
ax.xaxis.set_major_formatter(lon_formatter)
ax.yaxis.set_major_formatter(lat_formatter)
ax.set_aspect(1.3)
#ax.set_title(title)
#fig.colorbar(ca,ax=ax,ticks=ct,location='bottom')
plt.savefig(outpng, bbox_inches='tight', pad_inches=0.01)
plt.savefig(outpdf, bbox_inches='tight', pad_inches=0.01)
plt.show()
