# -*- coding: utf-8 -*-
import sys
import json
import numpy as np
import netCDF4
import xarray as xr
import cartopy.crs as ccrs
import matplotlib as mpl
import matplotlib.pyplot as plt
from cartopy.mpl.ticker import LongitudeFormatter, LatitudeFormatter
import datetime

########

ro = 1036.0
ysec = 86400*365.25

if (len(sys.argv) < 2):
    print ('Usage: ' + sys.argv[0] + ' start_year end_year dens1 dens2')
    sys.exit()

ystr = int(sys.argv[1])
yend = int(sys.argv[2])
dens1 = float(sys.argv[3])
dens2 = float(sys.argv[4])
densm = 0.5 * (dens1+dens2)

nyr = yend-ystr+1

title = [ r'(a) C$_{\mathrm{anth}}$ flux by density change through ',
          r'(b) C$_{\mathrm{anth}}$ flux by density change through ',
          r'(c) C$_{\mathrm{anth}}$ flux by density change through ',
          r'(d) C$_{\mathrm{anth}}$ flux by density change through ',
          r'(e) Convergence of C$_{\mathrm{anth}}$ flux (c-a) for ',
          r'(f) Convergence of C$_{\mathrm{anth}}$ flux (d-b) for ',
          r'(g) Surface uptake of C$_{\mathrm{anth}}$ for ',
          r'(h) Surface uptake of C$_{\mathrm{anth}}$ for ', ]

suptitle = r'C$_{\mathrm{anth}}$ budget for ' + r'$\sigma_0$ = ' + str(dens1) + '-' + str(dens2) + ' ('+str(ystr)+'-'+str(yend)+' mean)'
outfile = './fig/Cant_budget_' + str(int(dens1*100)) + '-' + str(int(dens2*100))+'_v2'

print("Drawing "+outfile)

####################################################################

time = np.empty(yend-ystr+1,dtype='object')
for yr in range(ystr,yend+1):
    time[yr-ystr] = datetime.datetime(yr,1,1)

####################################################################
path_glb100km='../data/glb100km/'
path_np10km='../data/np10km/'
####################################################################
# Diapycnal Transport

# GLB100km

path_g100=path_glb100km + 'run-20211104o2/ANALY/annclim'

dpy_g100 = 'hs_dicanth_e_sig_exp_sigf_anncl_woa1x1.'+str(ystr)+'-'+str(yend)+'.nc'
dpy_file = path_g100 + '/' + dpy_g100
ncg100 = netCDF4.Dataset(dpy_file,'r')
lonf_mod = ncg100.variables['lon'][:]
latf_mod = ncg100.variables['lat'][:]
depf_mod = ncg100.variables['depth'][:]
nxf = len(ncg100.dimensions['lon'])
nyf = len(ncg100.dimensions['lat'])
nzf = len(ncg100.dimensions['depth'])

dpy_sig_tmp = np.array(np.empty((nzf,nyf,nxf)),dtype=np.float64)

dpy_sig_g100 = np.array(np.empty((nzf,nyf,nxf)),dtype=np.float64)
dpy_sig_tmp[:,:,:] = ncg100.variables['dicanthdpyexp'][0,:,:,:]
dpy_sig_g100 = np.where(dpy_sig_tmp < -9.0e33, np.NaN, dpy_sig_tmp*1e-2*ysec) # cm/s->m/s

ncg100.close()

kz1=0
sigdist=abs(depf_mod[kz1]-dens1)
for k in range(1,nzf):
   if (abs(depf_mod[k]-dens1) < sigdist):
       kz1=k
       sigdist = abs(depf_mod[k]-dens1)

kz2=0
sigdist=abs(depf_mod[kz2]-dens2)
for k in range(1,nzf):
   if (abs(depf_mod[k]-dens2) < sigdist):
       kz2=k
       sigdist = abs(depf_mod[k]-dens2)

print("drawing at ", depf_mod[kz1], depf_mod[kz2])

# NP10km

path_fill=path_np10km+'hst_annclim-glb_fill'

dpy_fill = 'hs_dicanth_e_sig_exp_sigf_anncl_woa1x1.'+str(ystr)+'-'+str(yend)+'.nc'
dpy_file = path_fill + '/' + dpy_fill
ncfill = netCDF4.Dataset(dpy_file,'r')
dpy_sig_fill = np.array(np.empty((nzf,nyf,nxf)),dtype=np.float64)

dpy_sig_tmp[:,:,:] = ncfill.variables['dicanthdpyexp'][0,:,:,:]
dpy_sig_fill = np.where(dpy_sig_tmp < -9.0e33, np.NaN, dpy_sig_tmp*1.0e-2*ysec)

ncfill.close()

##########

DS_dpy = xr.Dataset( {'GLB100km': (['density','lat','lon'], dpy_sig_g100[:,:,:]),
                      'NP10km': (['density','lat','lon'], dpy_sig_fill[:,:,:]),},
                      coords = { 'density': depf_mod, 'lat': latf_mod, 'lon': lonf_mod, } )

#print(DS_dpy)

DS_conv = xr.Dataset( {'GLB100km': (['lat','lon'], dpy_sig_g100[kz2,:,:]-dpy_sig_g100[kz1,:,:]),
                       'NP10km': (['lat','lon'], dpy_sig_fill[kz2,:,:]-dpy_sig_fill[kz1,:,:]),},
                      coords = { 'lat': latf_mod, 'lon': lonf_mod, } )

####################################################################
# Surface uptake

# GLB100km

path_g100=path_glb100km+'run-20211104o2/ANALY/annclim'

dic_g100 = 'hs_DIC_tend_sflx_sig_anncl_woa1x1.'+str(ystr)+'-'+str(yend)+'.nc'
dic_file = path_g100 + '/' + dic_g100
ncg100 = netCDF4.Dataset(dic_file,'r')
lon_mod = ncg100.variables['lon'][:]
lat_mod = ncg100.variables['lat'][:]
dep_mod = ncg100.variables['depth'][:]
nx = len(ncg100.dimensions['lon'])
ny = len(ncg100.dimensions['lat'])
nz = len(ncg100.dimensions['depth'])

dic_sig_tmp = np.array(np.empty((nz,ny,nx)),dtype=np.float64)

dic_sig_g100 = np.array(np.empty((nz,ny,nx)),dtype=np.float64)
dic_sig_g100[:,:,:] = ncg100.variables['fgco2'][0,:,:,:].filled(fill_value=np.nan)
#dic_sig_g100 = dic_sig_tmp.filled(fill_value=np.NaN)
#dic_sig_g100 = np.where(dic_sig_tmp < -9.0e33, np.NaN, dic_sig_tmp) 
#print(dic_sig_g100)

ncg100.close()

dic_prei_g100 = 'hs_DIC_prei_tend_sflx_sig_anncl_woa1x1.'+str(ystr)+'-'+str(yend)+'.nc'
dic_prei_file = path_g100 + '/' + dic_prei_g100
ncg100 = netCDF4.Dataset(dic_prei_file,'r')
dic_prei_sig_tmp = np.array(np.empty((nz,ny,nx)),dtype=np.float64)
dic_prei_sig_g100 = np.array(np.empty((nz,ny,nx)),dtype=np.float64)
dic_prei_sig_g100[:,:,:] = ncg100.variables['fgco2nat'][0,:,:,:].filled(fill_value=np.nan)
#dic_prei_sig_g100 = np.where(dic_prei_sig_tmp < -9.0e33, np.NaN, dic_prei_sig_tmp) 

ncg100.close()

thick_g100 = 'hs_thkh_anncl_woa1x1.'+str(ystr)+'-'+str(yend)+'.nc'
thick_file = path_g100 + '/' + thick_g100
ncg100 = netCDF4.Dataset(thick_file,'r')
thick_sig_tmp = np.array(np.empty((nz,ny,nx)),dtype=np.float64)
thick_sig_g100 = np.array(np.empty((nz,ny,nx)),dtype=np.float64)
thick_sig_g100[:,:,:] = ncg100.variables['thickness'][0,:,:,:].filled(fill_value=np.nan)
#thick_sig_g100 = np.where(thick_sig_tmp < -9.0e33, np.NaN, thick_sig_tmp) 

ncg100.close()

# NP10km

path_fill=path_np10km+'hst_annclim-glb_fill'

dic_fill = 'hs_DIC_tend_sflx_sig_anncl_woa1x1.'+str(ystr)+'-'+str(yend)+'.nc'
dic_file = path_fill + '/' + dic_fill
ncfill = netCDF4.Dataset(dic_file,'r')

dic_sig_tmp = np.array(np.empty((nz,ny,nx)),dtype=np.float64)

dic_sig_fill = np.array(np.empty((nz,ny,nx)),dtype=np.float64)
dic_sig_fill[:,:,:] = ncfill.variables['fgco2'][0,:,:,:].filled(fill_value=np.nan)
#dic_sig_fill = np.where(dic_sig_tmp < -9.0e33, np.NaN, dic_sig_tmp) 

ncfill.close()

dic_prei_fill = 'hs_DIC_prei_tend_sflx_sig_anncl_woa1x1.'+str(ystr)+'-'+str(yend)+'.nc'
dic_prei_file = path_fill + '/' + dic_prei_fill
ncfill = netCDF4.Dataset(dic_prei_file,'r')
dic_prei_sig_tmp = np.array(np.empty((nz,ny,nx)),dtype=np.float64)
dic_prei_sig_fill = np.array(np.empty((nz,ny,nx)),dtype=np.float64)
dic_prei_sig_fill[:,:,:] = ncfill.variables['fgco2nat'][0,:,:,:].filled(fill_value=np.nan)
#dic_prei_sig_fill = np.where(dic_prei_sig_tmp < -9.0e33, np.NaN, dic_prei_sig_tmp) 

ncfill.close()

thick_fill = 'hs_thkh_anncl_woa1x1.'+str(ystr)+'-'+str(yend)+'.nc'
thick_file = path_fill + '/' + thick_fill
ncfill = netCDF4.Dataset(thick_file,'r')
thick_sig_tmp = np.array(np.empty((nz,ny,nx)),dtype=np.float64)
thick_sig_fill = np.array(np.empty((nz,ny,nx)),dtype=np.float64)
thick_sig_fill[:,:,:] = ncfill.variables['thickness'][0,:,:,:].filled(fill_value=np.nan)
#thick_sig_fill = np.where(thick_sig_tmp < -9.0e33, np.NaN, thick_sig_tmp) 

ncfill.close()

##########

DS_sflx = xr.Dataset( {'GLB100km': (['depth','lat','lon'], (dic_sig_g100-dic_prei_sig_g100)*thick_sig_g100*ysec),
                       'NP10km': (['depth','lat','lon'], (dic_sig_fill-dic_prei_sig_fill)*thick_sig_fill*ysec),},
                      coords = { 'depth':dep_mod, 'lat': lat_mod, 'lon': lon_mod, } )

######################################################################
# Montgomery potential

# GLB100km

print( "Loading GLB100km data" )

path_g100=path_glb100km+'run-20211104o2/ANALY/monthly'

base_mont_g100 = 'hs_montgomery_pot_woa1x1'

suffix = '.' + '{:04}'.format(yr)
mont_file = path_g100 + '/' + base_mont_g100 + suffix
ncg100 = netCDF4.Dataset(mont_file,'r')
lon_mod = ncg100.variables['lon'][:]
lat_mod = ncg100.variables['lat'][:]
dep_mod = ncg100.variables['depth'][:]
nx = len(ncg100.dimensions['lon'])
ny = len(ncg100.dimensions['lat'])
nz = len(ncg100.dimensions['depth'])
ncg100.close()

kzm1=0
for k in range(1,nz):
   if (dep_mod[k] > dens1):
       kzm1=k
       break

kzm2=0
for k in range(1,nz):
   if (dep_mod[k] > dens2):
       kzm2=k
       break

print("motgomery potential at ", dep_mod[kzm1], dep_mod[kzm2])

kzm=0
sigdist=abs(dep_mod[kzm]-densm)
for k in range(1,nz):
   if (abs(dep_mod[k]-densm) < sigdist):
       kzm=k
       sigdist = abs(dep_mod[k]-densm)

print("central density at ", dep_mod[kzm])


mont_sig_tmp = np.array(np.empty((12,nz,ny,nx)),dtype=np.float64)
mont_sig_g100 = np.array(np.empty((nyr,nz,ny,nx)),dtype=np.float64)
base_mont_g100 = 'hs_montgomery_pot_woa1x1'

for yr in range(ystr,yend+1):

    suffix = '.' + '{:04}'.format(yr)

    mont_sig_file = path_g100 + '/' + base_mont_g100 + suffix
    ncg100 = netCDF4.Dataset(mont_sig_file,'r')
    mont_sig_tmp[:,:,:,:] = ncg100.variables['mont'][:,:,:,:]
    ncg100.close()

    mont_sig_tmp = np.where(mont_sig_tmp < -9.0e33, np.NaN, mont_sig_tmp)
    mont_sig_g100[yr-ystr,:,:,:] = np.nanmean(mont_sig_tmp,axis=0)


DS_g100_ann = xr.Dataset( {'mont_sig': (['time','density','lat','lon'], mont_sig_g100),},
                          coords = { 'time': time,'density': dep_mod,'lat': lat_mod,'lon': lon_mod, } )


####################################################################
# NP10km

print( "Loading NP10km data" )

path_fill=path_np10km+'hst_mon-glb_fill'

mont_sig_fill = np.array(np.empty((nyr,nz,ny,nx)),dtype=np.float64)

base_mont_fill = 'hs_montgomery_pot_woa1x1'

for yr in range(ystr,yend+1):
    for mn in range(1,13):

        suffix = '.' + '{:04}{:02}'.format(yr,mn)

        mont_sig_file = path_fill + '/y' + '{:04}'.format(yr) + '/' + base_mont_fill + suffix
        ncfill = netCDF4.Dataset(mont_sig_file,'r')
        mont_sig_tmp[mn-1,:,:,:] = ncfill.variables['mont'][0,:,:,:]
        ncfill.close()

    mont_sig_tmp = np.where(mont_sig_tmp < -9.0e33, np.NaN, mont_sig_tmp)
    mont_sig_fill[yr-ystr,:,:,:] = np.nanmean(mont_sig_tmp,axis=0)

DS_fill_ann = xr.Dataset( {'mont_sig': (['time','density','lat','lon'], mont_sig_fill),},
                          coords = { 'time': time,'density': dep_mod,'lat': lat_mod,'lon': lon_mod, } )

#print(DS_fill_ann)

####################################################################

bias = np.empty( (4,ny,nx) )

da3 = DS_g100_ann.mont_sig.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)
da4 = DS_fill_ann.mont_sig.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)

DS_mont = xr.Dataset( {'GLB100km': (['density', 'lat','lon'], da3.values),
                       'NP10km': (['density','lat','lon'], da4.values),
                       'NP10km-GLB100km': (['density','lat','lon'], da4.values - da3.values), },
                      coords = { 'density': dep_mod,
                                 'lat': np.linspace(-89.5,89.5,num=180), 
                                 'lon': np.linspace(0.5,359.5,num=360), } )

#######################################################


print( 'Start drawing' )
fig = plt.figure(figsize=(8.5,11.0))
fig.suptitle( suptitle, fontsize=16, y=0.985 )

proj = ccrs.PlateCarree(central_longitude=-140.)
lon_formatter = LongitudeFormatter(zero_direction_label=True)
lat_formatter = LatitudeFormatter()

# [left, bottom, width, height]

ax = [
    plt.axes([0.07,0.775,0.4,0.155],projection=proj),
    plt.axes([0.55,0.775,0.4,0.155],projection=proj),
    plt.axes([0.07,0.540,0.4,0.155],projection=proj),
    plt.axes([0.55,0.540,0.4,0.155],projection=proj),
    plt.axes([0.07,0.305,0.4,0.155],projection=proj),
    plt.axes([0.55,0.305,0.4,0.155],projection=proj),
    plt.axes([0.07,0.070,0.4,0.155],projection=proj),
    plt.axes([0.55,0.070,0.4,0.155],projection=proj),
]

ax_cbar = [
    plt.axes([0.07,0.745,0.4,0.012]),
    plt.axes([0.55,0.745,0.4,0.012]),
    plt.axes([0.07,0.510,0.4,0.012]),
    plt.axes([0.55,0.510,0.4,0.012]),
    plt.axes([0.07,0.275,0.4,0.012]),
    plt.axes([0.55,0.275,0.4,0.012]),
    plt.axes([0.07,0.04,0.4,0.012]),
    plt.axes([0.55,0.04,0.4,0.012]),
]


plt.text(0.27,0.945,'Low Resolution',ha='center',size=16,transform=fig.transFigure)
plt.text(0.75,0.945,'High Resolution',ha='center',size=16,transform=fig.transFigure)

cmap = [ 'RdYlBu_r', 'RdBu_r', 'RdBu_r' ]
model = [ 'GLB100km', 'NP10km']

#units = r'[$ \times 10^{-9} \mathrm{mol}\, \mathrm{m}^{-2} \mathrm{s}^{-1}$]'
units = r'[$ \mathrm{mol}\, \mathrm{m}^{-2} \mathrm{yr}^{-1}$]'

panel=0

for item in range(3):

    if (item == 0):

#        bounds1 = [-200.0, -100.0, -50.0, -20.0, -10.0, -5.0, 0.0, 5.0, 10.0, 20.0, 50.0, 100.0, 200.0]
        bounds1 = [-10.0, -5.0, -3.0, -2.0, -1.0, -0.5, -0.2, 0.2, 0.5, 1.0, 2.0, 3.0, 5.0, 10.0]
        #bounds2 = [-2.0, -1.0, -0.5, -0.2, -0.1, -0.05, 0.05, 0.1, 0.2, 0.5, 1.0, 2.0]
        #bounds3 = [0.0, 0.01, 0.02, 0.05, 0.1, 0.15, 0.2, 0.3, 0.4, 0.5, 0.7, 1.0]
        bounds4 = np.arange(-10000,10000.1,1000)

        for level in range(2):

            if (level == 0):
                klev=kz1
                klevm=kzm1
            elif (level == 1):
                klev=kz2
                klevm=kzm2

            for mod in range(2):

                print("Drawing panel "+ str(panel))

                if (level == 0):
                    bounds = bounds1
                    ticks_bounds = bounds1
                    da = DS_dpy[model[mod]].isel(density=klev)
                elif (level == 1):
                    bounds = bounds1
                    ticks_bounds = bounds1
                    da = DS_dpy[model[mod]].isel(density=klev)

                im = da.plot(ax=ax[panel],cmap=cmap[item],
                             levels=bounds,
                             extend='both',
                             add_colorbar=False,
                             transform=ccrs.PlateCarree())

                cbar = plt.colorbar(im,orientation='horizontal',spacing='uniform',label=units,
                                    ticks=ticks_bounds, cax=ax_cbar[panel])
                cbar.set_label(label=units,fontsize=7,x=0.95,labelpad=0)
                cbar.ax.tick_params(labelsize=7)

                DS_mont[model[mod]].isel(density=klevm).plot.contour(ax=ax[panel],
                                                                    levels=bounds4,
                                                                    colors='black',add_colorbar=False,
                                                                    linewidths=0.5,
                                                                    transform=ccrs.PlateCarree())


                ax[panel].coastlines()
#                ax[panel].set_xticks(np.arange(120,200.1,10),crs=ccrs.PlateCarree())
                ax[panel].set_xticks(np.arange(120,250.1,20),crs=ccrs.PlateCarree())
                ax[panel].set_yticks(np.arange(10,60.1,5),crs=ccrs.PlateCarree())
                ax[panel].set_extent([114,251,9,61],crs=ccrs.PlateCarree())
#                ax[panel].set_extent([114,201,9,61],crs=ccrs.PlateCarree())
                ax[panel].set_aspect(1.2)
#                ax[panel].set_aspect(1.0)
                ax[panel].xaxis.set_major_formatter(lon_formatter)
                ax[panel].yaxis.set_major_formatter(lat_formatter)
                ax[panel].set_xlabel('')
                ax[panel].set_title(title[panel]+r' $\sigma_{\theta}$ ='+str(depf_mod[klev]),{'fontsize':10, 'verticalalignment':'top'})
                ax[panel].tick_params(labelsize=7)
                ax[panel].patch.set_facecolor('lightgray')

                panel+=1

    else:
        bounds1 = [-5.0, -2.0, -1.0, -0.5, -0.3, -0.2, -0.1, 0.1, 0.2, 0.3, 0.5, 1.0, 2.0, 5.0]
        #bounds1 = [-100.0, -50.0, -20.0, -10.0, -5.0, -2.0, 0.0, 2.0, 5.0, 10.0, 20.0, 50.0, 100.0]
        #bounds2 = [-2.0, -1.0, -0.5, -0.2, -0.1, -0.05, 0.05, 0.1, 0.2, 0.5, 1.0, 2.0]
        #bounds3 = [0.0, 0.01, 0.02, 0.05, 0.1, 0.15, 0.2, 0.3, 0.4, 0.5, 0.7, 1.0]
        #bounds4 = np.arange(-5000,5000.1,500)
        bounds4 = np.arange(-10000,10000.1,1000)
        for mod in range(2):

            print("Drawing panel "+ str(panel))

            bounds = bounds1
            ticks_bounds = bounds1
            if (item == 1):
                da = DS_conv[model[mod]]
            if (item == 2):
                da = DS_sflx[model[mod]].isel(depth=slice(kz1,kz2)).sum(dim='depth',skipna=False)

            im = da.plot(ax=ax[panel],cmap=cmap[item],
                         levels=bounds,
                         extend='both',
                         add_colorbar=False,
                         transform=ccrs.PlateCarree())

            cbar = plt.colorbar(im,orientation='horizontal',spacing='uniform',label=units,
                                ticks=ticks_bounds, cax=ax_cbar[panel])
            cbar.set_label(label=units,fontsize=7,x=0.95,labelpad=0)
            cbar.ax.tick_params(labelsize=7)

            DS_mont[model[mod]].isel(density=kzm).plot.contour(ax=ax[panel],
                                                              levels=bounds4,
                                                              colors='black',add_colorbar=False,
                                                              linewidths=0.5,
                                                              transform=ccrs.PlateCarree())

            ax[panel].coastlines()
#            ax[panel].set_xticks(np.arange(120,200.1,10),crs=ccrs.PlateCarree())
            ax[panel].set_xticks(np.arange(120,250.1,20),crs=ccrs.PlateCarree())
            ax[panel].set_yticks(np.arange(10,60.1,5),crs=ccrs.PlateCarree())
#            ax[panel].set_extent([114,201,9,61],crs=ccrs.PlateCarree())
            ax[panel].set_extent([114,251,9,61],crs=ccrs.PlateCarree())
#            ax[panel].set_aspect(1.0)
            ax[panel].set_aspect(1.2)
            ax[panel].xaxis.set_major_formatter(lon_formatter)
            ax[panel].yaxis.set_major_formatter(lat_formatter)
            ax[panel].set_xlabel('')
            ax[panel].set_title(title[panel]+r' $\sigma_{\theta}$ ='+str(depf_mod[kz1])+'-'+str(depf_mod[kz2]),{'fontsize':10, 'verticalalignment':'top'})
            ax[panel].tick_params(labelsize=7)
            ax[panel].patch.set_facecolor('lightgray')

            panel+=1


#plt.subplots_adjust(left=0.07,right=0.98,bottom=0.05,top=0.92,wspace=0.16,hspace=0.15)

outpdf = outfile+'.pdf'
outpng = outfile+'.png'

plt.savefig(outpng, bbox_inches='tight', pad_inches=0.1)
plt.savefig(outpdf, bbox_inches='tight', pad_inches=0.1)

print("figure is saved to " + outpng + " and " + outpdf)

plt.show()
