# -*- coding: utf-8 -*-
import sys
import json
import numpy as np
import netCDF4
import xarray as xr
import cartopy.crs as ccrs
import matplotlib as mpl
import matplotlib.pyplot as plt
from cartopy.mpl.ticker import LongitudeFormatter, LatitudeFormatter
import datetime

########

ro = 1036.0

if (len(sys.argv) < 2):
    print ('Usage: ' + sys.argv[0] + ' start_year end_year dens1 dens2')
    sys.exit()

ystr = int(sys.argv[1])
yend = int(sys.argv[2])
dens1 = float(sys.argv[3])
dens2 = float(sys.argv[4])
densm = 0.5 * (dens1+dens2)
nyr = yend-ystr+1

model_title = [ ' ', ' ', 'difference (HighRes - LowRes)' ]
item_title = [ r'C$_{\mathrm{anth}}$ storage', 'thickness' ]
fig_title = [ '(a)', '(b)', '(c)', '(d)', '(e)', '(f)', '(g)', '(h)', '(i)' ]

#suptitle = r'C$_{\mathrm{anth}}$ storage, concentration, and layer thickness ' + r'$\sigma_0$ = ' + str(dens1) + '-' + str(dens2) + ' (ave. from '+str(ystr)+' to '+str(yend)+')'
outfile = './fig/Thick_Cant_int_' + str(int(dens1*100)) + '-' + str(int(dens2*100)) + '_v4'

print("Drawing "+outfile)

####################################################################
path_glb100km='../data/glb100km/'
path_np10km='../data/np10km/'
####################################################################

time = np.empty(yend-ystr+1,dtype='object')
for yr in range(ystr,yend+1):
    time[yr-ystr] = datetime.datetime(yr,1,1)

####################################################################
# GLB100km

print( "Loading GLB100km data" )

path_g100 = path_glb100km + 'run-20211104o2/ANALY/monthly'
base_dic_g100 = 'hs_DIC_sig_woa1x1'

suffix = '.' + '{:04}'.format(yr)
dic_file = path_g100 + '/' + base_dic_g100 + suffix
ncg100 = netCDF4.Dataset(dic_file,'r')
lon_mod = ncg100.variables['lon'][:]
lat_mod = ncg100.variables['lat'][:]
dep_mod = ncg100.variables['depth'][:]
nx = len(ncg100.dimensions['lon'])
ny = len(ncg100.dimensions['lat'])
nz = len(ncg100.dimensions['depth'])
ncg100.close()

####################################################################

kz1=0
sigdist=abs(dep_mod[kz1]-dens1)
for k in range(1,nz):
   if (abs(dep_mod[k]-dens1) < sigdist):
       kz1=k
       sigdist = abs(dep_mod[k]-dens1)

dens1t = 0.5 * (dep_mod[kz1-1]+dep_mod[kz1])

kz2=0
sigdist=abs(dep_mod[kz2]-dens2)
for k in range(1,nz):
   if (abs(dep_mod[k]-dens2) < sigdist):
       kz2=k
       sigdist = abs(dep_mod[k]-dens2)

dens2b = 0.5 * (dep_mod[kz2]+dep_mod[kz2+1])

print("bounded by ", dep_mod[kz1], dep_mod[kz2])

kzm=0
sigdist=abs(dep_mod[kzm]-densm)
for k in range(1,nz):
   if (abs(dep_mod[k]-densm) < sigdist):
       kzm=k
       sigdist = abs(dep_mod[k]-densm)

print("central density at ", dep_mod[kzm])

####################################################################

dic_sig_tmp = np.array(np.empty((12,nz,ny,nx)),dtype=np.float64)
dic_prei_sig_tmp = np.array(np.empty((12,nz,ny,nx)),dtype=np.float64)
thkh_sig_tmp = np.array(np.empty((12,nz,ny,nx)),dtype=np.float64)
cant_sig_tmp = np.array(np.empty((12,nz,ny,nx)),dtype=np.float64)
mont_sig_tmp = np.array(np.empty((12,nz,ny,nx)),dtype=np.float64)

thkh_sig_g100 = np.array(np.empty((nyr,nz,ny,nx)),dtype=np.float64)
cant_stor_sig_g100 = np.array(np.empty((nyr,nz,ny,nx)),dtype=np.float64)
cant_conc_sig_g100 = np.array(np.empty((nyr,nz,ny,nx)),dtype=np.float64)
mont_sig_g100 = np.array(np.empty((nyr,nz,ny,nx)),dtype=np.float64)

base_dic_g100 = 'hs_DIC_sig_woa1x1'
base_dic_prei_g100 = 'hs_DIC_prei_sig_woa1x1'
base_thkh_g100 = 'hs_thkh_woa1x1'
base_mont_g100 = 'hs_montgomery_pot_woa1x1'

for yr in range(ystr,yend+1):

    suffix = '.' + '{:04}'.format(yr)

    dic_sig_file = path_g100 + '/' + base_dic_g100 + suffix
    ncg100 = netCDF4.Dataset(dic_sig_file,'r')
    dic_sig_tmp[:,:,:,:] = ncg100.variables['dissic'][:,:,:,:]
    ncg100.close()

    dic_prei_sig_file = path_g100 + '/' + base_dic_prei_g100 + suffix
    ncg100 = netCDF4.Dataset(dic_prei_sig_file,'r')
    dic_prei_sig_tmp[:,:,:,:] = ncg100.variables['dissicnat'][:,:,:,:]
    ncg100.close()

    dic_sig_tmp = np.where(dic_sig_tmp < -9.0e33, np.NaN, dic_sig_tmp)
    dic_prei_sig_tmp = np.where(dic_prei_sig_tmp < -9.0e33, np.NaN, dic_prei_sig_tmp)

    thkh_sig_file = path_g100 + '/' + base_thkh_g100 + suffix
    ncg100 = netCDF4.Dataset(thkh_sig_file,'r')
    thkh_sig_tmp[:,:,:,:] = ncg100.variables['thickness'][:,:,:,:]
    ncg100.close()

    thkh_sig_tmp = np.where(thkh_sig_tmp < -9.0e33, np.NaN, thkh_sig_tmp)
    thkh_sig_g100[yr-ystr,:,:,:] = np.nanmean(thkh_sig_tmp,axis=0)

    cant_conc_sig_g100[yr-ystr,:,:,:] = (np.nanmean(dic_sig_tmp,axis=0) - np.nanmean(dic_prei_sig_tmp,axis=0)) * 1e3 # mol/m3 -> mmol/m3
    #cant_conc_sig_g100[yr-ystr,:,:,:] = (np.nanmean(cant_conc_sig_tmp,axis=0))
    cant_stor_sig_tmp = (dic_sig_tmp - dic_prei_sig_tmp) * thkh_sig_tmp

    cant_stor_sig_g100[yr-ystr,:,:,:] = (np.nanmean(cant_stor_sig_tmp,axis=0))

    mont_sig_file = path_g100 + '/' + base_mont_g100 + suffix
    ncg100 = netCDF4.Dataset(mont_sig_file,'r')
    mont_sig_tmp[:,:,:,:] = ncg100.variables['mont'][:,:,:,:]
    ncg100.close()

    mont_sig_tmp = np.where(mont_sig_tmp < -9.0e33, np.NaN, mont_sig_tmp)
    mont_sig_g100[yr-ystr,:,:,:] = np.nanmean(mont_sig_tmp,axis=0)


DS_g100_ann = xr.Dataset( {'cant_conc_sig': (['time','depth','lat','lon'], cant_conc_sig_g100),
                           'cant_stor_sig': (['time','depth','lat','lon'], cant_stor_sig_g100),
                           'mont_sig': (['time','depth','lat','lon'], mont_sig_g100),
                           'thkh_sig': (['time','depth','lat','lon'], thkh_sig_g100),},
                          coords = { 'time': time,'depth': dep_mod,'lat': lat_mod,'lon': lon_mod, } )

####################################################################
# NP10km

print( "Loading NP10km data" )

path_fill=path_np10km+'hst_mon-glb_fill'

thkh_sig_fill = np.array(np.empty((nyr,nz,ny,nx)),dtype=np.float64)
cant_conc_sig_fill = np.array(np.empty((nyr,nz,ny,nx)),dtype=np.float64)
cant_stor_sig_fill = np.array(np.empty((nyr,nz,ny,nx)),dtype=np.float64)
mont_sig_fill = np.array(np.empty((nyr,nz,ny,nx)),dtype=np.float64)

base_dic_fill = 'hs_DIC_sig_woa1x1'
base_dic_prei_fill = 'hs_DIC_prei_sig_woa1x1'
base_thkh_fill = 'hs_thkh_woa1x1'
base_mont_fill = 'hs_montgomery_pot_woa1x1'

for yr in range(ystr,yend+1):
    for mn in range(1,13):

        suffix = '.' + '{:04}{:02}'.format(yr,mn)

        dic_sig_file = path_fill + '/y' + '{:04}'.format(yr) + '/' + base_dic_fill + suffix
        ncfill = netCDF4.Dataset(dic_sig_file,'r')
        dic_sig_tmp[mn-1,:,:,:] = ncfill.variables['dissic'][0,:,:,:]
        ncfill.close()

        dic_prei_sig_file = path_fill + '/y' + '{:04}'.format(yr) + '/' + base_dic_prei_fill + suffix
        ncfill = netCDF4.Dataset(dic_prei_sig_file,'r')
        dic_prei_sig_tmp[mn-1,:,:,:] = ncfill.variables['dissicnat'][0,:,:,:]
        ncfill.close()

        thkh_sig_file = path_fill + '/y' + '{:04}'.format(yr) + '/' + base_thkh_fill + suffix
        ncfill = netCDF4.Dataset(thkh_sig_file,'r')
        thkh_sig_tmp[mn-1,:,:,:] = ncfill.variables['thickness'][0,:,:,:]
        ncfill.close()

        mont_sig_file = path_fill + '/y' + '{:04}'.format(yr) + '/' + base_mont_fill + suffix
        ncfill = netCDF4.Dataset(mont_sig_file,'r')
        mont_sig_tmp[mn-1,:,:,:] = ncfill.variables['mont'][0,:,:,:]
        ncfill.close()

    dic_sig_tmp = np.where(dic_sig_tmp < -9.0e33, np.NaN, dic_sig_tmp)
    dic_prei_sig_tmp = np.where(dic_prei_sig_tmp < -9.0e33, np.NaN, dic_prei_sig_tmp)
    thkh_sig_tmp = np.where(thkh_sig_tmp < -9.0e33, np.NaN, thkh_sig_tmp)
    thkh_sig_fill[yr-ystr,:,:,:] = np.nanmean(thkh_sig_tmp,axis=0)

    cant_conc_sig_fill[yr-ystr,:,:,:] = (np.nanmean(dic_sig_tmp,axis=0) - np.nanmean(dic_prei_sig_tmp,axis=0)) * 1e3 # mol/m3 -> mmol/m3
    cant_stor_sig_tmp = (dic_sig_tmp - dic_prei_sig_tmp) * thkh_sig_tmp

    #cant_conc_sig_fill[yr-ystr,:,:,:] = np.nanmean(cant_conc_sig_tmp,axis=0)
    cant_stor_sig_fill[yr-ystr,:,:,:] = np.nanmean(cant_stor_sig_tmp,axis=0)

    mont_sig_tmp = np.where(mont_sig_tmp < -9.0e33, np.NaN, mont_sig_tmp)
    mont_sig_fill[yr-ystr,:,:,:] = np.nanmean(mont_sig_tmp,axis=0)

DS_fill_ann = xr.Dataset( {'cant_conc_sig': (['time','depth','lat','lon'], cant_conc_sig_fill),
                           'cant_stor_sig': (['time','depth','lat','lon'], cant_stor_sig_fill),
                           'mont_sig': (['time','depth','lat','lon'], mont_sig_fill),
                           'thkh_sig': (['time','depth','lat','lon'], thkh_sig_fill),},
                          coords = { 'time': time,'depth': dep_mod,'lat': lat_mod,'lon': lon_mod, } )

#print(DS_fill_ann)

####################################################################

da1 = DS_g100_ann.cant_stor_sig.isel(depth=(slice(kz1,kz2))).sum(dim='depth',skipna=True,min_count=1).sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)
da5 = DS_g100_ann.thkh_sig.isel(depth=(slice(kz1,kz2))).sum(dim='depth',skipna=True,min_count=1).sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)

da2 = DS_fill_ann.cant_stor_sig.isel(depth=(slice(kz1,kz2))).sum(dim='depth',skipna=True,min_count=1).sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)
da6 = DS_fill_ann.thkh_sig.isel(depth=(slice(kz1,kz2))).sum(dim='depth',skipna=True,min_count=1).sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)

print(da5.values)

DS_stor = xr.Dataset( {'GLB100km': (['lat','lon'], da1.values),
                       'NP10km': (['lat','lon'], da2.values),
                       'NP10km-GLB100km': (['lat','lon'], da2.values-da1.values), },
                      coords = { 'lat': np.linspace(-89.5,89.5,num=180), 
                                 'lon': np.linspace(0.5,359.5,num=360), } )

DS_thck = xr.Dataset( {'GLB100km': (['lat','lon'], da5.values),
                       'NP10km': (['lat','lon'], da6.values),
                       'NP10km-GLB100km': (['lat','lon'], da6.values-da5.values), },
                      coords = { 'lat': np.linspace(-89.5,89.5,num=180), 
                                 'lon': np.linspace(0.5,359.5,num=360), } )

da3 = DS_g100_ann.mont_sig.isel(depth=kzm).sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)
da4 = DS_fill_ann.mont_sig.isel(depth=kzm).sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)
da7 = DS_g100_ann.cant_conc_sig.isel(depth=kzm).sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)
da8 = DS_fill_ann.cant_conc_sig.isel(depth=kzm).sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)

DS_mont = xr.Dataset( {'GLB100km': (['lat','lon'], da3.values),
                       'NP10km': (['lat','lon'], da4.values),
                       'NP10km-GLB100km': (['lat','lon'], da4.values - da3.values), },
                      coords = { 'lat': np.linspace(-89.5,89.5,num=180), 
                                 'lon': np.linspace(0.5,359.5,num=360), } )

DS_conc = xr.Dataset( {'GLB100km': (['lat','lon'], da7.values),
                       'NP10km': (['lat','lon'], da8.values),
                       'NP10km-GLB100km': (['lat','lon'], da8.values-da7.values), },
                      coords = { 'lat': np.linspace(-89.5,89.5,num=180), 
                                 'lon': np.linspace(0.5,359.5,num=360), } )

######

print( 'Start drawing' )
fig = plt.figure(figsize=(11,5.0))
#fig.suptitle( suptitle, fontsize=16 )

proj = ccrs.PlateCarree(central_longitude=-140.)
lon_formatter = LongitudeFormatter(zero_direction_label=True)
lat_formatter = LatitudeFormatter()
# [left, bottom, width, height]

ax = [
    plt.axes([0.06,0.545,0.275,0.38],projection=proj),
    plt.axes([0.38,0.545,0.275,0.38],projection=proj),
    plt.axes([0.70,0.545,0.275,0.38],projection=proj),
    plt.axes([0.06,0.09,0.275,0.38],projection=proj),
    plt.axes([0.38,0.09,0.275,0.38],projection=proj),
    plt.axes([0.70,0.09,0.275,0.38],projection=proj),
]

# [left, bottom, width, height]

ax_cbar = [
    plt.axes([0.05,0.520,0.30,0.012]),
    plt.axes([0.37,0.520,0.30,0.012]),
    plt.axes([0.69,0.520,0.30,0.012]),
    plt.axes([0.05,0.065,0.30,0.012]),
    plt.axes([0.37,0.065,0.30,0.012]),
    plt.axes([0.69,0.065,0.30,0.012]),
]

plt.text(0.21,0.92,'Low Resolution',ha='center',size=16,transform=fig.transFigure)
plt.text(0.53,0.92,'High Resolution',ha='center',size=16,transform=fig.transFigure)
plt.text(0.85,0.92,'HighRes - LowRes',ha='center',size=16,transform=fig.transFigure)

suptitle = r'$\sigma_0$ = ' + '{:.1f}'.format(dens1t) + '-' + '{:.1f}'.format(dens2b) + ' ('+str(ystr)+'-'+str(yend)+' mean)'
plt.text(0.53,0.965,suptitle,ha='center',size=16,transform=fig.transFigure)

# 25.0-25.5

# Montgomery potential

if (23.0 < densm and densm < 25.0):
    bounds0 = np.arange(-10000,10000.1,1000) # 23.5
    bounds1 = [0.0, 0.2, 0.5, 1.0, 1.5, 2.0, 3.0, 5.0, 7.0, 10.0]
    bounds2 = [-2.0, -1.0, -0.5, -0.2, -0.1, -0.05, 0.05, 0.1, 0.2, 0.5, 1.0, 2.0]
    bounds3 = np.arange(40,70.1,2) # 23.5
# thickness 
    bounds5 = [0.0, 5.0, 10.0, 20.0, 30.0, 40.0, 50.0, 70.0, 100.0, 150.0, 200.0]
    bounds6 = [-50.0, -30.0, -20.0, -10.0, -7.0, -4.0, -2.0, 2.0, 4.0, 7.0, 10.0, 20.0, 30.0, 50.0]
elif (25.0 < densm and densm < 25.5):
    bounds0 = np.arange(-10000,10000.1,1000) # 25.25
    bounds1 = [0.0, 0.2, 0.5, 1.0, 1.5, 2.0, 3.0, 5.0, 7.0, 10.0]
    #bounds2 = [-2.0, -1.0, -0.5, -0.2, -0.1, -0.05, 0.05, 0.1, 0.2, 0.5, 1.0, 2.0]
    bounds2 = [-3.0, -2.0, -1.0, -0.5, -0.2, -0.1, 0.1, 0.2, 0.5, 1.0, 2.0, 3.0]
    bounds3 = np.arange(30,60.1,2) # 25.25
    #bounds5 = [0.0, 5.0, 10.0, 20.0, 30.0, 40.0, 50.0, 70.0, 100.0, 150.0, 200.0]
    bounds5 = [0.0, 10.0, 20.0, 30.0, 40.0, 50.0, 70.0, 100.0, 150.0]
    bounds6 = [-70.0, -50.0, -30.0, -20.0, -10.0, -5.0, 5.0, 10.0, 20.0, 30.0, 50.0, 70.0]
elif (25.5 < densm and densm < 26.0):
    bounds0 = np.arange(-5000,10000.1,500) # 25.75
    bounds1 = [0.0, 0.2, 0.5, 1.0, 1.5, 2.0, 3.0, 5.0, 7.0, 10.0]
    bounds2 = [-2.0, -1.0, -0.5, -0.2, -0.1, -0.05, 0.05, 0.1, 0.2, 0.5, 1.0, 2.0]
    bounds3 = np.arange(0,60.1,5) # 26.25
    bounds5 = [0.0, 5.0, 10.0, 20.0, 30.0, 40.0, 50.0, 70.0, 100.0, 150.0, 200.0]
    bounds6 = [-50.0, -30.0, -20.0, -10.0, -7.0, -4.0, -2.0, 2.0, 4.0, 7.0, 10.0, 20.0, 30.0, 50.0]
elif (26.0 < densm and densm < 26.5):
    bounds0 = np.arange(-5000,5000.1,500) # 26.25
    bounds1 = [0.0, 0.5, 1.0, 1.5, 2.0, 3.0, 5.0, 7.0, 10.0, 15.0]
    #bounds2 = [-2.0, -1.0, -0.5, -0.2, -0.1, -0.05, 0.05, 0.1, 0.2, 0.5, 1.0, 2.0]
    bounds2 = [-3.0, -2.0, -1.0, -0.5, -0.2, -0.1, 0.1, 0.2, 0.5, 1.0, 2.0, 3.0]
    bounds3 = np.arange(0,60.1,5) # 26.25
#    bounds5 = [0.0, 10.0, 20.0, 30.0, 40.0, 50.0, 70.0, 100.0, 150.0, 200.0, 250.0]
    bounds5 = [0.0, 10.0, 20.0, 30.0, 50.0, 70.0, 100.0, 150.0, 200.0, 250.0]
#    bounds6 = [-50.0, -30.0, -20.0, -10.0, -7.0, -4.0, -2.0, 2.0, 4.0, 7.0, 10.0, 20.0, 30.0, 50.0]
    bounds6 = [-70.0, -50.0, -30.0, -20.0, -10.0, -5.0, 5.0, 10.0, 20.0, 30.0, 50.0, 70.0]
elif (26.5 < densm):
    bounds0 = np.arange(-10000,5000.1,500) # 26.75
    bounds1 = [0.0, 0.2, 0.5, 1.0, 1.5, 2.0, 3.0, 5.0, 7.0, 10.0]
    bounds2 = [-2.0, -1.0, -0.5, -0.2, -0.1, -0.05, 0.05, 0.1, 0.2, 0.5, 1.0, 2.0]
    bounds3 = np.arange(0,60.1,5) # 26.25
    bounds5 = [0.0, 50.0, 100.0, 150.0, 200.0, 250.0, 300.0, 350.0, 400.0]
    bounds6 = [-100.0, -70.0, -50.0, -30.0, -20.0, -10.0, -5.0, 5.0, 10.0, 20.0, 30.0, 50.0, 70.0, 100.0]

# storage
# concentration
#bounds3 = np.arange(0,80.1,5)
tbounds3 = bounds3
bounds4 = [-20.0, -10.0, -7.0, -4.0, -2.0, -1.0, 1.0, 2.0, 4.0, 7.0, 10.0, 20.0]


cmap = [[ 'YlOrRd', 'YlOrRd', 'bwr' ],
        [ 'terrain', 'terrain', 'bwr' ]]

mod = [ 'GLB100km', 'NP10km', 'NP10km-GLB100km' ]

bar = [ 'max', 'max', 'both' ]

panel=0

for item in range(2):
    for model in range(3):

        title = fig_title[panel]+' '+item_title[item]+' '+model_title[model]

        print("Drawing panel "+ str(panel) + ' ' + title)

        if (item == 0):

            units = '[$\mathrm{mol}\, \mathrm{m}^{-2}$]'
            if (mod[model] == 'GLB100km'):
                bounds = bounds1
                ticks_bounds = bounds1
                da = DS_stor[mod[model]]
            elif (mod[panel] == 'NP10km'):
                bounds = bounds1
                ticks_bounds = bounds1
                da = DS_stor[mod[model]]
            elif mod[panel] == 'NP10km-GLB100km':
                bounds = bounds2
                ticks_bounds = bounds2
                da = DS_stor[mod[model]]

        if (item == 1):

            units = '[$\mathrm{m}$]'
            if (mod[model] == 'GLB100km'):
                bounds = bounds5
                ticks_bounds = bounds5
                da = DS_thck[mod[model]]
            elif (mod[model] == 'NP10km'):
                bounds = bounds5
                ticks_bounds = bounds5
                da = DS_thck[mod[model]]
            elif mod[model] == 'NP10km-GLB100km':
                bounds = bounds6
                ticks_bounds = bounds6
                da = DS_thck[mod[model]]


        im = da.plot(ax=ax[panel],cmap=cmap[item][model],
                     levels=bounds,
                     extend=bar[model],
                     add_colorbar=False,
                     transform=ccrs.PlateCarree())

#                cbar_kwargs={'orientation': 'horizontal',
#                             'spacing':'uniform',
#                             'shrink':1.0,
#                             'label': units,
#                             'ticks': ticks_bounds,},

        if (item == 0 and model < 2):
            DS_mont[mod[model]].plot.contour(ax=ax[panel],
                                              levels=bounds0,
                                              colors='black',add_colorbar=False,
                                              linewidths=0.5,
                                              transform=ccrs.PlateCarree())

        cbar = plt.colorbar(im,orientation='horizontal',spacing='uniform',
                            ticks=ticks_bounds, cax=ax_cbar[panel])
        cbar.set_label(label=units,fontsize=7,x=0.90,labelpad=0)
        cbar.ax.tick_params(labelsize=7)

        ax[panel].coastlines()
        ax[panel].set_xticks(np.arange(120,250.1,20),crs=ccrs.PlateCarree())
        ax[panel].set_yticks(np.arange(10,60.1,5),crs=ccrs.PlateCarree())
        ax[panel].set_extent([114,251,9,61],crs=ccrs.PlateCarree())
        #ax[panel].set_xticks(np.arange(100,285.1,20),crs=ccrs.PlateCarree())
        #ax[panel].set_yticks(np.arange(-15,65.1,5),crs=ccrs.PlateCarree())
        #ax[panel].set_extent([100,285,-15,65],crs=ccrs.PlateCarree())
        ax[panel].set_aspect(1.25)
        #ax[panel].set_xticks(np.arange(-180,180.1,60),crs=ccrs.PlateCarree())
        #ax[panel].set_yticks(np.arange(-90,90.1,30),crs=ccrs.PlateCarree())
        ax[panel].xaxis.set_major_formatter(lon_formatter)
        ax[panel].yaxis.set_major_formatter(lat_formatter)
        ax[panel].set_xlabel('')
        ax[panel].set_ylabel('')
        ax[panel].set_title(title,{'fontsize':10, 'verticalalignment':'top'})
        ax[panel].tick_params(labelsize=9)
        ax[panel].patch.set_facecolor('lightgray')

        panel+=1

plt.subplots_adjust(left=0.07,right=0.98,bottom=0.05,top=0.92,wspace=0.16,hspace=0.15)

outpdf = outfile+'.pdf'
outpng = outfile+'.png'

plt.savefig(outpng, bbox_inches='tight', pad_inches=0.1)
plt.savefig(outpdf, bbox_inches='tight', pad_inches=0.1)

print("figure is saved to " + outpng + " and " + outpdf)

plt.show()
