# -*- coding: utf-8 -*-
import sys
import json
import numpy as np
import netCDF4
import xarray as xr
import cartopy.crs as ccrs
import matplotlib as mpl
import matplotlib.pyplot as plt
from cartopy.mpl.ticker import LongitudeFormatter, LatitudeFormatter
import datetime

########
#ro = 1036.0
ro_scl = 1024.5

if (len(sys.argv) < 2):
    print ('Usage: ' + sys.argv[0] + ' exp1 exp2')
    sys.exit()

exp1=sys.argv[1]
exp2=sys.argv[2]

ystr_anth = 1987
yend_anth = 2017

nyr_anth = yend_anth - ystr_anth + 1

title_top = [[ r"(a) C$_{\mathrm{anth}}$ column inventory " + exp1, r"(b) C$_{\mathrm{anth}}$ column inventory " + exp2, "(c) "+exp2+'-'+exp1, "(d) GLODAPv2"],
             [ r"(e) C$_{\mathrm{anth}}$ storage rate " + exp1, r"(f) C$_{\mathrm{anth}}$ storage rate " + exp2,"(g) "+ exp2+'-'+exp1, "(h) Gruber et al. (2019)"],
             [ "(i) Surface uptake rate " + exp1, "(j) Surface uptake rate " + exp2, "(k) "+exp2+'-'+exp1, "(l) Zonal integration"]]

#suptitle = 'Cumulative anthropogenic CO2 inventory (ave. from '+str(ystr)+' to '+str(yend)+')'
outfile = './fig/Cant_comp_all'

####################################################################

timey = np.empty(yend_anth-ystr_anth+1,dtype='object')
for yr in range(ystr_anth,yend_anth+1):
    timey[yr-ystr_anth] = datetime.datetime(yr,1,1)

####################################################################

time_snp = np.empty(2,dtype='object')

nyr=0
for yr in [ystr_anth,yend_anth+1]:
    time_snp[nyr] = datetime.datetime(yr,1,1)
    nyr+=1

####################################################################
path_glb100km='../data/glb100km/'
path_np10km='../data/np10km/'
path_ref='../data/refdata/'
####################################################################
# Inventory
#   Ref data

print( "Loading GLODAPv2 data" )

path_glodap= path_ref + 'GLODAPv2/'
reffile = path_glodap + '/GLODAPv2.2016b.Cant.nc'
ncref = netCDF4.Dataset(reffile,'r')
lon = ncref.variables['lon'][:]
lat = ncref.variables['lat'][:]
depth = ncref.variables['Depth'][:]
nx = len(ncref.dimensions['lon'])
ny = len(ncref.dimensions['lat'])
nz = len(ncref.dimensions['depth_surface'])
cant_3d = ncref.variables['Cant'][:,:,:]
ncref.close()

cant_3d = np.where(cant_3d < -990, np.NaN, cant_3d)
print("GLODAPv2 depth ", depth[0:nz])

#path_glodap_gr= "/storage1/C401/htsujino/GLODAPv2/2016b_surakawa/grads"
#sig0 = np.array(np.empty((nz,ny,nx)),dtype=np.float64)
#
#sig0_file = path_glodap_gr + "/GLODAPv2.2016b.sigma0.gd"
#fl = open(sig0_file,'rb')
#for k in range(nz):
#    for j in range(ny):
#        sig0[k,j,:] = np.fromfile(fl, dtype = '>f', count= nx)
#
#sig0 = np.where(sig0 < -9.0e33, np.NaN, sig0 + 1.e3)
#sig0 = np.roll(sig0,-20,axis=2).copy()
#cant_3d = np.where(cant_3d < -990, np.NaN, cant_3d * sig0 * 1.e-6) # mol / m3

cant_3d = np.where(cant_3d < -990, np.NaN, cant_3d * ro_scl * 1.e-6) # mol / m3
cant_ref = np.array(np.zeros((ny,nx)),dtype=np.float64)

for j in range(ny):
    for i in range(nx):
        for k in range(nz-1):
            if (not np.isnan(cant_3d[k,j,i])) and (not np.isnan(cant_3d[k+1,j,i])):
                cant_ref[j,i] = cant_ref[j,i] \
                    + 0.5 * (cant_3d[k,j,i]+cant_3d[k+1,j,i]) * (depth[k+1]-depth[k])

cant_ref = np.where(cant_ref == 0.0, np.NaN, cant_ref)

DS_ref = xr.Dataset( {'cant_vint': (['lat','lon'], cant_ref),},
                      coords = { 'lat': lat,'lon': lon, } )
DS_ref = DS_ref.roll(lon=20, roll_coords=True)

#print(DS_ref)

####################################################################
print( "Loading Gruber (2019) data" )

path_gruber= path_ref + 'Gruber_2019/'
reffile = path_gruber + 'inv_dcant_emlr_cstar_gruber_94-07_vs1.nc'
ncref = netCDF4.Dataset(reffile,'r')
longr = ncref.variables['LONGITUDE'][:]
latgr = ncref.variables['LATITUDE'][:]
dcant_int = ncref.variables['DCANT_INV01'][:,:]
ncref.close()

dcant_int = np.where(dcant_int < -9.e33, np.NaN, dcant_int)
dcant_int = dcant_int/(2007-1994)
DS_Gruber = xr.Dataset( {'dcant_int': (['lat','lon'], dcant_int),},
                        coords = { 'lat': latgr,'lon': longr, } )

####################################################################
dic_vint_tmp = np.array(np.empty((ny,nx)),dtype=np.float64)
dic_prei_vint_tmp = np.array(np.empty((ny,nx)),dtype=np.float64)
####################################################################
# GLB100km

print( "Loading GLB100km data" )

path_g100 = path_glb100km + 'run-20211104o2/ANALY/annual'

cant_vint_g100 = np.array(np.empty((nyr_anth,ny,nx)),dtype=np.float64)

base_dic_g100 = 'hs_DIC_vint_woa1x1'
base_dic_prei_g100 = 'hs_DIC_prei_vint_woa1x1'

for yr in range(ystr_anth,yend_anth+1):

    suffix = '.' + '{:04}'.format(yr)

    dic_vint_file = path_g100 + '/' + base_dic_g100 + suffix
    ncg100 = netCDF4.Dataset(dic_vint_file,'r')
    dic_vint_tmp[:,:] = ncg100.variables['dissicint'][0,:,:]
    ncg100.close()

    dic_prei_vint_file = path_g100 + '/' + base_dic_prei_g100 + suffix
    ncg100 = netCDF4.Dataset(dic_prei_vint_file,'r')
    dic_prei_vint_tmp[:,:] = ncg100.variables['dissicnatint'][0,:,:]
    ncg100.close()

    dic_vint_tmp = np.where(dic_vint_tmp < -9.0e33, np.NaN, dic_vint_tmp)
    dic_prei_vint_tmp = np.where(dic_prei_vint_tmp < -9.0e33, np.NaN, dic_prei_vint_tmp)

    cant_vint_g100[yr-ystr_anth,:,:] = (dic_vint_tmp[:,:] - dic_prei_vint_tmp[:,:])

DS_g100_ann = xr.Dataset( {'cant_vint': (['time','lat','lon'], cant_vint_g100),},
                          coords = { 'time': timey,'lat': lat,'lon': lon, } )

####################################################################
# NP10km

print( "Loading NP10km data" )

path_fill = path_np10km + 'hst_ann-glb_fill/'
cant_vint_fill = np.array(np.empty((nyr_anth,ny,nx)),dtype=np.float64)

base_dic_fill = 'hs_DIC_vint_woa1x1'
base_dic_prei_fill = 'hs_DIC_prei_vint_woa1x1'

for yr in range(ystr_anth,yend_anth+1):

    suffix = '.' + '{:04}'.format(yr)

    dic_vint_file = path_fill + '/y' + '{:04}'.format(yr) + '/' + base_dic_fill + suffix
    ncfill = netCDF4.Dataset(dic_vint_file,'r')
    dic_vint_tmp[:,:] = ncfill.variables['dissicint'][0,:,:]
    ncfill.close()

    dic_prei_vint_file = path_fill + '/y' + '{:04}'.format(yr) + '/' + base_dic_prei_fill + suffix
    ncfill = netCDF4.Dataset(dic_prei_vint_file,'r')
    dic_prei_vint_tmp[:,:] = ncfill.variables['dissicnatint'][0,:,:]
    ncfill.close()

    dic_vint_tmp = np.where(dic_vint_tmp < -9.0e33, np.NaN, dic_vint_tmp)
    dic_prei_vint_tmp = np.where(dic_prei_vint_tmp < -9.0e33, np.NaN, dic_prei_vint_tmp)

    cant_vint_fill[yr-ystr_anth,:,:] = (dic_vint_tmp[:,:] - dic_prei_vint_tmp[:,:])*1.0e-2

DS_fill_ann = xr.Dataset( {'cant_vint': (['time','lat','lon'], cant_vint_fill),},
                          coords = { 'time': timey,'lat': lat,'lon': lon, } )

#print(DS_fill_ann)

####################################################################

da0 = DS_ref.cant_vint
da1 = DS_g100_ann.cant_vint.sel(time=slice(str(ystr_anth),str(yend_anth))).mean(dim='time',skipna=False)
da2 = DS_fill_ann.cant_vint.sel(time=slice(str(ystr_anth),str(yend_anth))).mean(dim='time',skipna=False)

DS_anth = xr.Dataset( {'exp1': (['lat','lon'], da1.values),
                        'exp2': (['lat','lon'], da2.values),
                        'exp2-1': (['lat','lon'], da2.values - da1.values),
                        'obs':   (['lat','lon'], da0.values), },
                       coords = { 'lat': np.linspace(-89.5,89.5,num=180), 
                                  'lon': np.linspace(0.5,359.5,num=360), } )

####################################################################
# Inventory rate
#   GLB100km

print( "Loading GLB100km data" )

path_g100 = path_glb100km + 'run-20211104o2/ANALY/snap'

cant_vint_g100 = np.array(np.empty((2,ny,nx)),dtype=np.float64)

base_dic_g100 = 'rs_DIC_vint_woa1x1'
base_dic_prei_g100 = 'rs_DIC_prei_vint_woa1x1'

nyr=0
for yr in [ystr_anth,yend_anth+1]:

    suffix = '.' + '{:04}0101000000'.format(yr)

    dic_vint_file = path_g100 + '/y' + '{:04}'.format(yr) + '/' + base_dic_g100 + suffix
    ncg100 = netCDF4.Dataset(dic_vint_file,'r')
    dic_vint_tmp[:,:] = ncg100.variables['dissicint'][0,:,:]
    ncg100.close()

    dic_prei_vint_file = path_g100 + '/y' + '{:04}'.format(yr) + '/' + base_dic_prei_g100 + suffix
    ncg100 = netCDF4.Dataset(dic_prei_vint_file,'r')
    dic_prei_vint_tmp[:,:] = ncg100.variables['dissicnatint'][0,:,:]
    ncg100.close()

    dic_vint_tmp = np.where(dic_vint_tmp < -9.0e33, np.NaN, dic_vint_tmp)
    dic_prei_vint_tmp = np.where(dic_prei_vint_tmp < -9.0e33, np.NaN, dic_prei_vint_tmp)

    cant_vint_g100[nyr,:,:] = (dic_vint_tmp[:,:] - dic_prei_vint_tmp[:,:])*1.0e-2
    nyr+=1

DS_g100_snp = xr.Dataset( {'cant_vint': (['time','lat','lon'], cant_vint_g100),},
                          coords = { 'time': time_snp,
                                     'lat': np.linspace(-89.5,89.5,num=180), 
                                     'lon': np.linspace(0.5,359.5,num=360), } )

####################################################################
# NP10km

print( "Loading NP10km data" )

path_fill = path_np10km + 'rst_snp-glb_fill'

cant_vint_fill = np.array(np.empty((nyr,ny,nx)),dtype=np.float64)

base_dic_fill = 'rs_DIC_vint_woa1x1'
base_dic_prei_fill = 'rs_DIC_prei_vint_woa1x1'

nyr=0
for yr in [ystr_anth,yend_anth+1]:

    suffix = '.' + '{:04}0101000000'.format(yr)

    dic_vint_file = path_fill + '/y' + '{:04}'.format(yr) + '/' + base_dic_fill + suffix
    ncfill = netCDF4.Dataset(dic_vint_file,'r')
    dic_vint_tmp[:,:] = ncfill.variables['dissicint'][0,:,:]
    ncfill.close()

    dic_prei_vint_file = path_fill + '/y' + '{:04}'.format(yr) + '/' + base_dic_prei_fill + suffix
    ncfill = netCDF4.Dataset(dic_prei_vint_file,'r')
    dic_prei_vint_tmp[:,:] = ncfill.variables['dissicnatint'][0,:,:]
    ncfill.close()

    dic_vint_tmp = np.where(dic_vint_tmp < -9.0e33, np.NaN, dic_vint_tmp)
    dic_prei_vint_tmp = np.where(dic_prei_vint_tmp < -9.0e33, np.NaN, dic_prei_vint_tmp)

    cant_vint_fill[nyr,:,:] = (dic_vint_tmp[:,:] - dic_prei_vint_tmp[:,:])*1.0e-2
    nyr+=1

DS_fill_snp = xr.Dataset( {'cant_vint': (['time','lat','lon'], cant_vint_fill),},
                          coords = { 'time': time_snp,
                                     'lat': np.linspace(-89.5,89.5,num=180), 
                                     'lon': np.linspace(0.5,359.5,num=360), } )

####################################################################

print(DS_g100_snp)
print(DS_fill_snp)

da0 = DS_Gruber.dcant_int

da1 = (DS_g100_snp.cant_vint.sel(time=str(yend_anth+1)).values - DS_g100_snp.cant_vint.sel(time=str(ystr_anth)).values) / (yend_anth-ystr_anth+1)
print(da1.shape)

da2 = (DS_fill_snp.cant_vint.sel(time=str(yend_anth+1)).values - DS_fill_ann.cant_vint.sel(time=str(ystr_anth)).values) / (yend_anth-ystr_anth+1)

DS_rate = xr.Dataset( {'exp1': (['lat','lon'], da1[0]),
                       'exp2': (['lat','lon'], da2[0]),
                       'exp2-1': (['lat','lon'], da2[0]-da1[0]),
                       'obs': (['lat','lon'], da0.values), },
                      coords = { 'lat': np.linspace(-89.5,89.5,num=180), 
                                 'lon': np.linspace(0.5,359.5,num=360), } )

####################################################################
# GLB100km

print( "Loading GLB100km data" )

path_g100 = path_glb100km + 'run-20211104o2/ANALY/annual'
#path_g100='/NasB2019_1/hotspot2_a0206/htsujino/OMIP2-like-BGC/GONDOLA_100_BGC/exp/run-20180525j/result_org/ANALY/annual'


fgco2_g100 = np.array(np.empty((nyr_anth,ny,nx)),dtype=np.float64)
fgco2_prei_g100 = np.array(np.empty((nyr_anth,ny,nx)),dtype=np.float64)

fgdic_base_g100 = 'hs_FgDIC_woa1x1'
fgdic_base_prei_g100 = 'hs_FgDIC_prei_woa1x1'

fgco2_mod = np.array(np.empty((ny,nx)),dtype=np.float64)

for yr in range(ystr_anth,yend_anth+1):
    suffix = '.' + '{:04}'.format(yr)
    fgco2_file = path_g100 + '/' + fgdic_base_g100 + suffix
    ncg100 = netCDF4.Dataset(fgco2_file,'r')
    lon = ncg100.variables['lon'][:]
    lat = ncg100.variables['lat'][:]
    fgco2_mod[:,:] = ncg100.variables['fgco2'][0,:,:].copy()
    fgco2_mod = np.where(fgco2_mod < -9.0e33, np.NaN, fgco2_mod * 365.25 * 86400 * 1e3 / 12) # gC/m2/sec --> mol/m2/yr
    fgco2_g100[yr-ystr_anth,:,:] = fgco2_mod[:,:].copy()
    ncg100.close()

    fgco2_file = path_g100 + '/' + fgdic_base_prei_g100 + suffix
    ncg100 = netCDF4.Dataset(fgco2_file,'r')
    lon = ncg100.variables['lon'][:]
    lat = ncg100.variables['lat'][:]
    fgco2_mod[:,:] = ncg100.variables['fgco2nat'][0,:,:].copy()
    fgco2_mod = np.where(fgco2_mod < -9.0e33, np.NaN, fgco2_mod * 365.25 * 86400 * 1e3 / 12) # gC/m2/sec --> mol/m2/yr
    fgco2_prei_g100[yr-ystr_anth,:,:] = fgco2_mod[:,:].copy()
    ncg100.close()

DS_g100_uptake = xr.Dataset( {'fgco2anth': (['time','lat','lon'], fgco2_g100 - fgco2_prei_g100),},
                             coords = { 'time': timey,
                                        'lat': np.linspace(-89.5,89.5,num=180), 
                                        'lon': np.linspace(0.5,359.5,num=360), } )

####################################################################
# NP10km

print( "Loading NP10km data" )

path_fill = path_np10km + 'hst_ann-glb_fill'

fgco2_fill = np.array(np.empty((nyr_anth,ny,nx)),dtype=np.float64)
fgco2_prei_fill = np.array(np.empty((nyr_anth,ny,nx)),dtype=np.float64)

fgdic_base_fill = 'hs_FgDIC_woa1x1'
fgdic_prei_base_fill = 'hs_FgDIC_prei_woa1x1'

fgco2_mod = np.array(np.empty((ny,nx)),dtype=np.float64)

for yr in range(ystr_anth,yend_anth+1):

    suffix = '.' + '{:04}'.format(yr)

    fgco2_file = path_fill + '/y' + '{:04}'.format(yr) + '/' + fgdic_base_fill + suffix
    ncfill = netCDF4.Dataset(fgco2_file,'r')
    lon = ncfill.variables['lon'][:]
    lat = ncfill.variables['lat'][:]
    fgco2_mod[:,:] = ncfill.variables['fgco2'][0,:,:]
    fgco2_mod = np.where(fgco2_mod < -9.0e33, np.NaN, fgco2_mod * 365.25 * 86400 * 1e3 /12) # gC/m2/sec --> mol/m2/yr
    fgco2_fill[yr-ystr_anth,:,:] = fgco2_mod[:,:].copy()
    ncfill.close()

    fgco2_file = path_fill + '/y' + '{:04}'.format(yr) + '/' + fgdic_prei_base_fill + suffix
    ncfill = netCDF4.Dataset(fgco2_file,'r')
    lon = ncfill.variables['lon'][:]
    lat = ncfill.variables['lat'][:]
    fgco2_mod[:,:] = ncfill.variables['fgco2nat'][0,:,:]
    fgco2_mod = np.where(fgco2_mod < -9.0e33, np.NaN, fgco2_mod * 365.25 * 86400 * 1e3 /12 ) # gC/m2/sec --> mol/m2/yr
    fgco2_prei_fill[yr-ystr_anth,:,:] = fgco2_mod[:,:].copy()
    ncfill.close()

DS_fill_uptake = xr.Dataset( {'fgco2anth': (['time','lat','lon'], fgco2_fill - fgco2_prei_fill),},
                             coords = { 'time': timey,
                                        'lat': np.linspace(-89.5,89.5,num=180), 
                                        'lon': np.linspace(0.5,359.5,num=360), } )

da1 = DS_g100_uptake.fgco2anth.sel(time=slice(str(ystr_anth),str(yend_anth))).mean(dim='time',skipna=False)
da2 = DS_fill_uptake.fgco2anth.sel(time=slice(str(ystr_anth),str(yend_anth))).mean(dim='time',skipna=False)
DS_uptake = xr.Dataset( {'exp1': (['lat','lon'], da1.values),
                         'exp2': (['lat','lon'], da2.values),
                         'exp2-1': (['lat','lon'], da2.values-da1.values),},
                         coords = { 'lat': np.linspace(-89.5,89.5,num=180), 
                                    'lon': np.linspace(0.5,359.5,num=360), })

######################################################################################### 
dir_glb100km  = path_glb100km + "run-20211104o2/ANALY/annclim/"
dir_np10km  = path_np10km + "hst_annclim-glb_fill/"

### read netCDF data as xarray.DataSet ###
fgco2name_g100 = f"fgco2_anth_{yend_anth}-{ystr_anth}_{exp1}.nc"
fgco2path_g100 = f"{dir_glb100km}{fgco2name_g100}"
fgco2_anth_g100_ds = xr.open_dataset(fgco2path_g100)

fgco2name_np10 = f"fgco2_anth_{yend_anth}-{ystr_anth}_{exp2}.nc"
fgco2path_np10 = f"{dir_np10km}{fgco2name_np10}"
fgco2_anth_np10_ds = xr.open_dataset(fgco2path_np10)

dissicname_g100 = f"cant_storage_rate_{yend_anth}-{ystr_anth}_{exp1}.nc"
dissicpath_g100 = f"{dir_glb100km}{dissicname_g100}"
dissic_anth_g100_ds = xr.open_dataset(dissicpath_g100)

dissicname_np10 = f"cant_storage_rate_{yend_anth}-{ystr_anth}_{exp2}.nc"
dissicpath_np10 = f"{dir_np10km}{dissicname_np10}"
dissic_anth_np10_ds = xr.open_dataset(dissicpath_np10)


######################################################################################### 

print( 'Start drawing' )
fig = plt.figure(figsize=(11,6.5))
#fig.suptitle( suptitle, fontsize=18 )

proj = ccrs.PlateCarree(central_longitude=-140.)
lon_formatter = LongitudeFormatter(zero_direction_label=True)
lat_formatter = LatitudeFormatter()

ax = [
    plt.subplot(3,4,1,projection=proj),
    plt.subplot(3,4,2,projection=proj),
    plt.subplot(3,4,3,projection=proj),
    plt.subplot(3,4,4,projection=proj),
    plt.subplot(3,4,5,projection=proj),
    plt.subplot(3,4,6,projection=proj),
    plt.subplot(3,4,7,projection=proj),
    plt.subplot(3,4,8,projection=proj),
    plt.subplot(3,4,9,projection=proj),
    plt.subplot(3,4,10,projection=proj),
    plt.subplot(3,4,11,projection=proj),
    plt.axes([0.78,0.06,0.20,0.24])
]

# [left, bottom, width, height]
ax_cbar = [
    plt.axes([0.030,0.70,0.22,0.012]),
    plt.axes([0.275,0.70,0.22,0.012]),
    plt.axes([0.520,0.70,0.22,0.012]),
    plt.axes([0.765,0.70,0.22,0.012]),
    plt.axes([0.030,0.38,0.22,0.012]),
    plt.axes([0.275,0.38,0.22,0.012]),
    plt.axes([0.520,0.38,0.22,0.012]),
    plt.axes([0.765,0.38,0.22,0.012]),
    plt.axes([0.030,0.06,0.22,0.012]),
    plt.axes([0.275,0.06,0.22,0.012]),
    plt.axes([0.520,0.06,0.22,0.012]),
]

cmap = [[ 'YlOrRd', 'YlOrRd', 'bwr', 'YlOrRd' ],
        [ 'YlOrRd', 'YlOrRd', 'bwr', 'YlOrRd' ],
        [ 'twilight_shifted', 'twilight_shifted', 'bwr', 'dummy' ]]

unit = [ '[$\mathrm{mol}\, \mathrm{m}^{-2}$]',
         '[$\mathrm{mol}\, \mathrm{m}^{-2} \, \mathrm{yr}^{-1}$]',
         '[$\mathrm{mol}\, \mathrm{m}^{-2} \, \mathrm{yr}^{-1}$]',]

var = [ 'exp1', 'exp2', 'exp2-1', 'obs' ]

panel = 0
item = 0

for item in range(3):

    if (item == 0):
        DS=DS_anth
        bounds1 = [0.0, 5.0, 10.0, 15.0, 20.0, 25.0, 30.0, 35.0, 40.0, 45.0, 50.0, 60.0, 70.0, 100.0]
        bounds2 = [-20.0, -10.0, -7.0, -4.0, -2.0, -1.0, 1.0, 2.0, 4.0, 7.0, 10.0, 20.0]
        bounds3 = [0.0, 5.0, 10.0, 15.0, 20.0, 25.0, 30.0, 35.0, 40.0, 45.0, 50.0, 60.0, 70.0, 100.0]

    if (item == 1):
        DS=DS_rate
        bounds1 = [0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]
        bounds2 = [-0.3, -0.2, -0.1, -0.05, -0.02, 0.02, 0.05, 0.1, 0.2, 0.3]
        bounds3 = [0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]
#        bounds3 = [0.0, 5.0, 10.0, 15.0, 20.0, 25.0, 30.0, 35.0, 40.0, 45.0, 50.0, 60.0, 70.0, 100.0]

    if (item == 2):
        DS=DS_uptake
        bounds1 = [-2.0, -1.5, -1.0, -0.7, -0.4, -0.2, 0.2, 0.4, 0.7, 1.0, 1.5, 2.0]
        bounds2 = [-0.3, -0.2, -0.1, -0.05, -0.02, 0.02, 0.05, 0.1, 0.2, 0.3]
        bounds3 = [-10.0, -5.0, -4.0, -3.0, -2.0, -1.0, -0.5, 0.5, 1.0, 2.0, 3.0, 4.0, 5.0, 10.0]

    for nv_out in range(4):

        print("Drawing panel "+ str(panel))

        if (nv_out == 0 or nv_out == 1):
            bounds = bounds1
            ticks_bounds = bounds1
            da = DS[var[nv_out]]
            units = unit[item]
            title = title_top[item][nv_out]
        elif (nv_out == 2):
            bounds = bounds2
            ticks_bounds = bounds2
            da = DS[var[nv_out]]
            units = unit[item]
            title = title_top[item][nv_out]
        else:
            if (item != 2):
                bounds = bounds3
                ticks_bounds = bounds3
                da = DS[var[nv_out]]
                title = title_top[item][nv_out]
                units = unit[item]


        if (item == 2 and nv_out == 3):
            ylim=[-2,15]
            yint=2
            fgco2_anth_g100_ds['fgco2_anth'].plot(ax=ax[panel],color='skyblue',label="GLB100km-fgco2",linewidth=1)
            dissic_anth_g100_ds['cant_storage_rate'].plot(ax=ax[panel],color='darkblue',label="GLB100km-dissic",linewidth=1)
            fgco2_anth_np10_ds['fgco2_anth'].plot(ax=ax[panel],color='lightsalmon',label="NP10km-fgco2",linewidth=1)
            dissic_anth_np10_ds['cant_storage_rate'].plot(ax=ax[panel],color='darkred',label="NP10km-dissic",linewidth=1)
            ax[panel].set_xlabel("latitude",fontsize=7)
            ax[panel].set_xlim(-90,90)
            ax[panel].set_xticks(np.linspace(-90,90,7))
            ax[panel].set_ylabel(r"[$\mathrm{TgC} \mathrm{yr}^{-1}$]",fontsize=7,labelpad=0)
            ax[panel].set_ylim(ylim[0],ylim[1])
            ax[panel].set_yticks(np.arange(ylim[0],ylim[1]+yint*0.1,yint))
            ax[panel].tick_params(labelsize=7)
            ax[panel].legend(bbox_to_anchor=(0.00,0.00),loc='lower left',ncol=1,fontsize=6)
            ax[panel].grid()

            ax[panel].set_title(r"(l) Surface uptake and storage rate of C$_{\mathrm{anth}}$", fontsize=8)
        else:
            im = da.plot(ax=ax[panel],cmap=cmap[item][nv_out],
                         levels=bounds,
                         extend='both',
                         add_colorbar=False,
                         transform=ccrs.PlateCarree())
            
            cbar = plt.colorbar(im,orientation='horizontal',spacing='uniform',
                                ticks=ticks_bounds, cax=ax_cbar[panel])
            cbar.set_label(label=units,fontsize=6,x=0.90)
            cbar.ax.tick_params(labelsize=6)

            ax[panel].coastlines()
            ax[panel].set_xticks(np.arange(120,285.1,30),crs=ccrs.PlateCarree())
            ax[panel].set_yticks(np.arange(-10,60.1,10),crs=ccrs.PlateCarree())
            ax[panel].set_extent([99,286,-16,66],crs=ccrs.PlateCarree())
            ax[panel].set_aspect(1.3)
            ax[panel].xaxis.set_major_formatter(lon_formatter)
            ax[panel].yaxis.set_major_formatter(lat_formatter)
            ax[panel].set_xlabel('')
            ax[panel].set_ylabel('')
            ax[panel].set_title(title,{'fontsize':8, 'verticalalignment':'top'})
            ax[panel].tick_params(labelsize=7)
            ax[panel].patch.set_facecolor('lightgray')

        panel+=1

plt.subplots_adjust(left=0.04,right=0.98,bottom=0.07,top=0.98,wspace=0.20,hspace=0.23)

outpdf = outfile+'.pdf'
outpng = outfile+'.png'

plt.savefig(outpng, bbox_inches='tight', pad_inches=0.1)
plt.savefig(outpdf, bbox_inches='tight', pad_inches=0.1)

print("figure is saved to " + outpng + " and " + outpdf)

plt.show()
