# -*- coding: utf-8 -*-
import sys
import json
import math
import calendar
import datetime
import numpy as np
import pandas as pd
import xarray as xr

import cartopy.crs as ccrs
import cartopy.feature as cfeature
from cartopy.mpl.ticker import LongitudeFormatter, LatitudeFormatter
from cartopy.util import add_cyclic_point

import matplotlib.pyplot as plt
from matplotlib.dates import drange, date2num, DateFormatter

import netCDF4
from netCDF4 import Dataset, num2date

import py3grads
from pathlib import Path

##########################################

if (len(sys.argv) < 4):
    print ('Usage: ' + sys.argv[0] + ' start_year end_year region1 region2')
    print ([ "(1) Tropics", "(2) SW-Subtropical", "(3) NW-Subtropical", "(4) E-Subtropical", "(5) Subarctic", "(6) JapanSea", "(7) OkhotskSea" ])
    sys.exit()

styr = int(sys.argv[1])
edyr = int(sys.argv[2])

tregion= np.empty(2,dtype='int64')
tregion[0] = int(sys.argv[3]) - 1
tregion[1] = int(sys.argv[4]) - 1

nyr = edyr - styr + 1

#suptitle_sto = r'Anth CO$_2$ column inventory time series,  year = ' + str(styr) + ' - ' + str(edyr)

##########################################
# In principle, following should be stored in json file.

title_list = [ "(a) Tropics",
               "(b) SW-Subtropical",
               "(c) NW-Subtropical",
               "(d) E-Subtropical",
               "(e) Subarctic",
               "(f) Japan Sea",
               "(g) Okhotsk sea" ]

item_list = [ "Tropics", "SW-Subtropical", "NW-Subtropical",
              "E-Subtropical", "Subarctic", "JapanSea", "OkhotskSea" ]

fig_list = [ "TR", "SWST", "NWST", "EST", "SA", "JPN", "OKH" ]


glb_list = [ "GLB-TRP", "GLB-SWST", "GLB-NWST", "GLB-SAR", "GLB-JPS", "GLB-OKH", "GLB-EST" ]
np_list = [ "NP-TRP", "NP-SWST", "NP-NWST", "NP-SAR", "NP-JPS", "NP-OKH", "NP-EST" ]

storange_list = [ [-0.5, 16.0], [-0.5, 8.0], [-0.2, 4.0],
                  [-0.2, 4.0], [-0.2, 4.0], [-0.2, 4.0], [-0.2, 4.0],  ] 

diff_list = [ [-0.5, 0.2], [-0.2, 0.5], [-0.2, 0.5],
              [-0.2, 0.5], [-0.2, 0.5], [-0.2, 0.5], [-0.2, 0.5],  ] 

##########################################
# basin mask

path_mask = '../data/np10km/mask'
file_mask_ctl = path_mask + '/' + 'pacific_mask3_np.ctl'

ga = py3grads.Grads(verbose=False)
print(file_mask_ctl)
ga('open '+str(file_mask_ctl))

ga('set t 1')
ga('set y 1')
ga('set x 1 2049')
lons = ga.exp('lon')
print(lons.shape)
ga('set x 1')
ga('set y 1 784')
lats = ga.exp('lat')
print(lats.shape)

ga('set x 1 2049')
ga('set y 1 784')

mask = ga.exp('index')
print(mask.shape)

nx=len(lons)
lonb=np.array(np.empty((nx+1)),dtype=np.float64)
lonb[0] = lons[0] - 0.5 * (lons[1]-lons[0])
for i in range(1,nx):
   lonb[i] = 0.5 * (lons[i-1]+lons[i])

lonb[nx] = lons[nx-1] + 0.5 * (lons[nx-1]-lons[nx-2])

ny=len(lats)
latb=np.array(np.empty((ny+1)),dtype=np.float64)
latb[0] = lats[0] - 0.5 * (lats[1]-lats[0])
for i in range(1,ny):
   latb[i] = 0.5 * (lats[i-1]+lats[i])

latb[ny] = lats[ny-1] + 0.5 * (lats[ny-1]-lats[ny-2])

mask = np.where(mask==21,10,mask)
mask = np.where(mask==22,9,mask)
mask = np.where(mask==23,8,mask)
mask = np.where(mask==24,0,mask)
mask = np.where(mask==25,6,mask)

####################################################################
path_glb100km='../data/glb100km/'
path_np10km='../data/np10km/'
###############################################################

num_mon=12

time_mon = np.empty(num_mon,dtype='object')

# gondola_100

for year in range(styr,edyr+1):

    leap = calendar.isleap(year)
    print(year,leap)

    for mon in range(12):
        time_mon[mon] = datetime.datetime(year,mon+1,1)

    #####
    print( "Loading monthly data" )

    path_mon=path_glb100km+'run-20211104o2/ANALY/monthly'

    #base_st='hs_pac2_Cant_storage_lyr.*'
    #st_file = path_mon + '/' + base_st
    #DS_st = xr.open_mfdataset(paths=st_file,concat_dim='time',combine='nested',decode_times=False)
    base_st='hs_pac3_Cant_storage_lyr.'+str(year)
    st_file = path_mon + '/' + base_st
    DS_st = xr.open_dataset(st_file,decode_times=False)
    DS_st = DS_st.assign_coords({'time':time_mon})
    DS_st = DS_st*1.e-15 # [gC] -> [PgC]

    if (year == styr):
        DS_st_all = DS_st.copy()
    else:
        DS_st_all = xr.concat([DS_st_all,DS_st],dim='time')


DS_st_all=DS_st_all.rename_dims({'depth':'density'})
DS_st_all=DS_st_all.rename_vars({'depth':'density'})
print(DS_st_all)

###########################################
# np

for year in range(styr,edyr+1):

    leap = calendar.isleap(year)
    print(year,leap)

    for mon in range(12):
        time_mon[mon] = datetime.datetime(year,mon+1,1)

    #####
    print( "Loading monthly data" )

    path_mon=path_np10km+'hst_mon-np'

    base_st='hs_pac3_Cant_storage_lyr.*'
    st_file = path_mon + '/y' + str(year) + '/' + base_st
    DS_st = xr.open_mfdataset(paths=st_file,concat_dim='time',combine='nested',decode_times=False)
    DS_st = DS_st.assign_coords({'time':time_mon})
    DS_st = DS_st*1.e-15 # [gC] -> [PgC]

    if (year == styr):
        DS_st.loc[dict(time=time_mon[0])]=np.NaN
        DS_st_np = DS_st.copy()
    else:
        DS_st_np = xr.concat([DS_st_np,DS_st],dim='time')


DS_st_np=DS_st_np.rename_dims({'depth':'density'})
DS_st_np=DS_st_np.rename_vars({'depth':'density'})
print(DS_st_np)

dens=DS_st_all.coords["density"].values
print(dens)

#####################################

infl1 = path_glb100km+"./run-20211104o2/csv/Pac3_Cant_storage_lyr_GLB100km_1987-2017.csv"
df1=pd.read_csv(infl1,index_col=0)
infl2 = path_glb100km+"./run-20211104o2/csv/Pac3_Volume_lyr_GLB100km_1987-2017.csv"
df2=pd.read_csv(infl2,index_col=0)
df3=df1/df2*1e9/12 # mmol/m3
#otfl3 = path_glb100km+"./run-20211104o2/csv/Pac3_Cant_conc_lyr_GLB100km_1987-2017.csv"
#df3.to_csv(otfl3)

df1.set_axis(pd.Index(glb_list),axis=1,inplace=True)
df2.set_axis(pd.Index(glb_list),axis=1,inplace=True)
df3.set_axis(pd.Index(glb_list),axis=1,inplace=True)

infl4 = path_np10km+"csv/Pac3_Cant_storage_lyr_NP10km_1987-2017.csv"
df4=pd.read_csv(infl4,index_col=0)
infl5 = path_np10km+"csv/Pac3_Volume_lyr_NP10km_1987-2017.csv"
df5=pd.read_csv(infl5,index_col=0)
df6=df4/df5*1e9/12 # mmol/m3

df4.set_axis(pd.Index(np_list),axis=1,inplace=True)
df5.set_axis(pd.Index(np_list),axis=1,inplace=True)
df6.set_axis(pd.Index(np_list),axis=1,inplace=True)
#otfl6 = path_np10km+"csv/Pac3_Cant_conc_lyr_NP10km_1987-2017.csv"
#df6.to_csv(otfl6)

dfa=pd.concat([df1,df4],axis=1)
dfb=pd.concat([df2,df5],axis=1)
dfc=pd.concat([df3,df6],axis=1)

print(dfa)

#####################################

fig = plt.figure(figsize=(8.5,11))
#fig.suptitle( suptitle_sto, fontsize=18 )

proj = ccrs.PlateCarree(central_longitude=-160.)
lon_formatter = LongitudeFormatter(zero_direction_label=True)
lat_formatter = LatitudeFormatter()

fig_index = ["(a)", "(b)", "(d)", "(f)", "(h)", "(c)", "(e)", "(g)", "(i)" ]

# [left, bottom, width, height]

ax = [ plt.axes([0.30,0.78,0.41,0.18],projection=proj),
       plt.axes([0.07,0.59,0.41,0.13]),
       plt.axes([0.07,0.41,0.41,0.13]),
       plt.axes([0.07,0.23,0.41,0.13]),
       plt.axes([0.07,0.05,0.41,0.13]),
       plt.axes([0.56,0.59,0.41,0.13]),
       plt.axes([0.56,0.41,0.41,0.13]),
       plt.axes([0.56,0.23,0.41,0.13]),
       plt.axes([0.56,0.05,0.41,0.13]), ]

title = '(a) Pacific region mask '
ca=ax[0].pcolormesh(lonb, latb, mask, cmap='tab20',transform=ccrs.PlateCarree())
ax[0].coastlines()
ax[0].text(145,35,'NW\nSubtropical',horizontalalignment='left',verticalalignment='center',fontsize=8,transform=ccrs.PlateCarree())
ax[0].text(145,20,'SW\nSubtropical',horizontalalignment='left',verticalalignment='center',fontsize=8,transform=ccrs.PlateCarree())
ax[0].text(200,28,'E-\nSubtropical',horizontalalignment='left',verticalalignment='center',fontsize=8,transform=ccrs.PlateCarree())
ax[0].set_xticks(np.arange(120,290.1,30),crs=ccrs.PlateCarree())
ax[0].set_yticks(np.arange(-20,66.1,10),crs=ccrs.PlateCarree())
ax[0].xaxis.set_major_formatter(lon_formatter)
ax[0].yaxis.set_major_formatter(lat_formatter)
ax[0].set_aspect(1.4)
ax[0].set_title(title,{'fontsize':9,})

nax=1
for reg in range(2):

    print("Region : " + item_list[tregion[reg]])

    ds_glb = DS_st_all[item_list[tregion[reg]]].sum(dim='density',skipna=False)
    ds_glb.plot.line(ax=ax[nax],x="time",label='GLB100km',color='blue')
    ds_np = DS_st_np[item_list[tregion[reg]]].sum(dim='density',skipna=False)
    ds_np.plot.line(ax=ax[nax],x="time",label='NP10km',color='orange')
    ds_diff=ds_np-ds_glb
    ds_diff.plot.line(ax=ax[nax],x="time",label='NP10km-GLB100km',color='red')

    ax[nax].set_title(fig_index[nax] + r' C$_{\mathrm{anth}}$ column inventory ' + item_list[tregion[reg]],{'fontsize':9,})
    ax[nax].set_xlabel('')
    ax[nax].set_ylabel('[$\mathrm{PgC}$]',fontsize=8)
    ax[nax].set_ylim(storange_list[tregion[reg]][0],storange_list[tregion[reg]][1])
    ax[nax].legend(bbox_to_anchor=(0.01,1.0),loc='upper left',fontsize=7,)
    ax[nax].tick_params(labelsize=7)
    ax[nax].grid()

    nax+=1

    if (tregion[reg] < 3):
        list2 = tregion[reg]
    elif (tregion[reg] == 3):
        list2 = 6
    else:
        list2 = tregion[reg] - 1 

    print("Region : " + glb_list[list2] + " " + np_list[list2])

    dfa[[glb_list[list2],np_list[list2]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
    ax[nax].set_title(fig_index[nax] + r' C$_{\mathrm{anth}}$ storage '+item_list[tregion[reg]],{'fontsize':9})
    ax[nax].set_xlabel('')
    ax[nax].set_ylabel('[$\mathrm{PgC}$]',fontsize=8)
    ax[nax].set_ylim(0.0,0.8)
    ax[nax].set_xticklabels(dfa.index.values[0:10],rotation=45)
    ax[nax].tick_params(labelsize=7)
    ax[nax].legend(fontsize=7)
    #ax[nax].set_ylim(cnvrange_list[item][0],cnvrange_list[item][1])

    nax+=1

    #print("before", dfb[[glb_list[list2],np_list[list2]]].iloc[9])
    dfb.loc[27.25,(glb_list[list2],np_list[list2])] = 0.0
    #print("after", dfb[[glb_list[list2],np_list[list2]]].iloc[9])
    dfb[[glb_list[list2],np_list[list2]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
    ax[nax].set_title(fig_index[nax] +' Water volume '+item_list[tregion[reg]],{'fontsize':9})
    ax[nax].set_xlabel('')
    ax[nax].set_ylabel('[$\mathrm{km}^{3}$]',fontsize=8)
    ax[nax].set_xticklabels(dfa.index.values[0:10],rotation=45)
    ax[nax].set_ylim(0.0,4e6)
    ax[nax].tick_params(labelsize=7)
    ax[nax].legend(fontsize=7,loc='upper left')
    #ax[nax].set_ylim(cnvrange_list[item][0],cnvrange_list[item][1])

    nax+=1

    dfc[[glb_list[list2],np_list[list2]]].iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
    ax[nax].set_title(fig_index[nax] +' Mean C$_{\mathrm{anth}}$ concentration '+item_list[tregion[reg]],{'fontsize':9})
    ax[nax].set_xlabel('')
    ax[nax].set_ylabel('[$\mathrm{mmol}\,\mathrm{m}^{-3}$]',fontsize=8)
    ax[nax].set_ylim(0.0,60)
    ax[nax].set_xticklabels(dfa.index.values[0:10],rotation=45)
    ax[nax].tick_params(labelsize=7)
    ax[nax].legend(fontsize=7,loc='upper right')
    #ax[nax].set_ylim(cnvrange_list[item][0],cnvrange_list[item][1])

    nax+=1

#plt.tight_layout(rect=[0,0,1,0.95])

outfile = 'fig/Canth_storage_all_time_mon_'+fig_list[tregion[0]]+'-'+fig_list[tregion[1]]+'_'+str(styr)+'-'+str(edyr)
outpng=outfile+'.png'
outpdf=outfile+'.pdf'
plt.savefig(outpng, bbox_inches='tight', pad_inches=0.1)
plt.savefig(outpdf, bbox_inches='tight', pad_inches=0.1)
plt.show()
plt.close()
