# -*- coding: utf-8 -*-
import sys
import json
import math
import calendar
import datetime
import numpy as np
import pandas as pd
import xarray as xr
import matplotlib.pyplot as plt
import matplotlib.ticker as ticker
from matplotlib.dates import drange, date2num, DateFormatter
import netCDF4
from netCDF4 import Dataset, num2date

##########################################

if (len(sys.argv) < 5):
    print ('Usage: ' + sys.argv[0] + ' start_year end_year region1 region2 density')
    print ([ "(1) Tropics", "(2) SW-Subtropical", "(3) NW-Subtropical", "(4) E-Subtropical", "(5) Subarctic", "(6) JapanSea", "(7) OkhotskSea" ])
    sys.exit()

styr = int(sys.argv[1])
edyr = int(sys.argv[2])

tregion= np.empty(2,dtype='int64')
tregion[0] = int(sys.argv[3]) - 1
tregion[1] = int(sys.argv[4]) - 1

tdens = float(sys.argv[5])

nyr = edyr - styr + 1

#suptitle_sto = r'Anth CO$_2$ storage time series, density = ' +str(tdens) + ', year = ' + str(styr) + ' - ' + str(edyr)

##########################################
# In principle, following should be stored in json file.

title_list = [ "(a) Tropics",
               "(b) SW-Subtropical",
               "(c) NW-Subtropical",
               "(d) E-Subtropical",
               "(e) Subarctic",
               "(f) Japan Sea",
               "(g) Okhotsk sea" ]

item_list = [ "Tropics", "SW-Subtropical", "NW-Subtropical",
              "E-Subtropical", "Subarctic", "JapanSea", "OkhotskSea" ]

fig_list = [ "TR", "SWST", "NWST", "EST", "SA", "JPN", "OHK" ]

storange_list = [ [-0.5, 2.0], [-0.5, 2.0], [-0.2, 1.0],
                  [-0.2, 1.0], [-0.2, 1.0], [-0.2, 1.0], [-0.2, 1.0],  ] 

diff_list = [ [-0.2, 0.5], [-0.2, 0.5], [-0.2, 0.5],
              [-0.2, 0.5], [-0.2, 0.5], [-0.2, 0.5], [-0.2, 0.5],  ] 

volrange_list = [ [-0.1e6, 1.5e6], [-0.1e6, 1.5e6], [-0.1e6, 1.2e6], [-0.1e6, 1.5e6],
                  [-0.1e6, 1.5e6], [-0.1e6, 1.0e6], [-0.2e6, 1.0e6],  ] 

vol_diff_list = [ [-0.1e6, 0.5e6], [-0.1e6, 0.5e6], [-0.1e6, 0.5e6], [-0.1e6, 0.5e6],
                  [-0.1e6, 0.5e6], [-0.1e6, 0.5e6], [-0.2e6, 0.5e6],  ] 

dic_list = [ [1.9, 2.05], [1.9, 2.05], [1.9, 2.05],
             [1.9, 2.05], [1.9, 2.05], [1.9, 2.05], [1.9, 2.05],  ] 

dic_diff_list = [ [-10., 80.0], [-10., 80.], [-10., 80.],
                  [-10., 80.0], [-10., 80,], [-10., 80.], [-10., 80.],  ] 

betad_list = [ [8.0, 12.0], [8.5, 11.0], [8.5, 11.0],
               [8.0, 12.0], [8.0, 12.0], [8.0, 12.0], [8.0, 12.0],  ] 

betad_diff_list = [ [-0.5, 0.5], [-0.5, 0.5], [-0.5, 0.5],
                    [-0.5, 0.5], [-0.5, 0.5], [-0.5, 0.5], [-0.5, 0.5],  ] 

####################################################################
path_glb100km='../data/glb100km/'
path_np10km='../data/np10km/'
##########################################

num_mon=12

time_mon = np.empty(num_mon,dtype='object')

# gondola_100

for year in range(styr,edyr+1):

    leap = calendar.isleap(year)
    print(year,leap)

    for mon in range(12):
        time_mon[mon] = datetime.datetime(year,mon+1,1)

    #####
    print( "Loading monthly data" )

    path_mon=path_glb100km+'run-20211104o2/ANALY/monthly'

    #base_st='hs_pac2_Cant_storage_lyr.*'
    #st_file = path_mon + '/' + base_st
    #DS_st = xr.open_mfdataset(paths=st_file,concat_dim='time',combine='nested',decode_times=False)
    base_st='hs_pac3_Cant_storage_lyr.'+str(year)
    st_file = path_mon + '/' + base_st
    DS_st = xr.open_dataset(st_file,decode_times=False)
    DS_st = DS_st.assign_coords({'time':time_mon})
    DS_st = DS_st*1.e-15 # [gC] -> [PgC]

    if (year == styr):
        DS_st_all = DS_st.copy()
    else:
        DS_st_all = xr.concat([DS_st_all,DS_st],dim='time')


DS_st_all=DS_st_all.rename_dims({'depth':'density'})
DS_st_all=DS_st_all.rename_vars({'depth':'density'})
print(DS_st_all)

###########################################
# np

for year in range(styr,edyr+1):

    leap = calendar.isleap(year)
    print(year,leap)

    for mon in range(12):
        time_mon[mon] = datetime.datetime(year,mon+1,1)

    #####
    print( "Loading monthly data" )

    path_mon=path_np10km+'hst_mon-np'

    base_st='hs_pac3_Cant_storage_lyr.*'
    st_file = path_mon + '/y' + str(year) + '/' + base_st
    DS_st = xr.open_mfdataset(paths=st_file,concat_dim='time',combine='nested',decode_times=False)
    DS_st = DS_st.assign_coords({'time':time_mon})
    DS_st = DS_st*1.e-15 # [gC] -> [PgC]

    if (year == styr):
        DS_st.loc[dict(time=time_mon[0])]=np.NaN
        DS_st_np = DS_st.copy()
    else:
        DS_st_np = xr.concat([DS_st_np,DS_st],dim='time')


DS_st_np=DS_st_np.rename_dims({'depth':'density'})
nnDS_st_np=DS_st_np.rename_vars({'depth':'density'})
print(DS_st_np)

dens=DS_st_all.coords["density"].values
print(dens)

#####################################
# gondola_100

for year in range(styr,edyr+1):

    leap = calendar.isleap(year)
    print(year,leap)

    for mon in range(12):
        time_mon[mon] = datetime.datetime(year,mon+1,1)

    #####
    print( "Loading monthly data" )

    path_mon=path_glb100km+'run-20211104o2/ANALY/monthly'

    base_vl='hs_pac3_volume_lyr.'+str(year)
    st_file = path_mon + '/' + base_vl
    DS_vl = xr.open_dataset(st_file,decode_times=False)
    DS_vl = DS_vl.assign_coords({'time':time_mon})
    DS_vl = DS_vl*1.e-15 # [cm3] -> [km3]

    if (year == styr):
        DS_vl_all = DS_vl.copy()
    else:
        DS_vl_all = xr.concat([DS_vl_all,DS_vl],dim='time')


DS_vl_all=DS_vl_all.rename_dims({'depth':'density'})
DS_vl_all=DS_vl_all.rename_vars({'depth':'density'})
print(DS_vl_all)

###########################################
# np

for year in range(styr,edyr+1):

    leap = calendar.isleap(year)
    print(year,leap)

    for mon in range(12):
        time_mon[mon] = datetime.datetime(year,mon+1,1)

    #####
    print( "Loading monthly data" )

    path_mon=path_np10km+'hst_mon-np'


    base_vl='hs_pac3_volume_lyr.*'
    st_file = path_mon + '/y' + str(year) + '/' + base_vl
    DS_vl = xr.open_mfdataset(paths=st_file,concat_dim='time',combine='nested',decode_times=False)
    DS_vl = DS_vl.assign_coords({'time':time_mon})
    DS_vl = DS_vl*1.e-15 # [cm3] -> [km3]

    if (year == styr):
        DS_vl.loc[dict(time=time_mon[0])]=np.NaN
        DS_vl_np = DS_vl.copy()
    else:
        DS_vl_np = xr.concat([DS_vl_np,DS_vl],dim='time')


DS_vl_np=DS_vl_np.rename_dims({'depth':'density'})
DS_vl_np=DS_vl_np.rename_vars({'depth':'density'})
print(DS_vl_np)

dens=DS_vl_all.coords["density"].values
print(dens)

#####################################
# gondola_100

for year in range(styr,edyr+1):

    leap = calendar.isleap(year)
    print(year,leap)

    for mon in range(12):
        time_mon[mon] = datetime.datetime(year,mon+1,1)

    #####
    print( "Loading monthly data" )

    path_mon=path_glb100km+'run-20211104o2/ANALY/monthly'

    base_dic='hs_pac3_DIC_average_lyr.'+str(year)
    st_file = path_mon + '/' + base_dic
    DS_dic = xr.open_dataset(st_file,decode_times=False)
    DS_dic = DS_dic.assign_coords({'time':time_mon})

    if (year == styr):
        DS_dic_all = DS_dic.copy()
    else:
        DS_dic_all = xr.concat([DS_dic_all,DS_dic],dim='time')

    base_prei_dic='hs_pac3_DIC_prei_average_lyr.'+str(year)
    st_prei_file = path_mon + '/' + base_prei_dic
    DS_prei_dic = xr.open_dataset(st_prei_file,decode_times=False)
    DS_prei_dic = DS_prei_dic.assign_coords({'time':time_mon})

    if (year == styr):
        DS_prei_dic_all = DS_prei_dic.copy()
    else:
        DS_prei_dic_all = xr.concat([DS_prei_dic_all,DS_prei_dic],dim='time')


DS_dic_all=DS_dic_all.rename_dims({'depth':'density'})
DS_dic_all=DS_dic_all.rename_vars({'depth':'density'})
print(DS_dic_all)

DS_prei_dic_all=DS_prei_dic_all.rename_dims({'depth':'density'})
DS_prei_dic_all=DS_prei_dic_all.rename_vars({'depth':'density'})

###########################################
# np

for year in range(styr,edyr+1):

    leap = calendar.isleap(year)
    print(year,leap)

    for mon in range(12):
        time_mon[mon] = datetime.datetime(year,mon+1,1)

    #####
    print( "Loading monthly data" )

    path_mon=path_np10km+'hst_mon-np'

    base_dic='hs_pac3_DIC_average_lyr.*'
    st_file = path_mon + '/y' + str(year) + '/' + base_dic
    DS_dic = xr.open_mfdataset(paths=st_file,concat_dim='time',combine='nested',decode_times=False)
    DS_dic = DS_dic.assign_coords({'time':time_mon})

    if (year==styr):
        DS_dic.loc[dict(time=time_mon[0])]=np.NaN

    if (year == styr):
        DS_dic_np = DS_dic.copy()
    else:
        DS_dic_np = xr.concat([DS_dic_np,DS_dic],dim='time')

    base_prei_dic='hs_pac3_DIC_prei_average_lyr.*'
    st_prei_file = path_mon + '/y' + str(year) + '/' + base_prei_dic
    DS_prei_dic = xr.open_mfdataset(paths=st_prei_file,concat_dim='time',combine='nested',decode_times=False)
    DS_prei_dic = DS_prei_dic.assign_coords({'time':time_mon})

    if (year==styr):
        DS_prei_dic.loc[dict(time=time_mon[0])]=np.NaN


    if (year == styr):
        DS_prei_dic_np = DS_prei_dic.copy()
    else:
        DS_prei_dic_np = xr.concat([DS_prei_dic_np,DS_prei_dic],dim='time')


DS_dic_np=DS_dic_np.rename_dims({'depth':'density'})
DS_dic_np=DS_dic_np.rename_vars({'depth':'density'})
print(DS_dic_np)

DS_prei_dic_np=DS_prei_dic_np.rename_dims({'depth':'density'})
DS_prei_dic_np=DS_prei_dic_np.rename_vars({'depth':'density'})

dens=DS_dic_all.coords["density"].values
print(dens)

##############################################
# gondola_100

for year in range(styr,edyr+1):

    leap = calendar.isleap(year)
    print(year,leap)

    for mon in range(12):
        time_mon[mon] = datetime.datetime(year,mon+1,1)

    #####
    print( "Loading monthly data" )

    path_mon=path_glb100km+'run-20211104o2/ANALY/monthly'

    base_btd='hs_pac3_Betad_3d_average_lyr.'+str(year)
    st_file = path_mon + '/' + base_btd
    DS_btd = xr.open_dataset(st_file,decode_times=False)
    DS_btd = DS_btd.assign_coords({'time':time_mon})

    if (year == styr):
        DS_btd_all = DS_btd.copy()
    else:
        DS_btd_all = xr.concat([DS_btd_all,DS_btd],dim='time')

    base_prei_btd='hs_pac3_Betad_3d_prei_average_lyr.'+str(year)
    st_prei_file = path_mon + '/' + base_prei_btd
    DS_prei_btd = xr.open_dataset(st_prei_file,decode_times=False)
    DS_prei_btd = DS_prei_btd.assign_coords({'time':time_mon})

    if (year == styr):
        DS_prei_btd_all = DS_prei_btd.copy()
    else:
        DS_prei_btd_all = xr.concat([DS_prei_btd_all,DS_prei_btd],dim='time')


DS_btd_all=DS_btd_all.rename_dims({'depth':'density'})
DS_btd_all=DS_btd_all.rename_vars({'depth':'density'})
print(DS_btd_all)
DS_prei_btd_all=DS_prei_btd_all.rename_dims({'depth':'density'})
DS_prei_btd_all=DS_prei_btd_all.rename_vars({'depth':'density'})

###########################################
# np

for year in range(styr,edyr+1):

    leap = calendar.isleap(year)
    print(year,leap)

    for mon in range(12):
        time_mon[mon] = datetime.datetime(year,mon+1,1)

    #####
    print( "Loading monthly data" )

    path_mon=path_np10km+'hst_mon-np'

    base_btd='hs_pac3_Betad_3d_average_lyr.*'
    st_file = path_mon + '/y' + str(year) + '/' + base_btd
    DS_btd = xr.open_mfdataset(paths=st_file,concat_dim='time',combine='nested',decode_times=False)
    DS_btd = DS_btd.assign_coords({'time':time_mon})

    if (year==styr):
        DS_btd.loc[dict(time=time_mon[0])]=np.NaN


    if (year == styr):
        DS_btd_np = DS_btd.copy()
    else:
        DS_btd_np = xr.concat([DS_btd_np,DS_btd],dim='time')


    base_prei_btd='hs_pac3_Betad_3d_prei_average_lyr.*'
    st_prei_file = path_mon + '/y' + str(year) + '/' + base_prei_btd
    DS_prei_btd = xr.open_mfdataset(paths=st_prei_file,concat_dim='time',combine='nested',decode_times=False)
    DS_prei_btd = DS_prei_btd.assign_coords({'time':time_mon})

    if (year==styr):
        DS_prei_btd.loc[dict(time=time_mon[0])]=np.NaN


    if (year == styr):
        DS_prei_btd_np = DS_prei_btd.copy()
    else:
        DS_prei_btd_np = xr.concat([DS_prei_btd_np,DS_prei_btd],dim='time')


DS_btd_np=DS_btd_np.rename_dims({'depth':'density'})
DS_btd_np=DS_btd_np.rename_vars({'depth':'density'})
print(DS_btd_np)
DS_prei_btd_np=DS_prei_btd_np.rename_dims({'depth':'density'})
DS_prei_btd_np=DS_prei_btd_np.rename_vars({'depth':'density'})

dens=DS_btd_all.coords["density"].values
print(dens)


##############################################

fig = plt.figure(figsize=(8.5,11))

fig.suptitle( r"$\sigma_0$ = 25.25", fontsize=16, y=0.985 )

# [left, bottom, width, height]
ax = [ plt.axes([0.07,0.84,0.41,0.11]),
       plt.axes([0.07,0.68,0.41,0.11]),
       plt.axes([0.07,0.52,0.41,0.11]),
       plt.axes([0.07,0.36,0.41,0.11]),
       plt.axes([0.07,0.20,0.41,0.11]),
       plt.axes([0.07,0.04,0.41,0.11]),
       plt.axes([0.56,0.84,0.41,0.11]),
       plt.axes([0.56,0.68,0.41,0.11]),
       plt.axes([0.56,0.52,0.41,0.11]),
       plt.axes([0.56,0.36,0.41,0.11]),
       plt.axes([0.56,0.20,0.41,0.11]),
       plt.axes([0.56,0.04,0.41,0.11]), ]

dist=1.0
itrg=-9
for iden in range(len(dens)):
    if (abs(tdens-dens[iden]) < dist):
        itrg = iden
        dist = abs(tdens-dens[iden])

print("density = "+str(dens[itrg]))

fig_index = ["(a)", "(c)", "(e)", "(g)", "(i)", "(k)", "(b)", "(d)", "(f)", "(h)", "(j)", "(l)" ]

nax=0

for reg in range(2):

    print("Region : " + item_list[tregion[reg]])

    ds_glb = DS_st_all[item_list[tregion[reg]]].isel(density=itrg)
    ds_glb.plot.line(ax=ax[nax],x="time",label='GLB')
    ds_np = DS_st_np[item_list[tregion[reg]]].isel(density=itrg)
    ds_np.plot.line(ax=ax[nax],x="time",label='NP')
    ds_diff=ds_np-ds_glb
    ds_diff.plot.line(ax=ax[nax],x="time",label='NP-GLB',color='red')

    ax[nax].set_title(fig_index[nax]+r" C$_{\mathrm{anth}}$ storage "+item_list[tregion[reg]],{'fontsize':9})
    ax[nax].set_xlabel('')
    ax[nax].set_ylabel('[$\mathrm{PgC}$]',fontsize=8)
    ax[nax].set_ylim(storange_list[tregion[reg]][0],storange_list[tregion[reg]][1])
    ax[nax].legend(bbox_to_anchor=(0.01,1.0),loc='upper left',fontsize=8,)
    ax[nax].tick_params(labelsize=6)
    ax[nax].grid()

    nax+=1

    ds_glb = DS_vl_all[item_list[tregion[reg]]].isel(density=itrg)
    ds_glb.plot.line(ax=ax[nax],x="time",label='GLB')
    ds_np = DS_vl_np[item_list[tregion[reg]]].isel(density=itrg)
    ds_np.plot.line(ax=ax[nax],x="time",label='NP')
    ds_diff=ds_np-ds_glb
    ds_diff.plot.line(ax=ax[nax],x="time",label='NP-GLB',color='red')

    ax[nax].set_title(fig_index[nax]+" volume " + item_list[tregion[reg]],{'fontsize':9})
    ax[nax].set_xlabel('')
    ax[nax].set_ylabel('[$\mathrm{km}^{3}$]',fontsize=8)
    ax[nax].set_ylim(volrange_list[tregion[reg]][0],volrange_list[tregion[reg]][1])
    ax[nax].legend(bbox_to_anchor=(1.0,0.0),loc='lower right',ncol=3,fontsize=8,)
    ax[nax].tick_params(labelsize=6)
    ax[nax].grid()

    nax+=1

    ds_glb = DS_dic_all[item_list[tregion[reg]]].isel(density=itrg)
    ds_glb.plot.line(ax=ax[nax],x="time",label='GLB',color='blue')
    ds_np = DS_dic_np[item_list[tregion[reg]]].isel(density=itrg)
    ds_np.plot.line(ax=ax[nax],x="time",label='NP',color='red')

    ds_prei_glb = DS_prei_dic_all[item_list[tregion[reg]]].isel(density=itrg)
    ds_prei_glb.plot.line(ax=ax[nax],x="time",label='GLB-prei',color='darkblue')
    ds_prei_np = DS_prei_dic_np[item_list[tregion[reg]]].isel(density=itrg)
    ds_prei_np.plot.line(ax=ax[nax],x="time",label='NP-prei',color='darkred')

    ax[nax].set_title(fig_index[nax]+" DIC " + item_list[tregion[reg]],{'fontsize':9})
    ax[nax].set_xlabel('')
    ax[nax].set_ylabel('[$\mathrm{mol} \mathrm{m}^{-3}$]',fontsize=8)
    ax[nax].set_ylim(dic_list[tregion[reg]][0],dic_list[tregion[reg]][1])
    ax[nax].legend(bbox_to_anchor=(0.01,1.0),loc='upper left',ncol=2,fontsize=8,)
    ax[nax].tick_params(labelsize=6)
    ax[nax].grid()
    
    nax+=1

    ds_diff_glb=(ds_glb-ds_prei_glb)*1e3
    ds_diff_glb.plot.line(ax=ax[nax],x="time",label='GLB-ANTH',color='royalblue')
    ds_diff_np=(ds_np-ds_prei_np)*1e3
    ds_diff_np.plot.line(ax=ax[nax],x="time",label='NP-ANTH',color='darkorange')
    ds_diff=ds_diff_np-ds_diff_glb
    ds_diff.plot.line(ax=ax[nax],x="time",label='NP-GLB',color='red')

    ax[nax].set_title(fig_index[nax]+r" C$_{\mathrm{anth}}$ concentration " + item_list[tregion[reg]],{'fontsize':9})
    ax[nax].set_xlabel('')
    ax[nax].set_ylabel('[$\mathrm{mmol} \mathrm{m}^{-3}$]',fontsize=8)
    ax[nax].set_ylim(dic_diff_list[tregion[reg]][0],dic_diff_list[tregion[reg]][1])
    ax[nax].set_yticks(np.arange(dic_diff_list[tregion[reg]][0],dic_diff_list[tregion[reg]][1]+0.1,10))
    ax[nax].legend(bbox_to_anchor=(0.01,1.0),loc='upper left',fontsize=8,)
    ax[nax].tick_params(labelsize=6)
    ax[nax].grid()

    nax+=1

    ds_glb = DS_btd_all[item_list[tregion[reg]]].isel(density=itrg)
    ds_glb.plot.line(ax=ax[nax],x="time",label='GLB',color='blue')
    ds_np = DS_btd_np[item_list[tregion[reg]]].isel(density=itrg)
    ds_np.plot.line(ax=ax[nax],x="time",label='NP',color='red')

    ds_prei_glb = DS_prei_btd_all[item_list[tregion[reg]]].isel(density=itrg)
    ds_prei_glb.plot.line(ax=ax[nax],x="time",label='GLB-prei',color='darkblue')
    ds_prei_np = DS_prei_btd_np[item_list[tregion[reg]]].isel(density=itrg)
    ds_prei_np.plot.line(ax=ax[nax],x="time",label='NP-prei',color='darkred')

    ax[nax].set_title(fig_index[nax]+" Revelle Factor "+item_list[tregion[reg]],{'fontsize':9})
    ax[nax].set_xlabel('')
    ax[nax].set_ylabel('')
    #ax[nax].set_ylabel('[$\mathrm{mol} \mathrm{m}^{-3}$]',fontsize=12)
    ax[nax].set_ylim(betad_list[tregion[reg]][0],betad_list[tregion[reg]][1])
    ax[nax].legend(bbox_to_anchor=(0.01,1.0),loc='upper left',ncol=2,fontsize=8,)
    ax[nax].tick_params(labelsize=6)
    ax[nax].grid()
    
    nax+=1

    ds_diff=ds_np-ds_glb
    ds_diff.plot.line(ax=ax[nax],x="time",label='NP-GLB (contenporary)',color='salmon')
    ds_prei_diff=ds_prei_np-ds_prei_glb
    ds_prei_diff.plot.line(ax=ax[nax],x="time",label='NP-GLB (prei)',color='cyan')
    ax[nax].set_title(fig_index[nax]+" Revelle factor difference " + item_list[tregion[reg]],{'fontsize':9})
    ax[nax].set_xlabel('')
    ax[nax].set_ylabel('')
    ax[nax].set_ylim(betad_diff_list[tregion[reg]][0],betad_diff_list[tregion[reg]][1])
    ax[nax].legend(bbox_to_anchor=(0.01,1.0),loc='upper left',fontsize=8,)
    ax[nax].tick_params(labelsize=6)
    ax[nax].grid()

    nax+=1

#plt.tight_layout(rect=[0,0,1,0.95])
outfile = 'fig/Canth_storage_lyr_time_mon_'+fig_list[tregion[0]]+'-'+fig_list[tregion[1]]+'_'+str(int(tdens*100)) + '_'+str(styr)+'-'+str(edyr)
outpng = outfile + '.png'
outpdf = outfile + '.pdf'
plt.savefig(outpng, bbox_inches='tight', pad_inches=0.1)
plt.savefig(outpdf, bbox_inches='tight', pad_inches=0.1)
plt.show()
plt.close()
