# -*- coding: utf-8 -*-
import sys
import json
import math
import calendar
import datetime
import numpy as np
import pandas as pd
import xarray as xr

import cartopy.crs as ccrs
import cartopy.feature as cfeature
from cartopy.mpl.ticker import LongitudeFormatter, LatitudeFormatter
from cartopy.util import add_cyclic_point

import matplotlib.pyplot as plt
from matplotlib.dates import drange, date2num, DateFormatter

import netCDF4
from netCDF4 import Dataset, num2date

#import py3grads
#from pathlib import Path

##########################################

if (len(sys.argv) < 4):
    print ('Usage: ' + sys.argv[0] + ' start_year end_year region1 region2 density')
    print ([ "(1) Tropics", "(2) SW-Subtropical", "(3) NW-Subtropical", "(4) Subarctic", "(5) JapanSea", "(6) OkhotskSea", "(7) E-Subtropical" ])
    sys.exit()

styr = int(sys.argv[1])
edyr = int(sys.argv[2])

tregion= np.empty(2,dtype='int64')
tregion[0] = int(sys.argv[3]) - 1
tregion[1] = int(sys.argv[4]) - 1

tdens = float(sys.argv[5])

nyr = edyr - styr + 1

#suptitle_sto = r'Anth CO$_2$ column inventory time series,  year = ' + str(styr) + ' - ' + str(edyr)

##########################################
# In principle, following should be stored in json file.

title_list = [ "(a) Tropics",
               "(b) SW-Subtropical",
               "(c) NW-Subtropical",
               "(d) Subarctic",
               "(e) Japan Sea",
               "(f) Okhotsk sea",
               "(g) E-Subtropical" ]

item_list = [ "Tropics", "SW-Subtropical", "NW-Subtropical",
              "Subarctic", "JapanSea", "OkhotskSea", "E-Subtropical" ]

fig_list = [ "TR", "SWST", "NWST", "SA", "JPN", "OKH", "EST" ]


glb_list = [ "GLB-TRP", "GLB-SWST", "GLB-NWST", "GLB-SAR", "GLB-JPS", "GLB-OKH", "GLB-EST" ]
np_list = [ "NP-TRP", "NP-SWST", "NP-NWST", "NP-SAR", "NP-JPS", "NP-OKH", "NP-EST" ]

inventrange_list = [ [-0.5, 16.0], [-0.5, 8.0], [-0.2, 4.0],
                  [-0.2, 4.0], [-0.2, 4.0], [-0.2, 4.0], [-0.2, 4.0],  ] 

storange_list = [ [-0.5, 2.0], [-0.5, 2.0], [-0.2, 1.0],
                  [-0.2, 1.0], [-0.2, 1.0], [-0.2, 1.0], [-0.2, 1.0],  ] 

diff_list = [ [-0.5, 0.2], [-0.2, 0.5], [-0.2, 0.5],
              [-0.2, 0.5], [-0.2, 0.5], [-0.2, 0.5], [-0.2, 0.5],  ] 

volrange_list = [ [-0.15e6, 1.5e6], [-0.15e6, 1.5e6], [-0.15e6, 1.2e6], [-0.15e6, 1.5e6],
                  [-0.15e6, 1.5e6], [-0.15e6, 1.0e6], [-0.2e6, 1.0e6],  ] 

dic_diff_list = [ [-10., 80.0], [-10., 80.], [-10., 80.],
                  [-10., 80.0], [-10., 80,], [-10., 80.], [-10., 80.],  ] 

plot_list = [ "LowRes", "HighRes" ]

rem_dict_glb = dict(zip(item_list,glb_list))
rem_dict_np = dict(zip(item_list,np_list))

##########################################
# basin mask

path_mask = '../data/np10km/mask'
file_mask = path_mask + '/' + 'pacific_mask3_np.gd'

nxm=2049
nym=784

fl = open(file_mask,'rb')
mask_tmp = np.fromfile(fl, dtype = '>f', count= nxm*nym)
mask=mask_tmp.astype(np.float64).reshape(nym,nxm).copy()

latb = np.array(np.empty((nym)),dtype=np.float64)
for j in range(nym):
    latb[j] = -15.0 + 1/10 * (j - 2)

lonb = np.array(np.empty((nxm)),dtype=np.float64)
for i in range(nxm):
    lonb[i] = 99.0 + 1/11 * (i - 2)

#ga = py3grads.Grads(verbose=False)
#print(file_mask_ctl)
#ga('open '+str(file_mask_ctl))
#
#ga('set t 1')
#ga('set y 1')
#ga('set x 1 2049')
#lons = ga.exp('lon')
#print(lons.shape)
#ga('set x 1')
#ga('set y 1 784')
#lats = ga.exp('lat')
#print(lats.shape)
#
#ga('set x 1 2049')
#ga('set y 1 784')
#
#mask = ga.exp('index')
#print(mask.shape)
#
#nx=len(lons)
#lonb=np.array(np.empty((nx+1)),dtype=np.float64)
#lonb[0] = lons[0] - 0.5 * (lons[1]-lons[0])
#for i in range(1,nx):
#   lonb[i] = 0.5 * (lons[i-1]+lons[i])
#
#lonb[nx] = lons[nx-1] + 0.5 * (lons[nx-1]-lons[nx-2])
#
#ny=len(lats)
#latb=np.array(np.empty((ny+1)),dtype=np.float64)
#latb[0] = lats[0] - 0.5 * (lats[1]-lats[0])
#for i in range(1,ny):
#   latb[i] = 0.5 * (lats[i-1]+lats[i])
#
#latb[ny] = lats[ny-1] + 0.5 * (lats[ny-1]-lats[ny-2])
#
mask = np.where(mask==21,10,mask)
mask = np.where(mask==22,9,mask)
mask = np.where(mask==23,8,mask)
mask = np.where(mask==24,0,mask)
mask = np.where(mask==25,6,mask)
mask = np.where(mask==-9,np.nan,mask)

####################################################################
path_glb100km='../data/glb100km/'
path_np10km='../data/np10km/'
###############################################################

num_mon=12

time_mon = np.empty(num_mon,dtype='object')

# gondola_100

for year in range(styr,edyr+1):

    leap = calendar.isleap(year)
    print(year,leap)

    for mon in range(12):
        time_mon[mon] = datetime.datetime(year,mon+1,1)

    #####
    print( "Loading monthly data" )

    path_mon=path_glb100km+'run-20211104o2/ANALY/monthly'

    #base_st='hs_pac2_Cant_storage_lyr.*'
    #st_file = path_mon + '/' + base_st
    #DS_st = xr.open_mfdataset(paths=st_file,concat_dim='time',combine='nested',decode_times=False)
    base_st='hs_pac3_Cant_storage_lyr.'+str(year)
    st_file = path_mon + '/' + base_st
    DS_st = xr.open_dataset(st_file,decode_times=False)
    DS_st = DS_st.assign_coords({'time':time_mon})
    DS_st = DS_st*1.e-15 # [gC] -> [PgC]

    if (year == styr):
        DS_st_all = DS_st.copy()
    else:
        DS_st_all = xr.concat([DS_st_all,DS_st],dim='time')


DS_st_all=DS_st_all.rename_dims({'depth':'density'})
DS_st_all=DS_st_all.rename_vars({'depth':'density'})
print(DS_st_all)

###########################################
# np

for year in range(styr,edyr+1):

    leap = calendar.isleap(year)
    print(year,leap)

    for mon in range(12):
        time_mon[mon] = datetime.datetime(year,mon+1,1)

    #####
    print( "Loading monthly data" )

    path_mon=path_np10km+'hst_mon-np'

    base_st='hs_pac3_Cant_storage_lyr.*'
    st_file = path_mon + '/y' + str(year) + '/' + base_st
    DS_st = xr.open_mfdataset(paths=st_file,concat_dim='time',combine='nested',decode_times=False)
    DS_st = DS_st.assign_coords({'time':time_mon})
    DS_st = DS_st*1.e-15 # [gC] -> [PgC]

    if (year == styr):
        DS_st.loc[dict(time=time_mon[0])]=np.NaN
        DS_st_np = DS_st.copy()
    else:
        DS_st_np = xr.concat([DS_st_np,DS_st],dim='time')


DS_st_np=DS_st_np.rename_dims({'depth':'density'})
DS_st_np=DS_st_np.rename_vars({'depth':'density'})
print(DS_st_np)

dens=DS_st_all.coords["density"].values
print(dens)

#####################################
# gondola_100

for year in range(styr,edyr+1):

    leap = calendar.isleap(year)
    print(year,leap)

    for mon in range(12):
        time_mon[mon] = datetime.datetime(year,mon+1,1)

    #####
    print( "Loading monthly data" )

    path_mon=path_glb100km+'run-20211104o2/ANALY/monthly'

    base_vl='hs_pac3_volume_lyr.'+str(year)
    st_file = path_mon + '/' + base_vl
    DS_vl = xr.open_dataset(st_file,decode_times=False)
    DS_vl = DS_vl.assign_coords({'time':time_mon})
    DS_vl = DS_vl*1.e-15 # [cm3] -> [km3]

    if (year == styr):
        DS_vl_all = DS_vl.copy()
    else:
        DS_vl_all = xr.concat([DS_vl_all,DS_vl],dim='time')


DS_vl_all=DS_vl_all.rename_dims({'depth':'density'})
DS_vl_all=DS_vl_all.rename_vars({'depth':'density'})
print(DS_vl_all)

###########################################
# np

for year in range(styr,edyr+1):

    leap = calendar.isleap(year)
    print(year,leap)

    for mon in range(12):
        time_mon[mon] = datetime.datetime(year,mon+1,1)

    #####
    print( "Loading monthly data" )

    path_mon=path_np10km+'hst_mon-np'


    base_vl='hs_pac3_volume_lyr.*'
    st_file = path_mon + '/y' + str(year) + '/' + base_vl
    DS_vl = xr.open_mfdataset(paths=st_file,concat_dim='time',combine='nested',decode_times=False)
    DS_vl = DS_vl.assign_coords({'time':time_mon})
    DS_vl = DS_vl*1.e-15 # [cm3] -> [km3]

    if (year == styr):
        DS_vl.loc[dict(time=time_mon[0])]=np.NaN
        DS_vl_np = DS_vl.copy()
    else:
        DS_vl_np = xr.concat([DS_vl_np,DS_vl],dim='time')


DS_vl_np=DS_vl_np.rename_dims({'depth':'density'})
DS_vl_np=DS_vl_np.rename_vars({'depth':'density'})
print(DS_vl_np)

dens=DS_vl_all.coords["density"].values
print(dens)

#####################################
# gondola_100

for year in range(styr,edyr+1):

    leap = calendar.isleap(year)
    print(year,leap)

    for mon in range(12):
        time_mon[mon] = datetime.datetime(year,mon+1,1)

    #####
    print( "Loading monthly data" )

    path_mon=path_glb100km+'run-20211104o2/ANALY/monthly'

    base_dic='hs_pac3_DIC_average_lyr.'+str(year)
    st_file = path_mon + '/' + base_dic
    DS_dic = xr.open_dataset(st_file,decode_times=False)
    DS_dic = DS_dic.assign_coords({'time':time_mon})

    if (year == styr):
        DS_dic_all = DS_dic.copy()
    else:
        DS_dic_all = xr.concat([DS_dic_all,DS_dic],dim='time')

    base_prei_dic='hs_pac3_DIC_prei_average_lyr.'+str(year)
    st_prei_file = path_mon + '/' + base_prei_dic
    DS_prei_dic = xr.open_dataset(st_prei_file,decode_times=False)
    DS_prei_dic = DS_prei_dic.assign_coords({'time':time_mon})

    if (year == styr):
        DS_prei_dic_all = DS_prei_dic.copy()
    else:
        DS_prei_dic_all = xr.concat([DS_prei_dic_all,DS_prei_dic],dim='time')


DS_dic_all=DS_dic_all.rename_dims({'depth':'density'})
DS_dic_all=DS_dic_all.rename_vars({'depth':'density'})
print(DS_dic_all)

DS_prei_dic_all=DS_prei_dic_all.rename_dims({'depth':'density'})
DS_prei_dic_all=DS_prei_dic_all.rename_vars({'depth':'density'})

###########################################
# np

for year in range(styr,edyr+1):

    leap = calendar.isleap(year)
    print(year,leap)

    for mon in range(12):
        time_mon[mon] = datetime.datetime(year,mon+1,1)

    #####
    print( "Loading monthly data" )

    path_mon=path_np10km+'hst_mon-np'

    base_dic='hs_pac3_DIC_average_lyr.*'
    st_file = path_mon + '/y' + str(year) + '/' + base_dic
    DS_dic = xr.open_mfdataset(paths=st_file,concat_dim='time',combine='nested',decode_times=False)
    DS_dic = DS_dic.assign_coords({'time':time_mon})

    if (year==styr):
        DS_dic.loc[dict(time=time_mon[0])]=np.NaN

    if (year == styr):
        DS_dic_np = DS_dic.copy()
    else:
        DS_dic_np = xr.concat([DS_dic_np,DS_dic],dim='time')

    base_prei_dic='hs_pac3_DIC_prei_average_lyr.*'
    st_prei_file = path_mon + '/y' + str(year) + '/' + base_prei_dic
    DS_prei_dic = xr.open_mfdataset(paths=st_prei_file,concat_dim='time',combine='nested',decode_times=False)
    DS_prei_dic = DS_prei_dic.assign_coords({'time':time_mon})

    if (year==styr):
        DS_prei_dic.loc[dict(time=time_mon[0])]=np.NaN


    if (year == styr):
        DS_prei_dic_np = DS_prei_dic.copy()
    else:
        DS_prei_dic_np = xr.concat([DS_prei_dic_np,DS_prei_dic],dim='time')


DS_dic_np=DS_dic_np.rename_dims({'depth':'density'})
DS_dic_np=DS_dic_np.rename_vars({'depth':'density'})
print(DS_dic_np)

DS_prei_dic_np=DS_prei_dic_np.rename_dims({'depth':'density'})
DS_prei_dic_np=DS_prei_dic_np.rename_vars({'depth':'density'})

dens=DS_dic_all.coords["density"].values
print(dens)

#####################################

infl1 = path_glb100km+"./run-20211104o2/csv/Pac3_Cant_storage_lyr_GLB100km_1987-2017.csv"
df1=pd.read_csv(infl1,index_col=0)
df1.rename(columns=rem_dict_glb,inplace=True)

infl4 = path_np10km+"csv/Pac3_Cant_storage_lyr_NP10km_1987-2017.csv"
df4=pd.read_csv(infl4,index_col=0)
df4.rename(columns=rem_dict_np,inplace=True)

dfa=pd.concat([df1,df4],axis=1)

print(dfa)

#####################################

fig = plt.figure(figsize=(8.5,11))
#fig.suptitle( suptitle_sto, fontsize=18 )

proj = ccrs.PlateCarree(central_longitude=-160.)
lon_formatter = LongitudeFormatter(zero_direction_label=True)
lat_formatter = LatitudeFormatter()

fig_index = ["(a)", "(b)", "(d)", "(f)", "(h)", "(j)", "(c)", "(e)", "(g)", "(i)", "(k)",]

# [left, bottom, width, height]

ax = [ plt.axes([0.23,0.84,0.54,0.14],projection=proj),
       plt.axes([0.07,0.68,0.41,0.11]),
       plt.axes([0.07,0.52,0.41,0.11]),
       plt.axes([0.07,0.36,0.41,0.11]),
       plt.axes([0.07,0.20,0.41,0.11]),
       plt.axes([0.07,0.04,0.41,0.11]),
       plt.axes([0.56,0.68,0.41,0.11]),
       plt.axes([0.56,0.52,0.41,0.11]),
       plt.axes([0.56,0.36,0.41,0.11]),
       plt.axes([0.56,0.20,0.41,0.11]),
       plt.axes([0.56,0.04,0.41,0.11]), ]

dist=1.0
itrg=-9
for iden in range(len(dens)):
    if (abs(tdens-dens[iden]) < dist):
        itrg = iden
        dist = abs(tdens-dens[iden])

print("density = "+str(dens[itrg]))

#-----
title = '(a) Pacific region mask '
ca=ax[0].pcolormesh(lonb, latb, mask, cmap='tab20',transform=ccrs.PlateCarree())
ax[0].coastlines()
ax[0].text(145,35,'NW\nSubtropical',horizontalalignment='left',verticalalignment='center',fontsize=8,transform=ccrs.PlateCarree())
ax[0].text(145,20,'SW\nSubtropical',horizontalalignment='left',verticalalignment='center',fontsize=8,transform=ccrs.PlateCarree())
ax[0].text(200,28,'E-\nSubtropical',horizontalalignment='left',verticalalignment='center',fontsize=8,transform=ccrs.PlateCarree())
ax[0].set_xticks(np.arange(120,290.1,30),crs=ccrs.PlateCarree())
ax[0].set_yticks(np.arange(-20,66.1,10),crs=ccrs.PlateCarree())
ax[0].xaxis.set_major_formatter(lon_formatter)
ax[0].yaxis.set_major_formatter(lat_formatter)
ax[0].set_aspect(1.2)
ax[0].set_title(title,{'fontsize':9,})
ax[0].tick_params(labelsize=8)

nax=1
for reg in range(2):

    print("Canth inventory Region : " + item_list[tregion[reg]])

    ds_glb = DS_st_all[item_list[tregion[reg]]].sum(dim='density',skipna=False)
    ds_glb.plot.line(ax=ax[nax],x="time",label='LowRes',color='blue')
    ds_np = DS_st_np[item_list[tregion[reg]]].sum(dim='density',skipna=False)
    ds_np.plot.line(ax=ax[nax],x="time",label='HighRes',color='orange')
    ds_diff=ds_np-ds_glb
    ds_diff.plot.line(ax=ax[nax],x="time",label='High - Low',color='red')

    ax[nax].set_title(fig_index[nax] + r' C$_{\mathrm{anth}}$ column inventory ' + item_list[tregion[reg]],{'fontsize':9,})
    ax[nax].set_xlabel('')
    ax[nax].set_ylabel('[$\mathrm{PgC}$]',fontsize=8)
    ax[nax].set_ylim(inventrange_list[tregion[reg]][0],inventrange_list[tregion[reg]][1])
    ax[nax].legend(bbox_to_anchor=(0.01,1.0),loc='upper left',fontsize=7,)
    ax[nax].tick_params(labelsize=7)
    ax[nax].grid()

    nax+=1

    list2 = tregion[reg]
    dfb = dfa[[glb_list[list2],np_list[list2]]].copy(deep=True)
    rem_dict_bar = dict(zip([glb_list[list2],np_list[list2]],plot_list))
    dfb.rename(columns=rem_dict_bar,inplace=True)

    print("Canth storage Region : " + glb_list[list2] + " " + np_list[list2])

    dfb.iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
    ax[nax].set_title(fig_index[nax] + r' C$_{\mathrm{anth}}$ storage '+item_list[tregion[reg]]+ ' (1987-2017 mean)',{'fontsize':9})
    ax[nax].set_xlabel('')
    ax[nax].set_ylabel('[$\mathrm{PgC}$]',fontsize=8)
    ax[nax].set_ylim(0.0,0.8)
    ax[nax].set_xticklabels(dfa.index.values[0:10],rotation=45)
    ax[nax].tick_params(labelsize=7)
    ax[nax].legend(fontsize=7)
    #ax[nax].set_ylim(cnvrange_list[item][0],cnvrange_list[item][1])

    nax+=1

    # Canth storage

    print("Canth storage time serires Region : " + item_list[tregion[reg]] + " " +str(dens[itrg]))

    ds_glb = DS_st_all[item_list[tregion[reg]]].isel(density=itrg)
    ds_glb.plot.line(ax=ax[nax],x="time",label='LowRes')
    ds_np = DS_st_np[item_list[tregion[reg]]].isel(density=itrg)
    ds_np.plot.line(ax=ax[nax],x="time",label='HighRes')
    ds_diff=ds_np-ds_glb
    ds_diff.plot.line(ax=ax[nax],x="time",label='High - Low',color='red')

    ax[nax].set_title(fig_index[nax]+r" C$_{\mathrm{anth}}$ storage "+item_list[tregion[reg]]+r" $\sigma_0$="+str(dens[itrg]),{'fontsize':9})
    ax[nax].set_xlabel('')
    ax[nax].set_ylabel('[$\mathrm{PgC}$]',fontsize=8)
    ax[nax].set_ylim(storange_list[tregion[reg]][0],storange_list[tregion[reg]][1])
    ax[nax].legend(bbox_to_anchor=(0.01,1.0),loc='upper left',fontsize=8,)
    ax[nax].tick_params(labelsize=7)
    ax[nax].grid()

    nax+=1

    # volume
    ds_glb = DS_vl_all[item_list[tregion[reg]]].isel(density=itrg)
    ds_glb.plot.line(ax=ax[nax],x="time",label='LowRes')
    ds_np = DS_vl_np[item_list[tregion[reg]]].isel(density=itrg)
    ds_np.plot.line(ax=ax[nax],x="time",label='HighRes')
    ds_diff=ds_np-ds_glb
    ds_diff.plot.line(ax=ax[nax],x="time",label='High - Low',color='red')

    ax[nax].set_title(fig_index[nax]+" Layer volume " + item_list[tregion[reg]]+r" $\sigma_0$="+str(dens[itrg]),{'fontsize':9})
    ax[nax].set_xlabel('')
    ax[nax].set_ylabel('[$\mathrm{km}^{3}$]',fontsize=8)
    ax[nax].set_ylim(volrange_list[tregion[reg]][0],volrange_list[tregion[reg]][1])
    ax[nax].legend(bbox_to_anchor=(1.0,0.0),loc='lower right',ncol=3,fontsize=8,)
    ax[nax].tick_params(labelsize=7)
    ax[nax].grid()

    nax+=1

    # Canth concentration
    ds_glb = DS_dic_all[item_list[tregion[reg]]].isel(density=itrg)
    ds_np = DS_dic_np[item_list[tregion[reg]]].isel(density=itrg)
    ds_prei_glb = DS_prei_dic_all[item_list[tregion[reg]]].isel(density=itrg)
    ds_prei_np = DS_prei_dic_np[item_list[tregion[reg]]].isel(density=itrg)
    ds_diff_glb=(ds_glb-ds_prei_glb)*1e3
    ds_diff_glb.plot.line(ax=ax[nax],x="time",label='LowRes',color='royalblue')
    ds_diff_np=(ds_np-ds_prei_np)*1e3
    ds_diff_np.plot.line(ax=ax[nax],x="time",label='HighRes',color='darkorange')
    ds_diff=ds_diff_np-ds_diff_glb
    ds_diff.plot.line(ax=ax[nax],x="time",label='High - Low',color='red')

    ax[nax].set_title(fig_index[nax]+r" C$_{\mathrm{anth}}$ concentration " + item_list[tregion[reg]]+r" $\sigma_0$="+str(dens[itrg]),{'fontsize':9})
    ax[nax].set_xlabel('')
    ax[nax].set_ylabel('[$\mathrm{mmol} \mathrm{m}^{-3}$]',fontsize=8)
    ax[nax].set_ylim(dic_diff_list[tregion[reg]][0],dic_diff_list[tregion[reg]][1])
    ax[nax].set_yticks(np.arange(dic_diff_list[tregion[reg]][0],dic_diff_list[tregion[reg]][1]+0.1,10))
    ax[nax].legend(bbox_to_anchor=(0.01,1.0),loc='upper left',fontsize=8,)
    ax[nax].tick_params(labelsize=7)
    ax[nax].grid()

    nax+=1

#plt.tight_layout(rect=[0,0,1,0.95])

outfile = 'fig/Canth_storage_time_mon_'+fig_list[tregion[0]]+'-'+fig_list[tregion[1]]+'_'+str(styr)+'-'+str(edyr)+'_v2'
outpng=outfile+'.png'
outpdf=outfile+'.pdf'
plt.savefig(outpng, bbox_inches='tight', pad_inches=0.1)
plt.savefig(outpdf, bbox_inches='tight', pad_inches=0.1)
plt.show()
plt.close()
