# -*- coding: utf-8 -*-
import sys
import json
import numpy as np
import netCDF4
import xarray as xr
import cartopy.crs as ccrs
import matplotlib as mpl
import matplotlib.pyplot as plt
from cartopy.mpl.ticker import LongitudeFormatter, LatitudeFormatter
import datetime

########
ro = 1024.5

if (len(sys.argv) < 2):
    print ('Usage: ' + sys.argv[0] + ' start_year end_year')
    sys.exit()

ystr = int(sys.argv[1])
yend = int(sys.argv[2])
nyr = yend-ystr+1

mon1 = 2
mon2 = 8

suptitle = r'Surface C$_{\mathrm{anth}}$ concentration: Ann Feb Aug (ave. from '+str(ystr)+' to '+str(yend)+')'
outfile = './fig/Cant_surf_comp_ann_feb_aug_v2'

print("Drawing "+outfile)

####################################################################

time = np.empty(yend-ystr+1,dtype='object')
for yr in range(ystr,yend+1):
    time[yr-ystr] = datetime.datetime(yr,1,1)

####################################################################
path_glb100km='../data/glb100km/'
path_np10km='../data/np10km/'
####################################################################
# GLB100km

print( "Loading GLB100km data" )

path_g100_mon=path_glb100km + 'run-20211104o2/ANALY/monthly'

base_dic_g100 = 'hs_sfc_DIC_woa1x1'
base_dic_prei_g100 = 'hs_sfc_DIC_prei_woa1x1'

suffix = '.' + '{:04}'.format(ystr)
dic_file = path_g100_mon + '/' + base_dic_g100 + suffix
ncg100 = netCDF4.Dataset(dic_file,'r')
lon_mod = ncg100.variables['lon'][:]
lat_mod = ncg100.variables['lat'][:]
nx = len(ncg100.dimensions['lon'])
ny = len(ncg100.dimensions['lat'])
ncg100.close()

dic_tmp = np.array(np.empty((12,ny,nx)),dtype=np.float64)
dic_prei_tmp = np.array(np.empty((12,ny,nx)),dtype=np.float64)

dic_ann = np.array(np.empty((ny,nx)),dtype=np.float64)
dic_prei_ann = np.array(np.empty((ny,nx)),dtype=np.float64)
dic_feb = np.array(np.empty((ny,nx)),dtype=np.float64)
dic_prei_feb = np.array(np.empty((ny,nx)),dtype=np.float64)
dic_aug = np.array(np.empty((ny,nx)),dtype=np.float64)
dic_prei_aug = np.array(np.empty((ny,nx)),dtype=np.float64)

cant_g100_ann = np.array(np.empty((nyr,ny,nx)),dtype=np.float64)
cant_g100_feb = np.array(np.empty((nyr,ny,nx)),dtype=np.float64)
cant_g100_aug = np.array(np.empty((nyr,ny,nx)),dtype=np.float64)

for yr in range(ystr,yend+1):

    suffix = '.' + '{:04}'.format(yr)

    dic_file = path_g100_mon + '/' + base_dic_g100 + suffix
    ncg100 = netCDF4.Dataset(dic_file,'r')
    dic_tmp[:,:,:] = ncg100.variables['dissicos'][:,:,:]
    ncg100.close()
    dic_tmp = np.where(dic_tmp < -9.0e33, np.NaN, dic_tmp)

    dic_prei_file = path_g100_mon + '/' + base_dic_prei_g100 + suffix
    ncg100 = netCDF4.Dataset(dic_prei_file,'r')
    dic_prei_tmp[:,:,:] = ncg100.variables['dissicnatos'][:,:,:]
    ncg100.close()
    dic_prei_tmp = np.where(dic_prei_tmp < -9.0e33, np.NaN, dic_prei_tmp)

    dic_ann[:,:] = np.nanmean(dic_tmp,axis=0)
    dic_prei_ann[:,:] = np.nanmean(dic_prei_tmp,axis=0)

    dic_feb = dic_tmp[mon1-1]
    dic_prei_feb = dic_prei_tmp[mon1-1]

    dic_aug = dic_tmp[mon2-1]
    dic_prei_aug = dic_prei_tmp[mon2-1]

    cant_g100_ann[yr-ystr,:,:] = (dic_ann - dic_prei_ann)*1e3
    cant_g100_feb[yr-ystr,:,:] = (dic_feb - dic_prei_feb)*1e3
    cant_g100_aug[yr-ystr,:,:] = (dic_aug - dic_prei_aug)*1e3


DS_g100_ann = xr.Dataset( {'cant': (['time','lat','lon'], cant_g100_ann),},
                          coords = { 'time': time,'lat': lat_mod,'lon': lon_mod, } )

DS_g100_feb = xr.Dataset( {'cant': (['time','lat','lon'], cant_g100_feb),},
                          coords = { 'time': time,'lat': lat_mod,'lon': lon_mod, } )

DS_g100_aug = xr.Dataset( {'cant': (['time','lat','lon'], cant_g100_aug),},
                          coords = { 'time': time,'lat': lat_mod,'lon': lon_mod, } )

####################################################################
# NP10km

print( "Loading NP10km data" )

path_fill=path_np10km + 'hst_mon-glb_fill'

base_dic_fill = 'hs_sfc_DIC_woa1x1'
base_dic_prei_fill = 'hs_sfc_DIC_prei_woa1x1'

suffix = '.' + '{:04}{:02}'.format(ystr,mon1)
dic_file = path_fill + '/y' + '{:04}'.format(ystr) + '/' + base_dic_fill + suffix
ncfill = netCDF4.Dataset(dic_file,'r')
lon_mod = ncfill.variables['lon'][:]
lat_mod = ncfill.variables['lat'][:]
nx = len(ncfill.dimensions['lon'])
ny = len(ncfill.dimensions['lat'])
ncfill.close()

dic_tmp = np.array(np.empty((12,ny,nx)),dtype=np.float64)
dic_prei_tmp = np.array(np.empty((12,ny,nx)),dtype=np.float64)

dic_ann = np.array(np.empty((ny,nx)),dtype=np.float64)
dic_prei_ann = np.array(np.empty((ny,nx)),dtype=np.float64)

dic_feb = np.array(np.empty((ny,nx)),dtype=np.float64)
dic_prei_feb = np.array(np.empty((ny,nx)),dtype=np.float64)

dic_aug = np.array(np.empty((ny,nx)),dtype=np.float64)
dic_prei_aug = np.array(np.empty((ny,nx)),dtype=np.float64)

cant_fill_ann = np.array(np.empty((nyr,ny,nx)),dtype=np.float64)
cant_fill_feb = np.array(np.empty((nyr,ny,nx)),dtype=np.float64)
cant_fill_aug = np.array(np.empty((nyr,ny,nx)),dtype=np.float64)

for yr in range(ystr,yend+1):

    for month in range(1,13):

        suffix = '.' + '{:04}{:02}'.format(yr,month)

        dic_file = path_fill + '/y' + '{:04}'.format(yr) + '/' + base_dic_fill + suffix
        ncfill = netCDF4.Dataset(dic_file,'r')
        dic_tmp[month-1,:,:] = ncfill.variables['dissic'][0,:,:]
        ncfill.close()

        dic_prei_file = path_fill + '/y' + '{:04}'.format(yr) + '/' + base_dic_prei_fill + suffix
        ncfill = netCDF4.Dataset(dic_prei_file,'r')
        dic_prei_tmp[month-1,:,:] = ncfill.variables['dissicnat'][0,:,:]
        ncfill.close()

    dic_tmp = np.where(dic_tmp < -9.0e33, np.NaN, dic_tmp)
    dic_prei_tmp = np.where(dic_prei_tmp < -9.0e33, np.NaN, dic_prei_tmp)

    dic_ann[:,:] = np.nanmean(dic_tmp,axis=0)
    dic_prei_ann[:,:] = np.nanmean(dic_prei_tmp,axis=0)

    dic_feb = dic_tmp[mon1-1]
    dic_prei_feb = dic_prei_tmp[mon1-1]

    dic_aug = dic_tmp[mon2-1]
    dic_prei_aug = dic_prei_tmp[mon2-1]

    cant_fill_ann[yr-ystr,:,:] = (dic_ann - dic_prei_ann)*1e3
    cant_fill_feb[yr-ystr,:,:] = (dic_feb - dic_prei_feb)*1e3
    cant_fill_aug[yr-ystr,:,:] = (dic_aug - dic_prei_aug)*1e3


DS_fill_ann = xr.Dataset( {'cant': (['time','lat','lon'], cant_fill_ann),},
                          coords = { 'time': time, 'lat': lat_mod,'lon': lon_mod, } )

DS_fill_feb = xr.Dataset( {'cant': (['time','lat','lon'], cant_fill_feb),},
                          coords = { 'time': time, 'lat': lat_mod,'lon': lon_mod, } )

DS_fill_aug = xr.Dataset( {'cant': (['time','lat','lon'], cant_fill_aug),},
                          coords = { 'time': time, 'lat': lat_mod,'lon': lon_mod, } )

#print(DS_fill_ann)

####################################################################

print( 'Start drawing' )
fig = plt.figure(figsize=(11,8))
fig.suptitle( suptitle, fontsize=16 )

proj = ccrs.PlateCarree(central_longitude=-140.)
lon_formatter = LongitudeFormatter(zero_direction_label=True)
lat_formatter = LatitudeFormatter()


ax = [
    plt.subplot(3,3,1,projection=proj),
    plt.subplot(3,3,2,projection=proj),
    plt.subplot(3,3,3,projection=proj),
    plt.subplot(3,3,4,projection=proj),
    plt.subplot(3,3,5,projection=proj),
    plt.subplot(3,3,6,projection=proj),
    plt.subplot(3,3,7,projection=proj),
    plt.subplot(3,3,8,projection=proj),
    plt.subplot(3,3,9,projection=proj),
]

# [left, bottom, width, height]

ax_cbar = [
    plt.axes([0.06,0.65,0.30,0.015]),
    plt.axes([0.38,0.65,0.30,0.015]),
    plt.axes([0.70,0.65,0.30,0.015]),
    plt.axes([0.06,0.35,0.30,0.015]),
    plt.axes([0.38,0.35,0.30,0.015]),
    plt.axes([0.70,0.35,0.30,0.015]),
    plt.axes([0.06,0.05,0.30,0.015]),
    plt.axes([0.38,0.05,0.30,0.015]),
    plt.axes([0.70,0.05,0.30,0.015]),
]

bounds1 = [20.0, 25.0, 30.0, 35.0, 40.0, 45.0, 50.0, 55.0, 60.0, 65.0, 70.0]
bounds2 = [-10.0, -7.0, -4.0, -2.0, -1.0, -0.5, -0.2, 0.2, 0.5, 1.0, 2.0, 4.0, 7.0, 10.0]
#bounds3 = [0.0, 1.0, 2.0, 5.0, 10.0, 15.0, 20.0, 30.0, 40.0, 50.0, 70.0, 100.0]

cmap = [ 'YlOrRd', 'YlOrRd', 'bwr' ]
mod = [ 'Low Resolution', 'High Resolution', 'HighRes - LowRes' ]

panel=0

for item in range(3):
    for model in range(3):

        if (item == 0):
            title = mod[model] +' annual mean'
            if (model == 0):
                da = DS_g100_ann.cant.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)
            if (model == 1):
                da = DS_fill_ann.cant.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)
            if (model == 2):
                da = DS_fill_ann.cant.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)  \
                     - DS_g100_ann.cant.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)
        elif (item == 1):
            title = mod[model] +' month = ' + str(mon1)
            if (model == 0):
                da = DS_g100_feb.cant.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)
            if (model == 1):
                da = DS_fill_feb.cant.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)
            if (model == 2):
                da = DS_fill_feb.cant.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)  \
                     - DS_g100_feb.cant.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)
        else:
            title = mod[model] +' month = ' + str(mon2)
            if (model == 0):
                da = DS_g100_aug.cant.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)
            if (model == 1):
                da = DS_fill_aug.cant.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)
            if (model == 2):
                da = DS_fill_aug.cant.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)  \
                     - DS_g100_aug.cant.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=True)

        print("Drawing panel "+ str(panel) + ' ' + title)

        units = '[$\mathrm{mmol}\, \mathrm{m}^{-3}$]'

        if (model == 0 or model == 1):
            bounds = bounds1
            ticks_bounds = bounds1
        elif (model == 1):
            bounds = bounds1
            ticks_bounds = bounds1
        elif (model == 2):
            bounds = bounds2
            ticks_bounds = bounds2

        im = da.plot(ax=ax[panel],cmap=cmap[model],
                     levels=bounds,
                     extend='both',
                     add_colorbar=False,
                     transform=ccrs.PlateCarree())

        cbar = plt.colorbar(im,orientation='horizontal',spacing='uniform',label=units,
                            ticks=ticks_bounds, cax=ax_cbar[panel])
        cbar.set_label(label=units,fontsize=6,x=0.90)
        cbar.ax.tick_params(labelsize=7)

        ax[panel].coastlines()
        ax[panel].set_xticks(np.arange(120,285.1,30),crs=ccrs.PlateCarree())
        ax[panel].set_yticks(np.arange(-10,66.1,10),crs=ccrs.PlateCarree())
        ax[panel].set_extent([99,286,-16,66],crs=ccrs.PlateCarree())
        ax[panel].set_aspect(1.1)
        ax[panel].xaxis.set_major_formatter(lon_formatter)
        ax[panel].yaxis.set_major_formatter(lat_formatter)
        ax[panel].set_xlabel('')
        ax[panel].set_title(title,{'fontsize':10, 'verticalalignment':'top'})
        ax[panel].tick_params(labelsize=9)
        ax[panel].patch.set_facecolor('lightgray')

        panel+=1

plt.subplots_adjust(left=0.07,right=0.98,bottom=0.06,top=0.93,wspace=0.16,hspace=0.15)

outpdf = outfile+'.pdf'
outpng = outfile+'.png'

plt.savefig(outpng, bbox_inches='tight', pad_inches=0.1)
plt.savefig(outpdf, bbox_inches='tight', pad_inches=0.1)

print("figure is saved to " + outpng + " and " + outpdf)

plt.show()
