# -*- coding: utf-8 -*-
import sys
import os
import json
import netCDF4
from netCDF4 import Dataset, num2date
import numpy as np
import pandas as pd
import cartopy.crs as ccrs
import matplotlib.pyplot as plt
from cartopy.mpl.ticker import LongitudeFormatter, LatitudeFormatter
import datetime
import xarray as xr

######

outfile = './fig/FgCO2_integ_spinup_2d_comp_gcb2022'
suptitle = 'Global ocean surface carbon uptake (positive into ocean)'

title = [ '(a) Spinup (GLB100km)', '(b) 1935-2018 ',
          '(c) GLB100km', '(d) NP10km',
          '(e) NP10km - GLB100km', '(f) JMA-MLR' ]

nyc=366
dtime=np.arange(2020-nyc*5-1,2020,1)
#print(dtime)

####################################################################

path_glb100km='../data/glb100km/'
path_iavf=path_glb100km + 'run-20211104o2/logs_org/MONIT/annual/'
path_5th=path_glb100km + 'run-20211104o2/logs_org/MONIT/annual/'
path_4th=path_glb100km + 'run-20211023o2/logs_org/MONIT/annual/'
path_3rd=path_glb100km + 'run-20211012o2/logs_org/MONIT/annual/'
path_2nd=path_glb100km + 'run-20210930o2/logs_org/MONIT/annual/'
path_1st=path_glb100km + 'run-20210919o2/logs_org/MONIT/annual/'
path_0th=path_glb100km + 'run-20210712o2/logs_org/MONIT/annual/'

fgco2_iav=np.empty(nyc*5+1,dtype=np.float64)
fgco2_iav[0:nyc*5+1]=np.nan

fgco2_all=np.empty(nyc*5+1,dtype=np.float64)
fgco2_all[0:nyc*5+1]=np.nan

fgco2_ref=np.empty(nyc*5+1,dtype=np.float64)
fgco2_ref[0:nyc*5+1]=np.nan

fgco2_iavf=np.empty(nyc,dtype=np.float64)
fgco2_5th=np.empty(nyc,dtype=np.float64)
fgco2_4th=np.empty(nyc,dtype=np.float64)
fgco2_3rd=np.empty(nyc,dtype=np.float64)
fgco2_2nd=np.empty(nyc,dtype=np.float64)
fgco2_1st=np.empty(nyc,dtype=np.float64)

fgco2_0th=np.empty(nyc,dtype=np.float64)

fgco2_iavf[0:nyc]=np.nan
fgco2_5th[0:nyc]=np.nan
fgco2_4th[0:nyc]=np.nan
fgco2_3rd[0:nyc]=np.nan
fgco2_2nd[0:nyc]=np.nan
fgco2_1st[0:nyc]=np.nan

fgco2_0th[0:nyc]=np.nan

for year in range(1653,2019):

    file_fgco2_5th = path_5th + 'hs_FgDIC_prei_integ.{yr:04}'.format(yr=year)
    if (os.path.isfile(file_fgco2_5th)):
        f3 = open(file_fgco2_5th,'rb')
        fgco2_tmp = np.fromfile(f3, dtype = '>f', count = 1)
        fgco2_5th[year-1653] = fgco2_tmp[0]
        f3.close()

    file_fgco2_4th = path_4th + 'hs_FgDIC_integ.{yr:04}'.format(yr=year)
    if (os.path.isfile(file_fgco2_4th)):
        f3 = open(file_fgco2_4th,'rb')
        fgco2_tmp = np.fromfile(f3, dtype = '>f', count = 1)
        fgco2_4th[year-1653] = fgco2_tmp[0]
        f3.close()

    file_fgco2_3rd = path_3rd + 'hs_FgDIC_integ.{yr:04}'.format(yr=year)
    if (os.path.isfile(file_fgco2_3rd)):
        f3 = open(file_fgco2_3rd,'rb')
        fgco2_tmp = np.fromfile(f3, dtype = '>f', count = 1)
        fgco2_3rd[year-1653] = fgco2_tmp[0]
        f3.close()

    file_fgco2_2nd = path_2nd + 'hs_FgDIC_integ.{yr:04}'.format(yr=year)
    if (os.path.isfile(file_fgco2_2nd)):
        f3 = open(file_fgco2_2nd,'rb')
        fgco2_tmp = np.fromfile(f3, dtype = '>f', count = 1)
        fgco2_2nd[year-1653] = fgco2_tmp[0]
        f3.close()

    file_fgco2_1st = path_1st + 'hs_FgDIC_integ.{yr:04}'.format(yr=year)
    if (os.path.isfile(file_fgco2_1st)):
        f3 = open(file_fgco2_1st,'rb')
        fgco2_tmp = np.fromfile(f3, dtype = '>f', count = 1)
        fgco2_1st[year-1653] = fgco2_tmp[0]
        f3.close()

    file_fgco2_0th = path_0th + 'hs_FgDIC_prei_integ.{yr:04}'.format(yr=year)
    if (os.path.isfile(file_fgco2_0th)):
        f3 = open(file_fgco2_0th,'rb')
        fgco2_tmp = np.fromfile(f3, dtype = '>f', count = 1)
        fgco2_0th[year-1653] = fgco2_tmp[0]
        f3.close()

    file_fgco2_iavf = path_iavf + 'hs_FgDIC_integ.{yr:04}'.format(yr=year)
    if (os.path.isfile(file_fgco2_iavf)):
        f3 = open(file_fgco2_iavf,'rb')
        fgco2_tmp = np.fromfile(f3, dtype = '>f', count = 1)
        fgco2_iavf[year-1653] = fgco2_tmp[0]
        iyok=year
        f3.close()

###############

fgco2_ref[nyc*0:nyc*1]=fgco2_0th[0:nyc]

fgco2_all[nyc*0:nyc*1]=fgco2_1st[0:nyc]
fgco2_all[nyc*1:nyc*2]=fgco2_2nd[0:nyc]
fgco2_all[nyc*2:nyc*3]=fgco2_3rd[0:nyc]
fgco2_all[nyc*3:nyc*4]=fgco2_4th[0:nyc]
fgco2_all[nyc*4:nyc*5]=fgco2_5th[0:nyc]
fgco2_iav[nyc*4:nyc*5]=fgco2_iavf[0:nyc]

col = pd.Index([r'constant $x\mathrm{CO}_2$'],name='run')
fgco2_m0 = pd.DataFrame(fgco2_all,index=dtime,columns=col)
col = pd.Index([r'increasing $x\mathrm{CO}_2$'],name='run')
fgco2_m1 = pd.DataFrame(fgco2_iav,index=dtime,columns=col)

col = pd.Index([r'constant $x\mathrm{CO}_2$ (sinking = 7$\,\mathrm{m}\,\mathrm{d}^{-1}$)'],name='run')
fgco2_m2 = pd.DataFrame(fgco2_ref,index=dtime,columns=col)

fgco2_df = pd.concat([fgco2_m0,fgco2_m1,fgco2_m2],axis=1)
#################

yst=1653
yed=2018
nall=yed-yst+2

dtime_c=np.arange(yst,yed+2,1)
print(dtime_c)

####################################################################

print( "Loading GRID data" )

# areacell

path_ref = '../data/refdata/'
arefile = path_ref + 'PCMDI-SST/areacello_input4MIPs_SSTsAndSeaIce_CMIP_PCMDI-AMIP-1-1-6_gn.nc'
ncare = netCDF4.Dataset(arefile,'r')
nx = len(ncare.dimensions['lon'])
ny = len(ncare.dimensions['lat'])
area = ncare.variables['areacello'][:,:]
ncare.close()

# basin index

basinfile = path_ref + 'WOA13v2/1deg_L33/const/WOA13_1x1_mask.nc'
ncbas = netCDF4.Dataset(basinfile,'r')
basin = ncbas.variables['basin_mask'][:,:]
ncbas.close()

maskglb = np.array(np.zeros((ny,nx)),dtype=np.float64)
for j in range(ny):
    for i in range(nx):
        if (basin[j,i] < 0 or basin[j,i] == 53):
            maskglb[j,i] = 0
        else:
            maskglb[j,i] = 1

####################################################################
# Ref data
#
#path_mpi = path_ref + 'MPI-SOM_FFN'
#reffile = path_mpi + '/0191304/2.2/data/0-data/MPI-SOM_FFN_SOCCOMv2018.nc'
#print( "Loading Landschutzer(2018) data" )
#time_ref = np.empty((2017-1982+1)*12,dtype='object')
#for yr in range(1982,2018):
#    for mon in range(1,13):
#        time_ref[(yr-1982)*12+mon-1] = datetime.datetime(yr,mon,1)
#
#ncref = netCDF4.Dataset(reffile,'r')
#lon = ncref.variables['lon'][:]
#lat = ncref.variables['lat'][:]
#nx = len(ncref.dimensions['lon'])
#ny = len(ncref.dimensions['lat'])
#fgco2_ref = ncref.variables['fgco2_smoothed'][:,:,:]
#ncref.close()
#
#fgco2_ref = np.where(fgco2_ref > 0.9e20, np.NaN, -fgco2_ref)
#
#DS_ref = xr.Dataset( {'fgco2': (['time','lat','lon'], fgco2_ref),},
#                      coords = { 'time': time_ref,'lat': lat,'lon': lon, } )
#
#DS_ref_ann = DS_ref.resample(time='1YS').mean()
#
##print(DS_ref_ann.fgco2)
#
#DS_ref_ann = DS_ref_ann.assign_coords(lon=('lon', np.where( DS_ref_ann.lon < 0, DS_ref_ann.lon + 360, DS_ref_ann.lon )))
#DS_ref_ann = DS_ref_ann.roll(lon=-180, roll_coords=True)
#
#print(DS_ref_ann)
#
####################################################################
# Ref data (JMA)
#
path_jma = path_ref + 'JMA_CO2flux'
print( "Loading JMA data" )

fgco2_jma = np.array(np.empty(((2018-1990+1)*12,ny,nx)),dtype=np.float64)
fgco2_tmp = np.array(np.empty((12,ny,nx)),dtype=np.float64)

time_ref = np.empty((2018-1990+1)*12,dtype='object')
for yr in range(1990,2019):
    for mon in range(1,13):
        time_ref[(yr-1990)*12+mon-1] = datetime.datetime(yr,mon,1)

    reffile = path_jma + '/JMA_co2map_' + str(yr) + '.nc'
    ncref = netCDF4.Dataset(reffile,'r')
    if (yr == 1990):
        lon = ncref.variables['lon'][:]
        lat = ncref.variables['lat'][:]
        nx = len(ncref.dimensions['lon'])
        ny = len(ncref.dimensions['lat'])
    fgco2_tmp = ncref.variables['flux'][:,:,:]
    ncref.close()

    smon = (yr-1990)*12
    emon = (yr-1990+1)*12
    fgco2_jma[smon:emon,:,:] = -fgco2_tmp[0:12,:,:]

fgco2_jma = np.where(fgco2_jma > 900.0, np.NaN, fgco2_jma)

DS_jma = xr.Dataset( {'fgco2': (['time','lat','lon'], fgco2_jma),},
                      coords = { 'time': time_ref,'lat': lat,'lon': lon, } )

DS_jma_ann = DS_jma.resample(time='1YS').mean()

#fgco2_gj=np.empty(nall,dtype=np.float64)
#fgco2_gj[0:nall]=np.nan
#
#for yr in range(1990,2019):
#
#    da = DS_jma_ann.fgco2.sel(time=str(yr))
#    print(da)
#    msktmp = np.where( np.isnan(da.values), 0.0, 1.0 )
#    datmp = np.where( np.isnan(da.values), 0.0, da.values )
#    tmp1 = (datmp * area * msktmp * maskglb).sum()
#    fgco2_gj[yr-yst] = tmp1 * 12 * 1e-15 # mol yr-1 -> PgC yr-1
#
#col = pd.Index(['JMA'],name='model')
#fgco2_glb_jma = pd.DataFrame(fgco2_gj,index=dtime_c,columns=col)

###############
# Global Carbon Budget

# GCB 2019
#file_gcb = path_ref + 'GlobalCarbonBudget/Ocean_sink_2019v1.0.nc'
#ncgcb = netCDF4.Dataset(file_gcb,'r')
#nd = len(ncgcb.dimensions['year'])
#fgco2_l = ncgcb.variables['Landschutzer'][:]
#fgco2_r = ncgcb.variables['Rodenbeck'][:]
#fgco2_c = ncgcb.variables['CMEMS'][:]
#time_gcb = ncgcb.variables['year'][:]
#ncgcb.close()

#for nyr in range(nd):
#    print(time_gcb[nyr])

#col = pd.Index(['Landschützer'],name='model')
#fgco2_lds = pd.DataFrame(fgco2_l,index=time_gcb,columns=col)
#col = pd.Index(['Rödenbeck'],name='model')
#fgco2_rds = pd.DataFrame(fgco2_r,index=time_gcb,columns=col)
#col = pd.Index(['CMEMS'],name='model')
#fgco2_cds = pd.DataFrame(fgco2_c,index=time_gcb,columns=col)

# GCB 2022
file_gcb = path_ref + 'GlobalCarbonBudget/Ocean_sink_2022v1.0.nc'
ncgcb = netCDF4.Dataset(file_gcb,'r')
nd = len(ncgcb.dimensions['year'])
fgco2_pmn = ncgcb.variables['Average_data-products_excl_Watson'][:]
fgco2_psd = ncgcb.variables['sd_data-products_excl_Watson'][:]
fgco2_mmn = ncgcb.variables['Multi-model_mean'][:]
fgco2_msd = ncgcb.variables['Model_Spread_sd'][:]
time_gcb = ncgcb.variables['year'][:]
ncgcb.close()

for nyr in range(nd):
    print(time_gcb[nyr])

col = pd.Index(['products_mean'],name='model')
fgco2_pmea = pd.DataFrame(fgco2_pmn,index=time_gcb,columns=col)
col = pd.Index(['products_min'],name='model')
fgco2_pmin = pd.DataFrame(fgco2_pmn-fgco2_psd,index=time_gcb,columns=col)
col = pd.Index(['products_max'],name='model')
fgco2_pmax = pd.DataFrame(fgco2_pmn+fgco2_psd,index=time_gcb,columns=col)

col = pd.Index(['multimodel_mean'],name='model')
fgco2_mmea = pd.DataFrame(fgco2_mmn,index=time_gcb,columns=col)
col = pd.Index(['multimodel_min'],name='model')
fgco2_mmin = pd.DataFrame(fgco2_mmn-fgco2_msd,index=time_gcb,columns=col)
col = pd.Index(['multimodel_max'],name='model')
fgco2_mmax = pd.DataFrame(fgco2_mmn+fgco2_msd,index=time_gcb,columns=col)


###############
# Last 80 years

path_np10km='../data/np10km/'
path_glb=path_np10km + 'hst_ann-glb'
path_np=path_np10km + 'hst_ann-np'

fgco2_g100=np.empty(nall,dtype=np.float64)
fgco2_fill=np.empty(nall,dtype=np.float64)

fgco2_g100[0:nall]=np.nan
fgco2_fill[0:nall]=np.nan

#for year in range(ystr,yend+1):
for year in range(yst,yed+1):

    file_fgco2_g100 = path_iavf + 'hs_FgDIC_integ.{yr:04}'.format(yr=year)
    if (os.path.isfile(file_fgco2_g100)):
        print('Reading from '+ file_fgco2_g100)
        f3 = open(file_fgco2_g100,'rb')
        fgco2_all = np.fromfile(f3, dtype = '>f', count = 1)
        fgco2_g100[year-yst] = fgco2_all[0]
        f3.close()

    file_fgco2_glb = path_glb + '/y' + str(year) + '/hs_FgDIC_integ.{yr:04}'.format(yr=year)
    file_fgco2_np = path_np +  '/y' + str(year) + '/hs_FgDIC_integ.{yr:04}'.format(yr=year)

    if (year > 1937 and os.path.isfile(file_fgco2_glb) and os.path.isfile(file_fgco2_np)):
        print('Reading from '+ file_fgco2_glb)
        f3 = open(file_fgco2_glb,'rb')
        fgco2_glb = np.fromfile(f3, dtype = '>f', count = 1)
        f3.close()

        print('Reading from '+ file_fgco2_np)
        f3 = open(file_fgco2_np,'rb')
        fgco2_np = np.fromfile(f3, dtype = '>f', count = 1)
        f3.close()

        fgco2_fill[year-yst] = (fgco2_glb[0] + fgco2_np[0])*1e-4*86400*365.25*1e3*1e-15

###############

col = pd.Index(['GLB100km'],name='model')
fgco2_om1 = pd.DataFrame(fgco2_g100,index=dtime_c,columns=col)
col = pd.Index(['NP10km'],name='model')
fgco2_om2 = pd.DataFrame(fgco2_fill,index=dtime_c,columns=col)

fgco2_dfm = pd.concat([fgco2_om1,fgco2_om2],axis=1)

#fgco2_dfo_tmp = pd.concat([fgco2_lds,fgco2_rds,fgco2_cds,fgco2_glb_jma],axis=1)
#col = pd.Index(['Products-mean'],name='model')
#fgco2_dfo_mean = fgco2_dfo_tmp.mean(axis=1)
#fgco2_dfo_mean = pd.DataFrame(fgco2_dfo_mean,columns=col)
#fgco2_dfo = pd.concat([fgco2_dfo_tmp,fgco2_dfo_mean],axis=1)
fgco2_dfp = pd.concat([fgco2_pmin,fgco2_pmax,fgco2_pmea],axis=1)
fgco2_dfo = pd.concat([fgco2_mmin,fgco2_mmax,fgco2_mmea],axis=1)

print (fgco2_dfm)
print (fgco2_dfp)
print (fgco2_dfo)

####################################################################
# 2-d data

ystr=1990
yend=2018
#ystr=1987
#yend=2017

nyr=yend-ystr+1

time = np.empty(nyr,dtype='object')
for yr in range(ystr,yend+1):
    time[yr-ystr] = datetime.datetime(yr,1,1)

print(time)
fgco2_tmp_2d = np.array(np.empty((ny,nx)),dtype=np.float64)

####################################################################
# GLB100km

print( "Loading GLB100km data" )

path_g100_2d=path_glb100km + 'run-20211104o2/ANALY/annual'
fgco2_g100_2d = np.array(np.empty((nyr,ny,nx)),dtype=np.float64)
basename_g100 = 'hs_FgDIC_woa1x1'

for yr in range(ystr,yend+1):
    suffix = '.' + '{:04}'.format(yr)
    fgco2_file_2d = path_g100_2d + '/' + basename_g100 + suffix
    ncg100 = netCDF4.Dataset(fgco2_file_2d,'r')
    lon = ncg100.variables['lon'][:]
    lat = ncg100.variables['lat'][:]
    fgco2_tmp_2d[:,:] = ncg100.variables['fgco2'][0,:,:]
    fgco2_g100_2d[yr-ystr,:,:] = fgco2_tmp_2d[:,:]

fgco2_g100_2d = np.where(fgco2_g100_2d < -9.0e33, np.NaN, fgco2_g100_2d * 365.25 * 86400 * 1e3 / 12) # gC/m2/sec --> mol/m2/yr
DS_g100_ann = xr.Dataset( {'fgco2': (['time','lat','lon'], fgco2_g100_2d),},
                          coords = { 'time': time,'lat': lat,'lon': lon, } )

####################################################################
# NP10km

print( "Loading NP10km data" )

path_fill_2d=path_np10km + 'hst_ann-glb_fill'

fgco2_fill_2d = np.array(np.empty((nyr,ny,nx)),dtype=np.float64)
basename_fill = 'hs_FgDIC_woa1x1'
for yr in range(ystr,yend+1):
    suffix = '.' + '{:04}'.format(yr)
    fgco2_file_2d = path_fill_2d + '/y' + '{:04}'.format(yr) + '/' + basename_fill + suffix
    #print(fgco2_file_2d)
    ncfill = netCDF4.Dataset(fgco2_file_2d,'r')
    lon = ncfill.variables['lon'][:]
    lat = ncfill.variables['lat'][:]
    fgco2_tmp_2d[:,:] = ncfill.variables['fgco2'][0,:,:]
    fgco2_fill_2d[yr-ystr,:,:] = fgco2_tmp_2d[:,:]

fgco2_fill_2d = np.where(fgco2_fill_2d < -9.0e33, np.NaN, fgco2_fill_2d * 365.25 * 86400 * 1e3 / 12)
DS_fill_ann = xr.Dataset( {'fgco2': (['time','lat','lon'], fgco2_fill_2d),},
                          coords = { 'time': time,'lat': lat,'lon': lon, } )

#print(DS_fill_ann)

####################################################################

#bias = np.empty( (4,180,360) )
#
#da0 = DS_jma_ann.fgco2.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=False)
#
#da1 = DS_g100_ann.fgco2.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=False)
#bias[1] = da1.values - da0.values
#
#da2 = DS_fill_ann.fgco2.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=False)
#bias[2] = da2.values - da0.values
#
#bias[3] = da2.values - da1.values
#
#DS = xr.Dataset( {'GLB100km': (['lat','lon'], bias[1]),
#                  'NP10km': (['lat','lon'], bias[2]),
#                  'NP10km-GLB100km': (['lat','lon'], bias[3]),
#                  'REF':   (['lat','lon'], da0.values), },
#                 coords = { 'lat': np.linspace(-89.5,89.5,num=180), 
#                            'lon': np.linspace(0.5,359.5,num=360), } )

################
# draw figures

fig = plt.figure(figsize = (8.5,11))
fig.suptitle( suptitle, fontsize=18 )

proj = ccrs.PlateCarree(central_longitude=-140.)
lon_formatter = LongitudeFormatter(zero_direction_label=True)
lat_formatter = LatitudeFormatter()


################
ax = [
    plt.subplot(3,2,1),
    plt.subplot(3,2,2),
    plt.subplot(3,2,3,projection=proj),
    plt.subplot(3,2,4,projection=proj),
    plt.subplot(3,2,5,projection=proj),
    plt.subplot(3,2,6,projection=proj),
]

# [left, bottom, width, height]

ax_cbar = [
    plt.axes([0.04,0.35,0.45,0.015]),
    plt.axes([0.54,0.35,0.45,0.015]),
    plt.axes([0.04,0.05,0.45,0.015]),
    plt.axes([0.54,0.05,0.45,0.015]),
]


fgco2_df.plot(y=fgco2_df.columns[0],ax=ax[0],xlim=[2020-nyc*5-1,2020],ylim=[-4.0,3.0],color='blue',linewidth=1)
fgco2_df.plot(y=fgco2_df.columns[1],ax=ax[0],xlim=[2020-nyc*5-1,2020],ylim=[-4.0,3.0],color='red',linewidth=1)
fgco2_df.plot(y=fgco2_df.columns[2],ax=ax[0],xlim=[2020-nyc*5-1,2020],ylim=[-4.0,3.0],color='darkgoldenrod',linewidth=1)

ax[0].grid()
ax[0].set_xlabel('')
ax[0].set_xticks(np.arange(2020-nyc*5-1,2020,nyc))
ax[0].set_xlabel('Year',fontsize=10)
ax[0].set_ylabel('PgC/yr',fontsize=10)
ax[0].set_yticks(np.linspace(-4.0,3.0,15))
ax[0].set_title(title[0],{'fontsize':10, 'verticalalignment':'top'})
ax[0].tick_params(labelsize=10)
leg = ax[0].legend(bbox_to_anchor=(0.98,0.02),loc='lower right')

################

fgco2_dfp.plot(y=fgco2_dfp.columns[2],ax=ax[1],xlim=[1935,2022],ylim=[-0.2,3.7],color='green',linewidth=2)
ax[1].fill_between(x=fgco2_dfp.index,y1=fgco2_dfp['products_min'],y2=fgco2_dfp['products_max'],\
                   alpha=0.5,facecolor='lightgreen')
fgco2_dfo.plot(y=fgco2_dfo.columns[2],ax=ax[1],xlim=[1935,2022],ylim=[-0.2,3.7],color='goldenrod',linewidth=2)
ax[1].fill_between(x=fgco2_dfo.index,y1=fgco2_dfo['multimodel_min'],y2=fgco2_dfo['multimodel_max'],\
                   alpha=0.5,facecolor='palegoldenrod')

fgco2_dfm.plot(y=fgco2_dfm.columns[0],ax=ax[1],xlim=[1935,2022],ylim=[-0.2,3.7],color='red',linewidth=2)
fgco2_dfm.plot(y=fgco2_dfm.columns[1],ax=ax[1],xlim=[1935,2022],ylim=[-0.2,3.7],color='darkblue',linewidth=2)

ax[1].grid()
ax[1].set_xlabel('')
ax[1].set_xticks(np.arange(1940,2021,10))
ax[1].set_xlim([1935,2022])
ax[1].set_ylim([-0.2,3.7])
ax[1].set_xlabel('Year',fontsize=10)
ax[1].set_ylabel('PgC/yr',fontsize=10)
ax[1].set_yticks(np.linspace(-0.2,3.4,19))
ax[1].set_title(title[1],{'fontsize':10, 'verticalalignment':'top'})
ax[1].tick_params(labelsize=10)
leg = ax[1].legend(bbox_to_anchor=(0.005,0.995),loc='upper left',ncol=1)

################

bounds1 = [-5.0, -2.0, -1.0, -0.7, -0.4, -0.1, 0.1, 0.4, 0.7, 1.0, 2.0, 5.0]
bounds2 = [-2.0, -1.0, -0.7, -0.4, -0.2, -0.1, 0.1, 0.2, 0.4, 0.7, 1.0, 2.0]
bounds3 = [-10.0, -5.0, -4.0, -3.0, -2.0, -1.0, -0.5, 0.5, 1.0, 2.0, 3.0, 4.0, 5.0, 10.0]

cmap = [ 'dummy', 'dummy', 'BrBG_r', 'BrBG_r', 'bwr', 'BrBG_r' ]

item = [ 'dummy', 'dummy', 'GLB100km', 'NP10km', 'NP10km-GLB100km', 'REF' ]

for panel in range(2,6):
    print("Drawing panel "+ str(panel))
    if (item[panel] == 'GLB100km'):
        bounds = bounds3
        ticks_bounds = bounds3
        da = DS_g100_ann.fgco2.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=False)
    elif (item[panel] == 'NP10km'):
        bounds = bounds3
        ticks_bounds = bounds3
        da = DS_fill_ann.fgco2.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=False)
    elif item[panel] == 'NP10km-GLB100km':
        bounds = bounds2
        ticks_bounds = bounds2
        da = DS_fill_ann.fgco2.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=False) \
           - DS_g100_ann.fgco2.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=False)
    else:
        bounds = bounds3
        ticks_bounds = bounds3
        da = DS_jma_ann.fgco2.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=False)
#        da = DS_ref_ann.fgco2.sel(time=slice(str(ystr),str(yend))).mean(dim='time',skipna=False)

#    da.plot(ax=ax[panel],cmap=cmap[panel],
#            levels=bounds,
#            extend='both',
#            cbar_kwargs={'orientation': 'horizontal',
#                         'spacing':'uniform',
#                         'shrink':1.00,
#                         'label': '[$\mathrm{mol}\, \mathrm{m}^{-2}\,  \mathrm{yr}^{-1}$]',
#                         'ticks': ticks_bounds,},
#            transform=ccrs.PlateCarree())

    im = da.plot(ax=ax[panel],cmap=cmap[panel],
                 levels=bounds,
                 extend='both',
                 add_colorbar=False,
                 transform=ccrs.PlateCarree())

    cbar = plt.colorbar(im,orientation='horizontal',spacing='uniform',label='[$\mathrm{mol}\, \mathrm{m}^{-2}\,  \mathrm{yr}^{-1}$]',
                        ticks=ticks_bounds, cax=ax_cbar[panel-2])
    cbar.ax.tick_params(labelsize=8)

    ax[panel].grid(linestyle=':')
    ax[panel].coastlines()
    ax[panel].set_xticks(np.arange(-180,180.1,60),crs=ccrs.PlateCarree())
    ax[panel].set_yticks(np.arange(-90,90.1,30),crs=ccrs.PlateCarree())
    ax[panel].xaxis.set_major_formatter(lon_formatter)
    ax[panel].yaxis.set_major_formatter(lat_formatter)
    ax[panel].set_xlabel('')
    ax[panel].set_title(title[panel],{'fontsize':10, 'verticalalignment':'top'})
    ax[panel].tick_params(labelsize=9)
    ax[panel].patch.set_facecolor('lightgray')

    if (panel == 3):
        ax[panel].hlines(y=[-15.0,63.0],xmin=99.0,xmax=285.0, colors="indigo",linewidth=1.0,transform=ccrs.PlateCarree())
        ax[panel].vlines(x=[99.0,285.0],ymin=-15.0,ymax=63.0, colors="indigo",linewidth=1.0,transform=ccrs.PlateCarree())

plt.subplots_adjust(left=0.08,right=0.98,bottom=0.05,top=0.92,hspace=0.18,wspace=0.20)
#
outpdf = outfile+'.pdf'
outpng = outfile+'.png'

plt.savefig(outpng, bbox_inches='tight', pad_inches=0.05)
plt.savefig(outpdf, bbox_inches='tight', pad_inches=0.05)
print("figure is saved to " + outpng + " and " + outpdf)
plt.show()
