# -*- coding: utf-8 -*-
import sys
import json
import numpy as np
import pandas as pd
import xarray as xr
import matplotlib.pyplot as plt
import matplotlib as mpl
import netCDF4
import datetime

if (len(sys.argv) < 4):
    print ('Usage: ' + sys.argv[0] + ' start_year end_year exp1 exp2')
    sys.exit()

ystr=int(sys.argv[1])
yend=int(sys.argv[2])

exp1=sys.argv[3]
exp2=sys.argv[4]

suptitle = ' MOC Indo-Pacific (ave. from ' + str(ystr) + ' - ' + str(yend) + ')'

xlim  = [ -30, 65 ]
varname = 'inp'
title = [ 'Indo-Pacific Ocean' ]

#xlim  = [ -8, 65 ]
#varname = 'pac'
#title = [ 'Pacific Ocean' ]

title2 = [ '(a) '+ exp1, '(b) '+ exp2, '(c) ' + exp1 + ' (GM)', '(d) ' + exp2 + '-' + exp1 ]

####################################################################
path_glb100km='../data/glb100km/'
path_np10km='../data/np10km/'
####################################################################
# GLB100km

print( "Loading GLB100km data" )

path_g100 = path_glb100km + 'run-20211104o2/ANALY/annclim'
basename_g100 = 'hs_uvmoc_anncl'

suffix = '.' + '{:04}'.format(ystr) + '-{:04}'.format(yend) + '.nc'
uvmoc_file = path_g100 + '/' + 'hs_uvmoc_anncl' + suffix
gmmoc_file = path_g100 + '/' + 'hs_gmmoc_anncl' + suffix
uvg100 = netCDF4.Dataset(uvmoc_file,'r')
gmg100 = netCDF4.Dataset(gmmoc_file,'r')
ny = len(uvg100.dimensions['lat'])
nz = len(uvg100.dimensions['depth'])
lat = uvg100.variables['lat'][:]
depth = uvg100.variables['depth'][:]

moc_uvg100 = np.array(np.empty((nz,ny)),dtype=np.float64)
moc_gmg100 = np.array(np.empty((nz,ny)),dtype=np.float64)

moc_uvg100[:,:] = uvg100.variables[varname][0,:,:]
moc_gmg100[:,:] = gmg100.variables[varname][0,:,:]

uvg100.close()
gmg100.close()

moc_uvg100 = np.where(moc_uvg100 < -9.0e33, np.NaN, moc_uvg100)
moc_gmg100 = np.where(moc_gmg100 < -9.0e33, np.NaN, moc_gmg100)

DS_g100_ann = xr.Dataset({'GLB100kmAL': (['depth','lat'], moc_uvg100+moc_gmg100),
                          'GLB100kmGM': (['depth','lat'], moc_gmg100),},
                         coords = {'depth': depth, 'lat': lat } )

####################################################################
# NP10km

print( "Loading NP10km data" )

path_gl=path_np10km+'hst_annclim-glb'
path_np=path_np10km+'hst_annclim-np'

uvmoc_fill = np.array(np.empty((nz,ny)),dtype=np.float64)
gmmoc_fill = np.array(np.empty((nz,ny)),dtype=np.float64)

uvmoc_np = np.array(np.empty((nz,ny)),dtype=np.float64)
uvmoc_gl = np.array(np.empty((nz,ny)),dtype=np.float64)
gmmoc_np = np.array(np.empty((nz,ny)),dtype=np.float64)
gmmoc_gl = np.array(np.empty((nz,ny)),dtype=np.float64)

suffix = '.' + '{:04}-{:04}'.format(ystr,yend) + '.nc'

gluv_file = path_gl + '/' + 'hs_uvmoc_anncl' + suffix
npuv_file = path_np + '/' + 'hs_uvmoc_anncl' + suffix
ncgluv = netCDF4.Dataset(gluv_file,'r')
ncnpuv = netCDF4.Dataset(npuv_file,'r')

glgm_file = path_gl + '/' + 'hs_gmmoc_anncl' + suffix
npgm_file = path_np + '/' + 'hs_gmmoc_anncl' + suffix
ncglgm = netCDF4.Dataset(glgm_file,'r')
ncnpgm = netCDF4.Dataset(npgm_file,'r')

uvmoc_gl[:,:] = ncgluv.variables[varname][0,:,:]
uvmoc_gl = np.where(uvmoc_gl < -9.0e33, np.NaN, uvmoc_gl)

uvmoc_np[:,:] = ncnpuv.variables[varname][0,:,:]
uvmoc_np = np.where(uvmoc_np < -9.0e33, np.NaN, uvmoc_np)

uvmoc_fill[:,:] = uvmoc_gl[:,:]+uvmoc_np[:,:]

gmmoc_gl[:,:] = ncglgm.variables[varname][0,:,:]
gmmoc_gl = np.where(gmmoc_gl < -9.0e33, np.NaN, gmmoc_gl)

gmmoc_np[:,:] = ncnpgm.variables[varname][0,:,:]
gmmoc_np = np.where(gmmoc_np < -9.0e33, np.NaN, gmmoc_np)

gmmoc_fill[:,:] = gmmoc_gl[:,:]+gmmoc_np[:,:]

DS_fill_ann = xr.Dataset({'NP10kmAL': (['depth','lat'], uvmoc_fill+gmmoc_fill),},
                         coords = {'depth': depth, 'lat': lat } )

#######################################################

da0 = DS_g100_ann['GLB100kmAL'].values
da1 = DS_g100_ann['GLB100kmGM'].values
da2 = DS_fill_ann['NP10kmAL'].values


DS = xr.Dataset({'GLB100kmAL': (['depth','lat'], da0*1e-9),
                 'GLB100kmGM': (['depth','lat'], da1*1e-9),
                 'NP10kmAL': (['depth','lat'], da2*1e-9),
                 'exp2-1': (['depth','lat'], (da2-da0)*1e-9),},
                coords = {'depth': depth, 'lat': lat } )

########################################

fig = plt.figure(figsize=(8.5,11))
fig.suptitle( suptitle, fontsize=16, y=0.98 )

# [left, bottom, width, height]

axes0 = np.array( [ [0.07, 0.7, 0.42, 0.23],
                    [0.02, 0.66, 0.46, 0.015], ])

ax = [ plt.axes(axes0[0]),
       plt.axes(axes0[0]+np.array([0.5,0,0,0])),
       plt.axes(axes0[0]+np.array([0.0,-0.33,0,0])),
       plt.axes(axes0[0]+np.array([0.5,-0.33,0,0])), 
       plt.axes([0.08,0.05,0.26,0.22]),
       plt.axes([0.39,0.05,0.26,0.22]),
       plt.axes([0.70,0.05,0.26,0.22]),]

ax_cbar = [ plt.axes(axes0[1]),
            plt.axes(axes0[1]+np.array([0.5,0,0,0])),
            plt.axes(axes0[1]+np.array([0,-0.33,0,0])),
            plt.axes(axes0[1]+np.array([0.5,-0.33,0,0])), ]

bounds1 = np.linspace(-30,30,31)
tbounds1 = np.linspace(-30,30,16)
bounds2 = [-5,-4,-3,-2,-1,-0.5,0,0.5,1,2,3,4,5]

cmap = [ 'RdBu_r', 'RdBu_r', 'RdBu_r', 'RdBu_r' ]

item = [ 'GLB100kmAL', 'NP10kmAL', 'GLB100kmGM', 'exp2-1' ]

for panel in range(4):
    if (item[panel] == 'GLB100kmAL' or item[panel] == 'NP10kmAL'):
        bounds = bounds1
        ticks_bounds = tbounds1
    elif (item[panel] == 'GLB100kmGM' or item[panel] == 'exp2-1') :
        bounds = bounds2
        ticks_bounds = bounds2

    da = DS[item[panel]]

    if (panel < 4):
        im = da.plot(ax=ax[panel],cmap=cmap[panel],
                     levels=bounds,
                     extend='both',
                     add_colorbar=False,
                     add_labels=False)

        da.plot.contour(ax=ax[panel],
                        colors='black',
                        levels=bounds,
                        add_labels=False,
                        linewidths=1.0)
        ax[panel].grid()
        ax[panel].set_title(title2[panel])
        ax[panel].set_xlim(xlim[0],xlim[1])
        ax[panel].set_ylim(20.0,27.5)
        ax[panel].invert_yaxis()
        #ax[panel].set_xticks(np.arange(xlim[0],xlim[1]+0.1,30))
        ax[panel].set_yticks(np.arange(20.0,27.6,0.5))
        ax[panel].set_facecolor('lightgray')

        cbar = plt.colorbar(im,orientation='horizontal',spacing='uniform',
                            ticks=ticks_bounds, cax=ax_cbar[panel])
        cbar.set_label(label='Sv',fontsize=7,x=0.95)
        cbar.ax.tick_params(labelsize=7)

#fig.text(0.25,0.94,title2[0],fontsize=14,
#         horizontalalignment='center',verticalalignment='center')
#fig.text(0.75,0.94,title2[1],fontsize=14,
#         horizontalalignment='center',verticalalignment='center')
#fig.text(0.25,0.62,title2[2],fontsize=14,
#         horizontalalignment='center',verticalalignment='center')
#fig.text(0.75,0.62,title2[3],fontsize=14,
#         horizontalalignment='center',verticalalignment='center')


#----------------------

item_list = [ "Tropics", "SW-Subtropical", "NW-Subtropical", "E-Subtropical", "Subarctic", 
              "JapanSea", "OkhotskSea" ]

glb_list = [ "GLB-TRP", "GLB-SWST", "GLB-NWST", "GLB-EST", "GLB-SAR", "GLB-JPS", "GLB-OKH" ]
np_list = [ "NP-TRP", "NP-SWST", "NP-NWST", "NP-EST", "NP-SAR", "NP-JPS", "NP-OKH" ]

infl1 = path_glb100km + "run-20211104o2/csv/Pac3_Volume_conversion_lyr_GLB100km_1987-2017.csv"
df1=pd.read_csv(infl1,index_col=0)
df1.set_axis(pd.Index(glb_list),axis=1,inplace=True)

infl4 = path_np10km + "csv/Pac3_Volume_conversion_lyr_NP10km_1987-2017.csv"
df4=pd.read_csv(infl4,index_col=0)
df4.set_axis(pd.Index(np_list),axis=1,inplace=True)

dfa=pd.concat([df1,df4],axis=1)

print(dfa)
#----------------------

reg=1
panel=5
dfa[[glb_list[reg],np_list[reg]]].iloc[0:10].plot.bar(ax=ax[panel],subplots=False)
ax[panel].set_title('(f) Vol conv rate for '+item_list[reg],fontsize=10)
ax[panel].set_xlabel('')
ax[panel].set_ylabel('')
ax[panel].set_xticklabels(dfa.index.values[0:10],rotation=45,fontsize=8)
ax[panel].set_ylim(-5,6)

reg=2
panel=4
dfa[[glb_list[reg],np_list[reg]]].iloc[0:10].plot.bar(ax=ax[panel],subplots=False)
ax[panel].set_title('(e) Vol conv rate for '+item_list[reg],fontsize=10)
ax[panel].set_xlabel('')
ax[panel].set_ylabel(r'[$\mathrm{Sv}$]',fontsize=12)
ax[panel].set_xticklabels(dfa.index.values[0:10],rotation=45,fontsize=8)
ax[panel].set_ylim(-5,6)

reg=3
panel=6
dfa[[glb_list[reg],np_list[reg]]].iloc[0:10].plot.bar(ax=ax[panel],subplots=False)
ax[panel].set_title('(g) Vol conv rate for '+item_list[reg],fontsize=10)
ax[panel].set_xlabel('')
ax[panel].set_ylabel('')
ax[panel].set_xticklabels(dfa.index.values[0:10],rotation=45,fontsize=8)
ax[panel].set_ylim(-5,6)

#axes.set_ylim(cnvrange_list[item][0],cnvrange_list[item][1])
#plt.subplots_adjust(top=0.92,hspace=0.33)
#plt.savefig('fig/Volume_conversion_rate_summary_pac3_'+item_list[region]+'.png')
#plt.show()

#----------------------

outpng = f'./fig/MOCIP_comp_{varname}_vol.png'
plt.savefig(outpng, bbox_inches='tight', pad_inches=0.1)
outpdf = f'./fig/MOCIP_comp_{varname}_vol.pdf'
plt.savefig(outpdf, bbox_inches='tight', pad_inches=0.1)
plt.show()
