# -*- coding: utf-8 -*-
import sys
import json
import numpy as np
import xarray as xr
import matplotlib.pyplot as plt
import matplotlib as mpl
import netCDF4
import datetime

if (len(sys.argv) < 4):
    print ('Usage: ' + sys.argv[0] + ' start_year end_year exp1 exp2')
    sys.exit()

ystr=int(sys.argv[1])
yend=int(sys.argv[2])
nyr=yend-ystr+1

exp1=sys.argv[3]
exp2=sys.argv[4]


suptitle = ' Meridional Overturning Circulation Indo-Pacific (' + str(ystr) + ' - ' + str(yend) + ' mean)'
xlim = [ [-90, -28], [-30, 90], [-30, 70] ]

title2 = [
 '(a) MOC (Indo-Pacific) on depth coordinate (MOC-z)',
 '(b) MOC-z (Indo-Pacific) difference (HighRes - LowRes)',
 r'(c) MOC (Indo-Pacific) on density coordinate (MOC-${\sigma}_0$)',
 r'(d) MOC-${\sigma}_0$ (Indo-Pacific) difference (HighRes - LowRes)' ]

latwoa = np.linspace(-89.5,89.5,num=180)

nyr = yend - ystr + 1
time0 = np.empty(nyr,dtype='object')
for yr in range(ystr,yend+1):
    time0[yr-ystr] = datetime.datetime(yr,1,1)

####################################################################
path_glb100km='../data/glb100km/'
path_np10km='../data/np10km/'
####################################################################
# GLB100km

print( "Loading GLB100km data" )

path_g100 = path_glb100km + 'run-20211104o2/ANALY/annual'
basename_g100 = 'hs_moc'

suffix = '.' + '{:04}'.format(ystr)
moc_file = path_g100 + '/' + basename_g100 + suffix
ncg100 = netCDF4.Dataset(moc_file,'r')
ny = len(ncg100.dimensions['lat'])
nz = len(ncg100.dimensions['depth'])
lat = ncg100.variables['lat'][:]
depth = ncg100.variables['depth'][:]

moc_g100 = np.array(np.empty((nyr,4,nz,ny)),dtype=np.float64)
moc_mod = np.array(np.empty((nz,ny)),dtype=np.float64)

for yr in range(ystr,yend+1):
    suffix = '.' + '{:04}'.format(yr)
    moc_file = path_g100 + '/' + basename_g100 + suffix
    ncg100 = netCDF4.Dataset(moc_file,'r')
    moc_mod[:,:] = ncg100.variables['glb'][0,:,:]
    moc_g100[yr-ystr,0,:,:] = moc_mod[:,:].copy()
    moc_mod[:,:] = ncg100.variables['atl'][0,:,:]
    moc_g100[yr-ystr,1,:,:] = moc_mod[:,:].copy()
    moc_mod[:,:] = ncg100.variables['inp'][0,:,:]
    moc_g100[yr-ystr,2,:,:] = moc_mod[:,:].copy()
    ncg100.close()

moc_g100 = np.where(moc_g100 < -9.0e33, np.NaN, moc_g100)

DS_g100_ann = xr.Dataset({'GLB100km': (['time','basin','depth','lat'], moc_g100),},
                         coords = {'time': time0, 'depth': depth, 'lat': lat } )

####################################################################
# NP10km

print( "Loading NP10km data" )

path_gl = path_np10km + 'hst_ann-glb'
path_np = path_np10km + 'hst_ann-np'
moc_fill = np.array(np.empty((nyr,3,nz,ny)),dtype=np.float64)
basename_gl = 'hs_moc'
basename_np = 'hs_moc'

moc_fill = np.array(np.empty((nyr,4,nz,ny)),dtype=np.float64)
moc_np = np.array(np.empty((nz,ny)),dtype=np.float64)
moc_gl = np.array(np.empty((nz,ny)),dtype=np.float64)

for yr in range(ystr,yend+1):
    suffix = '.' + '{:04}'.format(yr)
    gl_file = path_gl + '/y' + str(yr) + '/' + basename_gl + suffix
    np_file = path_np + '/y' + str(yr) + '/' + basename_np + suffix
    ncgl = netCDF4.Dataset(gl_file,'r')
    ncnp = netCDF4.Dataset(np_file,'r')
    moc_gl[:,:] = ncgl.variables['glb'][0,:,:]
    moc_np[:,:] = ncnp.variables['glb'][0,:,:]
    moc_fill[yr-ystr,0,:,:] = moc_gl[:,:]+moc_np[:,:]
    moc_gl[:,:] = ncgl.variables['atl'][0,:,:]
    moc_np[:,:] = ncnp.variables['atl'][0,:,:]
    moc_fill[yr-ystr,1,:,:] = moc_gl[:,:]+moc_np[:,:]
    moc_gl[:,:] = ncgl.variables['inp'][0,:,:]
    moc_np[:,:] = ncnp.variables['inp'][0,:,:]
    moc_fill[yr-ystr,2,:,:] = moc_gl[:,:]+moc_np[:,:]

moc_fill = np.where(moc_fill < -9.0e33, np.NaN, moc_fill)

DS_fill_ann = xr.Dataset({'NP10km': (['time','basin','depth','lat'], moc_fill),},
                         coords = {'time': time0, 'depth': depth, 'lat': lat } )

#######################################################
# density coordinates

path_g100 = path_glb100km + 'run-20211104o2/ANALY/annclim'
basename_g100 = 'hs_uvmoc_anncl'

suffix = '.' + '{:04}'.format(ystr) + '-{:04}'.format(yend) + '.nc'
uvmoc_file = path_g100 + '/' + 'hs_uvmoc_anncl' + suffix
gmmoc_file = path_g100 + '/' + 'hs_gmmoc_anncl' + suffix
uvg100 = netCDF4.Dataset(uvmoc_file,'r')
gmg100 = netCDF4.Dataset(gmmoc_file,'r')
ny_ip = len(uvg100.dimensions['lat'])
nz_ip = len(uvg100.dimensions['depth'])
lat_ip = uvg100.variables['lat'][:]
density = uvg100.variables['depth'][:]

moc_uvg100 = np.array(np.empty((nz_ip,ny_ip)),dtype=np.float64)
moc_gmg100 = np.array(np.empty((nz_ip,ny_ip)),dtype=np.float64)

moc_uvg100[:,:] = uvg100.variables['inp'][0,:,:]
moc_gmg100[:,:] = gmg100.variables['inp'][0,:,:]

uvg100.close()
gmg100.close()

moc_uvg100 = np.where(moc_uvg100 < -9.0e33, np.NaN, moc_uvg100)
moc_gmg100 = np.where(moc_gmg100 < -9.0e33, np.NaN, moc_gmg100)

DS_g100_ip_ann = xr.Dataset({'GLB100kmAL': (['depth','lat'], moc_uvg100+moc_gmg100),
                             'GLB100kmGM': (['depth','lat'], moc_gmg100),},
                            coords = {'depth': density, 'lat': lat_ip } )

#----

path_gl=path_np10km+'hst_annclim-glb'
path_np=path_np10km+'hst_annclim-np'

uvmoc_fill = np.array(np.empty((nz_ip,ny_ip)),dtype=np.float64)
gmmoc_fill = np.array(np.empty((nz_ip,ny_ip)),dtype=np.float64)

uvmoc_np = np.array(np.empty((nz_ip,ny_ip)),dtype=np.float64)
uvmoc_gl = np.array(np.empty((nz_ip,ny_ip)),dtype=np.float64)
gmmoc_np = np.array(np.empty((nz_ip,ny_ip)),dtype=np.float64)
gmmoc_gl = np.array(np.empty((nz_ip,ny_ip)),dtype=np.float64)

suffix = '.' + '{:04}-{:04}'.format(ystr,yend) + '.nc'

gluv_file = path_gl + '/' + 'hs_uvmoc_anncl' + suffix
npuv_file = path_np + '/' + 'hs_uvmoc_anncl' + suffix
ncgluv = netCDF4.Dataset(gluv_file,'r')
ncnpuv = netCDF4.Dataset(npuv_file,'r')

glgm_file = path_gl + '/' + 'hs_gmmoc_anncl' + suffix
npgm_file = path_np + '/' + 'hs_gmmoc_anncl' + suffix
ncglgm = netCDF4.Dataset(glgm_file,'r')
ncnpgm = netCDF4.Dataset(npgm_file,'r')

uvmoc_gl[:,:] = ncgluv.variables['inp'][0,:,:]
uvmoc_gl = np.where(uvmoc_gl < -9.0e33, np.NaN, uvmoc_gl)

uvmoc_np[:,:] = ncnpuv.variables['inp'][0,:,:]
uvmoc_np = np.where(uvmoc_np < -9.0e33, np.NaN, uvmoc_np)

uvmoc_fill[:,:] = uvmoc_gl[:,:]+uvmoc_np[:,:]

gmmoc_gl[:,:] = ncglgm.variables['inp'][0,:,:]
gmmoc_gl = np.where(gmmoc_gl < -9.0e33, np.NaN, gmmoc_gl)

gmmoc_np[:,:] = ncnpgm.variables['inp'][0,:,:]
gmmoc_np = np.where(gmmoc_np < -9.0e33, np.NaN, gmmoc_np)

gmmoc_fill[:,:] = gmmoc_gl[:,:]+gmmoc_np[:,:]

DS_fill_ip_ann = xr.Dataset({'NP10kmAL': (['depth','lat'], uvmoc_fill+gmmoc_fill),},
                            coords = {'depth': density, 'lat': lat_ip } )

#########################################
da0 = DS_g100_ann['GLB100km'].mean(dim='time').values
da1 = DS_fill_ann['NP10km'].mean(dim='time').values


DS = xr.Dataset({'GLB100km': (['basin','depth','lat'], da0*1e-9),
                 'NP10km': (['basin','depth','lat'], da1*1e-9),
                 'exp2-1': (['basin','depth','lat'], (da1-da0)*1e-9),},
                coords = {'depth': depth, 'lat': lat } )

#######################################################
da0 = DS_g100_ip_ann['GLB100kmAL'].values
da1 = DS_g100_ip_ann['GLB100kmGM'].values
da2 = DS_fill_ip_ann['NP10kmAL'].values

DS_ip = xr.Dataset({'GLB100kmAL': (['depth','lat'], da0*1e-9),
                    'GLB100kmGM': (['depth','lat'], da1*1e-9),
                    'NP10kmAL': (['depth','lat'], da2*1e-9),
                    'exp2-1': (['depth','lat'], (da2-da0)*1e-9),},
                   coords = {'depth': density, 'lat': lat_ip } )

########################################

fig = plt.figure(figsize=(11,8))
fig.suptitle( suptitle, fontsize=18, y=0.975 )

axes0 = np.array( [ [0.07, 0.6, 0.07, 0.3],
                    [0.15, 0.6, 0.17, 0.3],
                    [0.33, 0.6, 0.15, 0.3],
                    [0.07, 0.50, 0.40, 0.015], ])
ax = [ plt.axes([0.07,0.57,0.4,0.31]),
       plt.axes([0.57,0.57,0.4,0.31]),
       plt.axes([0.07,0.12,0.4,0.31]),
       plt.axes([0.57,0.12,0.4,0.31]), ]

ax_cbar = [ plt.axes(axes0[3]),
            plt.axes(axes0[3]+np.array([0.5,0,0,0])),
            plt.axes(axes0[3]+np.array([0,-0.45,0,0])),
            plt.axes(axes0[3]+np.array([0.5,-0.45,0,0])), ]

plt.text(0.27,0.915,'High Resolution',ha='center',size=18,transform=fig.transFigure)
plt.text(0.77,0.915,'HighRes - LowRes',ha='center',size=18,transform=fig.transFigure)

bounds1 = np.linspace(-30,30,31)
tbounds1 = np.linspace(-30,30,16)
bounds2 = [-10,-8,-6,-5,-4,-3,-2,-1,0,1,2,3,4,5,6,8,10]
bounds3 = [-5,-4,-3,-2,-1,-0.5,0,0.5,1,2,3,4,5]

cmap = [ 'RdBu_r', 'RdBu_r', 'RdBu_r', 'RdBu_r' ]

item = [ exp2, 'exp2-1', 'NP10kmAL', 'exp2-1' ]

for panel in range(2):
    if item[panel] == exp1 or item[panel] == exp2:
        bounds = bounds1
        ticks_bounds = tbounds1
    elif item[panel] == 'exp2-1':
        bounds = bounds2
        ticks_bounds = bounds2

    da = DS[item[panel]]

    if (panel == 0):
        m = 2
        im = da.isel(basin=m).plot(ax=ax[panel],cmap=cmap[panel],
                                   levels=bounds,
                                   extend='both',
                                   add_colorbar=False)
        cont = da.isel(basin=m).plot.contour(ax=ax[panel],
                                      colors='black',
                                      levels=bounds,
                                      add_labels=True,
                                      linewidths=1.0)
        ax[panel].set_title(title2[panel],{'fontsize':12, 'verticalalignment':'top'})
        ax[panel].invert_yaxis()
        ax[panel].set_xlim(xlim[m][0],xlim[m][1])
        ax[panel].set_xticks(np.arange(xlim[m][0],xlim[m][1]+0.1,10))
        ax[panel].set_xlabel("latitude")
        ax[panel].set_ylabel("depth")
        ax[panel].set_facecolor('lightgray')

        cbar = plt.colorbar(im,orientation='horizontal',spacing='uniform',
                            ticks=ticks_bounds, cax=ax_cbar[panel])
        cbar.set_label(label='[Sv]',fontsize=8,x=0.95,labelpad=0)
        cbar.ax.tick_params(labelsize=8)

        clevels=cont.levels
        cont.clabel(fmt='%4d',fontsize=10,colors='black',inline_spacing=1)

    else:
        m = 2
        #mp = 0
        im = da.isel(basin=m).plot(ax=ax[panel],cmap=cmap[panel],
                                   levels=bounds,
                                   extend='both',
                                   add_colorbar=False)
        da.isel(basin=m).plot.contour(ax=ax[panel],
                                      colors='black',
                                      levels=bounds,
                                      add_labels=False,
                                      linewidths=1.0)
        ax[panel].set_title(title2[panel],{'fontsize':12, 'verticalalignment':'top'})
        ax[panel].invert_yaxis()
        ax[panel].set_xlim(xlim[m][0],xlim[m][1])
        ax[panel].set_xticks(np.arange(xlim[m][0],xlim[m][1]+0.1,10))
        ax[panel].set_xlabel("latitude")
        ax[panel].set_ylabel("depth")
        ax[panel].set_facecolor('lightgray')
        ax[panel].tick_params(axis='y')
        
        cbar = plt.colorbar(im,orientation='horizontal',spacing='uniform',
                            ticks=ticks_bounds, cax=ax_cbar[panel])
        cbar.set_label(label='[Sv]',fontsize=8,x=0.95,labelpad=0)
        cbar.ax.tick_params(labelsize=8)

for panel in range(2,4):
    if (item[panel] == 'GLB100kmAL' or item[panel] == 'NP10kmAL'):
        bounds = bounds1
        ticks_bounds = tbounds1
    elif (item[panel] == 'GLB100kmGM' or item[panel] == 'exp2-1') :
        bounds = bounds3
        ticks_bounds = bounds3

    da = DS_ip[item[panel]]

    im = da.plot(ax=ax[panel],cmap=cmap[panel],
                 levels=bounds,
                 extend='both',
                 add_colorbar=False,
                 add_labels=False)

    da.plot.contour(ax=ax[panel],
                    colors='black',
                    levels=bounds,
                    add_labels=False,
                    linewidths=1.0)
    ax[panel].grid()
    ax[panel].set_title(title2[panel],{'fontsize':12, 'verticalalignment':'top'})
    ax[panel].set_xlim(xlim[m][0],xlim[m][1])
    ax[panel].set_ylim(20.0,27.5)
    ax[panel].set_xlabel("latitude")
    ax[panel].set_ylabel(r"density (${\sigma}_0$)")
    ax[panel].invert_yaxis()
    ax[panel].set_xticks(np.arange(xlim[m][0],xlim[m][1]+0.1,10))
    ax[panel].set_yticks(np.arange(20.0,27.6,0.5))
    ax[panel].set_facecolor('lightgray')

    cbar = plt.colorbar(im,orientation='horizontal',spacing='uniform',
                        ticks=ticks_bounds, cax=ax_cbar[panel])
    cbar.set_label(label='[Sv]',fontsize=8,x=0.95,labelpad=0)
    cbar.ax.tick_params(labelsize=8)


#fig.text(0.25,0.92,title2[0],fontsize=14,
#         horizontalalignment='center',verticalalignment='center')
#fig.text(0.75,0.92,title2[1],fontsize=14,
#         horizontalalignment='center',verticalalignment='center')
#fig.text(0.25,0.45,title2[2],fontsize=14,
#         horizontalalignment='center',verticalalignment='center')
#fig.text(0.75,0.45,title2[3],fontsize=14,
#         horizontalalignment='center',verticalalignment='center')

outpng = './fig/MOC_INP_all.png'
outpdf = './fig/MOC_INP_all.pdf'
plt.savefig(outpng, bbox_inches='tight', pad_inches=0.1)
plt.savefig(outpdf, bbox_inches='tight', pad_inches=0.1)
plt.show()
