# -*- coding: utf-8 -*-
import sys
import json
import numpy as np
import netCDF4
import pandas as pd
import xarray as xr
import cartopy.crs as ccrs
import matplotlib as mpl
import matplotlib.pyplot as plt
from cartopy.mpl.ticker import LongitudeFormatter, LatitudeFormatter
import datetime
import math

if (len(sys.argv) < 2):
    print ('Usage: ' + sys.argv[0] + ' exp1 exp2')
    sys.exit()

exp1=sys.argv[1]
exp2=sys.argv[2]

ystr_sst=1989
yend_sst=2018
nyr_sst=yend_sst-ystr_sst+1

ystr_ssh=1993
yend_ssh=2018
nyr_ssh=yend_ssh-ystr_ssh+1

ystr_mld=2000
yend_mld=2018
nyr_mld=yend_mld-ystr_mld+1

ystr_qtt=1988
yend_qtt=2017
nyr_qtt=yend_qtt-ystr_qtt+1

title_top = [[ 
  "(a) SST bias relative to observation",
  "(b) SST bias relative to observation",
 r"(c) SST difference (HighRes - LowRes)",
  "(d) PCMDI-SST"],
             [
  "(e) SSH bias relative to observation",
  "(f) SSH bias relative to observation",
 r"(g) SSH difference (HighRes - LowRes)",
  "(h) CMEMS-SSH"],
             [
  "(i) MLD (Winter) bias relative to observation",
  "(j) MLD (Winter) bias relative to observation",
  "(k) MLD difference (HighRes - LowRes)",
  "(l) deBoyer montégut"],
             [
  r"(m) Q$_{\mathrm{tot}}$ bias relative to observation",
  r"(n) Q$_{\mathrm{tot}}$ bias relative to observation",
  r"(o) Q$_{\mathrm{tot}}$ difference (HighRes - LowRes)",
  "(p) J-OFURO3"] ]

########################################

path_glb100km='../data/glb100km/'
path_np10km='../data/np10km/'
path_ref='../data/refdata/'

timey_sst = np.empty((yend_sst-ystr_sst+1),dtype='object')
for yr in range(ystr_sst,yend_sst+1):
    timey_sst[yr-ystr_sst] = datetime.datetime(yr,1,1)

timey_ssh = np.empty((yend_ssh-ystr_ssh+1),dtype='object')
for yr in range(ystr_ssh,yend_ssh+1):
    timey_ssh[yr-ystr_ssh] = datetime.datetime(yr,1,1)

timey_qtt = np.empty((yend_qtt-ystr_qtt+1),dtype='object')
for yr in range(ystr_qtt,yend_qtt+1):
    timey_qtt[yr-ystr_qtt] = datetime.datetime(yr,1,1)

################################################
# Mask to select Pacific Ocean
#
#print("Warning: pacific mask must be much more elaborated")
#maskpac = np.array(np.empty((180,360)),dtype=np.int64)
#maskpac[:,:] = 1
#maskpac[0:75,0:360] = 0
#maskpac[155:180,0:360] = 0
#maskpac[0:180,0:100] = 0
#maskpac[0:180,286:360] = 0

print("Loading WOA13v2 mask")

basinfile = path_ref + 'WOA13v2/1deg_L33/const/WOA13_1x1_mask.nc'

ncbas = netCDF4.Dataset(basinfile,'r')
basin = ncbas.variables['basin_mask'][:,:]
lonm = ncbas['lon'][:]
latm = ncbas['lat'][:]
ncbas.close()

nx = len(lonm)
ny = len(latm)

maskpac = np.array(np.empty((ny,nx)),dtype=np.int64)
maskpac[:,:] = 0
maskpjp = np.array(np.empty((ny,nx)),dtype=np.int64)
maskpjp[:,:] = 0

for j in range(ny):
    if (latm[j] > 12.0 and latm[j] < 63.0):
        for i in range(nx):
            if (basin[j,i] == 2):
                maskpac[j,i] = 1
                maskpjp[j,i] = 1
            if (basin[j,i] == 12):
                maskpjp[j,i] = 1

print("Loading PCMDI-AMIP-SST mask")

arefile = path_ref + 'PCMDI-SST/areacello_input4MIPs_SSTsAndSeaIce_CMIP_PCMDI-AMIP-1-1-6_gn.nc'
ncare = netCDF4.Dataset(arefile,'r')
area = ncare.variables['areacello'][:,:]
ncare.close()
################################################
#J データ読込・平均

# SST

metainfo = json.load(open("./json/thetao_list.json"))
exp_list = metainfo.keys()

print( "Loading AMIP data" )
reffile = path_ref + 'PCMDI-SST/tos_input4MIPs_SSTsAndSeaIce_CMIP_PCMDI-AMIP-1-1-6_gn_187001-201812.nc'
mskfile = path_ref + 'PCMDI-SST/sftof_input4MIPs_SSTsAndSeaIce_CMIP_PCMDI-AMIP-1-1-6_gn.nc'
#J 年平均計算も行う。ただし日数の重みがつかないので不正確
DS0 = xr.open_dataset( reffile ).resample(time='1YS').mean()
da0 = DS0.tos.sel(time=slice(str(ystr_sst),str(yend_sst)))
da0 = da0.mean(dim='time',skipna=False)
DS1 = xr.open_dataset( mskfile )
da1 = DS1.sftof

data = []
nexp = 0
for expnam in exp_list:
    if (expnam == exp1 or expnam == exp2): 

        sst = np.empty((nyr_sst,180,360))
        path  = metainfo[expnam]['path']
        basename = metainfo[expnam]['fname_base']
        varname = metainfo[expnam]['name']
        offset = float(metainfo[expnam]['offset'])
        print(expnam + ' offset = ' + str(offset))
        
        for yr in range(ystr_sst,yend_sst+1):
            if (expnam == 'GLB100km'):
                suffix = '.' + '{:04}'.format(yr)
                infile = path + '/' + basename + suffix
            if (expnam == 'NP10km'):
                suffix = '.' + '{:04}'.format(yr)
                infile = path + '/y' + '{:04}'.format(yr) + '/' + basename + suffix

            ncin = netCDF4.Dataset(infile,'r')
            lon = ncin.variables['lon'][:]
            lat = ncin.variables['lat'][:]
            #sst_tmp[:,:] = ncin.variables[varname][0,0,:,:]
            sst[yr-ystr_sst,:,:] = ncin.variables[varname][0,0,:,:]
            ncin.close()

        sst = np.where(sst < -9.0e33, np.NaN, sst)
        d = sst.mean(axis=0)
        #title_top[0][nexp] = expnam
        data += [d.copy()]
        nexp += 1
        del sst

DS_sst = xr.Dataset( {'exp1': (['lat','lon'], data[0] - da0.values),
                      'exp2': (['lat','lon'], data[1] - da0.values),
                      'exp2-1': (['lat','lon'], data[1] - data[0]),
                      'obs': (['lat','lon'], 
                              np.where(da1.values==0, np.nan, da0.values)), },
                     coords = { 'lat': np.linspace(-89.5,89.5,num=180), 
                                'lon': np.linspace(0.5,359.5,num=360), } )

###########################################################################
# SSH

metainfo = json.load(open("./json/zos_list.json"))
exp_list = metainfo.keys()

print( "Loading CMEMS data" )
reffile = path_ref + 'CMEMS/zos_adt_CMEMS_1x1_monthly_199301-201812.nc'
DS0 = xr.open_dataset( reffile )
da0 = DS0.zos.sel(time=slice(str(ystr_ssh),str(yend_ssh)))

##J mask0 = 50S以北,50N以南で True となる2次元配列
#mask0 = np.array(abs(DS0.lat)<50).reshape(len(DS0.lat),1)*np.array(~np.isnan(DS0.lon))

# mask based on CMEMS
cmemsmskf = path_ref + 'CMEMS/zos_mask_gn_199301-200912.nc'
ncmskcmems = netCDF4.Dataset(cmemsmskf,'r')
maskcmems = ncmskcmems.variables['zosmask'][:,:]
ncmskcmems.close()

##J wgt0 = 緯度に応じた重み (2次元配列, mask0 = False の場所は0に)
#wgt0 = np.empty(mask0.shape)
wgt0 = np.empty(maskcmems.shape)
for i in range(len(DS0.zos[0][0][:])):
    for j in range(len(DS0.zos[0][:])):
#        wgt0[j,i] = math.cos(math.radians(DS0.lat.values[j])) * mask0[j,i] * maskcmems[j,i]
        wgt0[j,i] = math.cos(math.radians(DS0.lat.values[j])) * maskcmems[j,i]
        #if (i == 180):
        #    print(DS0.lat.values[j],wgt0[j,i])

##J wgt = 平均に使う重み(時間方向も含めた3次元配列)
##J       未定義の格子では重みを 0 にする

print("length of CMEM data = ", str(len(da0)))
wgt = np.tile(wgt0,(len(da0),1,1)) * np.logical_not(np.isnan(da0))
##J 重み付き平均を計算、オフセットとして元データから差し引く
data_ave = np.average(da0.fillna(0),weights=wgt,axis=(1,2))
for n in range(len(data_ave)):
    da0[n] = da0[n] - data_ave[n]

da0 = da0.mean(dim='time',skipna=False)

d_tmp0 = np.empty((180,360))
d_tmp1 = np.empty((180,360))
d_tmp2 = np.empty((180,360))

data = []
nexp = 0
d = np.empty( (180,360) )
for expnam in exp_list:
    if (expnam == exp1 or expnam == exp2): 

        ssh = np.empty((yend_ssh-ystr_ssh+1,180,360))
        path  = metainfo[expnam]['path']
        basename = metainfo[expnam]['fname_base']
        varname = metainfo[expnam]['name']
        factor = float(metainfo[expnam]['factor'])
        print("factor = " + str(factor))

        for yr in range(ystr_ssh,yend_ssh+1):
            if (expnam == 'GLB100km'):
                suffix = '.' + '{:04}'.format(yr)
                infile = path + '/' + basename + suffix
            if (expnam == 'NP10km'):
                suffix = '.' + '{:04}'.format(yr)
                infile = path + '/y' + '{:04}'.format(yr) + '/' + basename + suffix

            ncin = netCDF4.Dataset(infile,'r')
            lon = ncin.variables['lon'][:]
            lat = ncin.variables['lat'][:]
            ssh[yr-ystr_ssh,:,:] = ncin.variables[varname][0,:,:]
            ncin.close()

        ssh = np.where(ssh < -9.0e33, np.NaN, ssh*factor)
        DS_ssh = xr.Dataset( {'zos': (['time','lat','lon'], ssh.copy()),},
                             coords = {'time': timey_ssh, 'lat': lat, 'lon': lon, } )

        tmp = DS_ssh.zos.sel(time=slice(str(ystr_ssh),str(yend_ssh)))
        #print(tmp)
        print("length of tmp = " + str(len(tmp)))

        ##J 重み付き平均を計算、オフセットとして元データから差し引く
        wgt = np.tile(wgt0,(len(tmp),1,1)) * np.logical_not(np.isnan(tmp))
        data_ave = np.average(tmp.fillna(0),weights=wgt,axis=(1,2))
        for n in range(len(data_ave)):
            print(data_ave[n])
            tmp[n] = tmp[n] - data_ave[n]

        d = tmp.mean(dim='time',skipna=False).values
        #title_top[1][nexp] = expnam

        data += [d.copy()]
        nexp += 1
        del ssh

d_tmp0=np.where(maskcmems==0, np.NaN, da0.values)
d_tmp1=np.where(maskcmems==0, np.NaN, data[0])
d_tmp2=np.where(maskcmems==0, np.NaN, data[1])

DS_zos = xr.Dataset( {'exp1': (['lat','lon'], d_tmp1 - d_tmp0),
                      'exp2': (['lat','lon'], d_tmp2 - d_tmp0),
                      'exp2-1': (['lat','lon'], d_tmp2 - d_tmp1),
                      'obs': (['lat','lon'], d_tmp0), },
                     coords = { 'lat': np.linspace(-89.5,89.5,num=180), 
                                'lon': np.linspace(0.5,359.5,num=360), } )

###########################################################################
# MLD

metainfo = json.load(open("./json/mld_list.json"))
exp_list = metainfo.keys()

print( "Loading deBoyerMotegut data" )
reffile = path_ref + 'deBoyerMontegut_MLD/mld_dr003_ref10m_v2023.nc'

ncobsmcl = netCDF4.Dataset(reffile,'r')
nx = len(ncobsmcl.dimensions['lon'])
ny = len(ncobsmcl.dimensions['lat'])
mldobs_mon = ncobsmcl.variables['mld_dr003'][:,:,:]
print(mldobs_mon.shape)
#mldobs_mon = mldobs_tmp.transpose(2,0,1)
#print(mldobs_mon.shape)
miss_val_obsmon = ncobsmcl.variables['mld_dr003']._FillValue
print(miss_val_obsmon)
lon_ = ncobsmcl.variables['lon'][:]
lat_ = ncobsmcl.variables['lat'][:]
mldobs_mon = np.where(mldobs_mon < miss_val_obsmon+1, np.NaN, mldobs_mon)
lon_p=np.where(lon_ < 0, lon_+360, lon_)

mldobs_win = np.array(np.empty((ny,nx)),dtype=np.float64)

mldobs_win[:,:] = 0.5 * (mldobs_mon[1,:,:]+mldobs_mon[2,:,:])

DS_mldobs = xr.Dataset( {'obs': (['lat','lon'], mldobs_win), },
                 coords = { 'lat': lat_, 'lon': lon_p, } )

print("MLD data as is ",DS_mldobs)
DS_mldobs = DS_mldobs.roll(lon=-180,roll_coords=True)
print("MLD roteted ",DS_mldobs)

data = []

nexp = 0
for expnam in exp_list:
    if (expnam == exp1 or expnam == exp2): 

        mldwin = np.empty((nyr_mld,180,360))
        mldfeb = np.empty((180,360))
        mldmar = np.empty((180,360))

        path  = metainfo[expnam]['path']
        basename = metainfo[expnam]['fname_base']
        varname = metainfo[expnam]['name']

        for yr in range(ystr_mld,yend_mld+1):
            if (expnam == 'GLB100km'):
                suffix = '.' + '{:04}'.format(yr)
                infile = path + '/' + basename + suffix
                ncin = netCDF4.Dataset(infile,'r')
                mldfeb[:,:] = ncin.variables[varname][1,:,:]
                mldmar[:,:] = ncin.variables[varname][2,:,:]
                ncin.close()

            if (expnam == 'NP10km'):
                suffix = '.' + '{:04}{:02}'.format(yr,2)
                infile = path + '/y' + '{:04}'.format(yr) + '/' + basename + suffix
                ncin = netCDF4.Dataset(infile,'r')
                mldfeb[:,:] = ncin.variables[varname][0,:,:]
                ncin.close()
                suffix = '.' + '{:04}{:02}'.format(yr,3)
                infile = path + '/y' + '{:04}'.format(yr) + '/' + basename + suffix
                ncin = netCDF4.Dataset(infile,'r')
                mldmar[:,:] = ncin.variables[varname][0,:,:]
                ncin.close()

            mldfeb = np.where(mldfeb < -9.0e33, np.NaN, mldfeb)
            mldmar = np.where(mldmar < -9.0e33, np.NaN, mldmar)
            mldwin[yr-ystr_mld,:,:] = 0.5 * (mldfeb[:,:]+mldmar[:,:])
        
        d = mldwin.mean(axis=0)
        #title_top[nexp] = expnam
        nexp += 1
        data += [d.copy()]

        del d
        del mldwin
        del mldfeb
        del mldmar

DS_mld = xr.Dataset( {'exp1': (['lat','lon'], data[0]-DS_mldobs['obs'].values),
                      'exp2': (['lat','lon'], data[1]-DS_mldobs['obs'].values),
                      'exp2-1': (['lat','lon'], data[1] - data[0]),
                      'obs':(['lat','lon'], DS_mldobs['obs'].values), },
                     coords = { 'lat': np.linspace(-89.5,89.5,num=180),
                                'lon': np.linspace(0.5,359.5,num=360), } )

###########################################################################
# Q-total

#print( "Loading J-OFURO2 data" )

#path_jofuro = path_ref + 'J-OFURO2/'
#reffile = path_jofuro + "J-OFURO2_NHF_HF004_MONTHLY_1988.nc"
#ncref = netCDF4.Dataset(reffile,'r')
#lon = ncref.variables['XLONG'][:]
#lat = ncref.variables['YLAT'][:]
#nx = len(ncref.dimensions['XLONG'])
#ny = len(ncref.dimensions['YLAT'])
#ncref.close()
#
#print(nx,ny)
#
#qtot_ref = np.array(np.empty(((yend_qtt-ystr_qtt+1)*12,ny,nx)),dtype=np.float64)
#qtot_tmp = np.array(np.empty((12,ny,nx)),dtype=np.float64)
#
#timem_qtt = np.empty((yend_qtt-ystr_qtt+1)*12,dtype='object')
#
#for yr in range(ystr_qtt,yend_qtt+1):
#    for mon in range(1,13):
#        timem_qtt[(yr-ystr_qtt)*12+mon-1] = datetime.datetime(yr,mon,1)
#
#    reffile = path_jofuro + "J-OFURO2_NHF_HF004_MONTHLY_" + str(yr) + ".nc"
#    ncref = netCDF4.Dataset(reffile,'r')
#    qtot_tmp = ncref.variables['NHF'][:,:,:]
#    ncref.close()
#    smon = (yr-ystr_qtt)*12
#    emon = (yr-ystr_qtt+1)*12
#    qtot_ref[smon:emon,:,:] = qtot_tmp[0:12,:,:]
#
#qtot_ref = np.where(qtot_ref < -9000, np.NaN, qtot_ref)
#
#DS_qtt_mon = xr.Dataset( {'qtotal': (['time','lat','lon'], -qtot_ref),},
#                      coords = { 'time': timem_qtt,'lat': lat,'lon': lon, } )
#
#DS_qttobs = DS_qtt_mon.mean(dim='time',skipna=True)
#
#####

print( "Loading J-OFURO3 data" )

path_jofuro = path_ref + 'J-OFURO3/'

reffile = path_jofuro + "J-OFURO3_NHF_V1.1_CLM_1x1_1988-2017.nc"
ncref = netCDF4.Dataset(reffile,'r')
lon = ncref.variables['lon'][:]
lat = ncref.variables['lat'][:]
nx = len(ncref.dimensions['lon'])
ny = len(ncref.dimensions['lat'])
print(nx,ny)
qtot_tmp = np.array(np.empty((12,ny,nx)),dtype=np.float64)
qtot_tmp = ncref.variables['nhf'][:,:,:]
ncref.close()

timem_qtt = np.empty(12,dtype='object')
for mon in range(1,13):
    timem_qtt[mon-1] = datetime.datetime(ystr_qtt,mon,1)

qtot_tmp = np.where(qtot_tmp < -9000, np.NaN, qtot_tmp)

DS_qtt_mon = xr.Dataset( {'qtotal': (['time','lat','lon'], -qtot_tmp),},
                      coords = { 'time': timem_qtt,'lat': lat,'lon': lon, } )

DS_qttobs = DS_qtt_mon.mean(dim='time',skipna=True)

####################################################################
# GLB100km

print( "Loading GLB100km data" )

path_g100=path_glb100km + 'run-20211104o2/ANALY/annual'
base_qtotal_g100 = 'hs_q_total_wocorr_woa1x1'

suffix = '.' + '{:04}'.format(ystr_qtt)
qtotal_file = path_g100 + '/' + base_qtotal_g100 + suffix
nc_g100 = netCDF4.Dataset(qtotal_file,'r')
lon = nc_g100.variables['lon'][:]
lat = nc_g100.variables['lat'][:]
nx = len(nc_g100.dimensions['lon'])
ny = len(nc_g100.dimensions['lat'])
print(nx,ny)

qtotal_tmp = np.array(np.empty((ny,nx)),dtype=np.float64)
qtotal_g100 = np.array(np.empty((nyr_qtt,ny,nx)),dtype=np.float64)

for yr in range(ystr_qtt,yend_qtt+1):
    suffix = '.' + '{:04}'.format(yr)
    qtotal_file = path_g100 + '/' + base_qtotal_g100 + suffix
    nc_g100 = netCDF4.Dataset(qtotal_file,'r')
    lon = nc_g100.variables['lon'][:]
    lat = nc_g100.variables['lat'][:]
    qtotal_tmp[:,:] = nc_g100.variables['hfds'][0,:,:]
    qtotal_g100[yr-ystr_qtt,:,:] = qtotal_tmp[:,:].copy()
    nc_g100.close()

qtotal_g100 = np.where(qtotal_g100 < -9.0e33, np.NaN, qtotal_g100)
DS_g100_ann = xr.Dataset( {'hfds': (['time','lat','lon'], qtotal_g100),},
                          coords = { 'time': timey_qtt,'lat': lat,'lon': lon, } )

####################################################################
# NP10km

print( "Loading NP10km data" )

path_fill=path_np10km + 'hst_ann-glb_fill'
base_qtotal_fill = 'hs_q_total_wocorr_woa1x1'

qtotal_fill = np.array(np.empty((nyr_qtt,ny,nx)),dtype=np.float64)

for yr in range(ystr_qtt,yend_qtt+1):
    suffix = '.' + '{:04}'.format(yr)
    qtotal_file = path_fill + '/y' + '{:04}'.format(yr) + '/' + base_qtotal_fill + suffix
    nc_fill = netCDF4.Dataset(qtotal_file,'r')
    lon = nc_fill.variables['lon'][:]
    lat = nc_fill.variables['lat'][:]
    qtotal_tmp[:,:] = nc_fill.variables['hfds'][0,:,:]
    qtotal_fill[yr-ystr_qtt,:,:] = qtotal_tmp[:,:].copy()
    nc_fill.close()

qtotal_fill = np.where(qtotal_fill < -9.0e33, np.NaN, qtotal_fill)
DS_fill_ann = xr.Dataset( {'hfds': (['time','lat','lon'], qtotal_fill),},
                          coords = { 'time': timey_qtt,'lat': lat,'lon': lon, } )

#

dq1=DS_g100_ann['hfds'].mean(dim='time').values
dq2=DS_fill_ann['hfds'].mean(dim='time').values

DS_qtt = xr.Dataset( {'exp1': (['lat','lon'], dq1-DS_qttobs['qtotal'].values),
                      'exp2': (['lat','lon'], dq2-DS_qttobs['qtotal'].values),
                      'exp2-1': (['lat','lon'], dq2-dq1),
                      'obs':(['lat','lon'], DS_qttobs['qtotal'].values), },
                     coords = { 'lat': np.linspace(-89.5,89.5,num=180), 
                                'lon': np.linspace(0.5,359.5,num=360), } )

###########################################################################

print( 'Start drawing' )
fig = plt.figure(figsize=(11.0,8.5))

#fig.suptitle( suptitle, fontsize=16, y=0.99 )

proj = ccrs.PlateCarree(central_longitude=-140.)
lon_formatter = LongitudeFormatter(zero_direction_label=True)
lat_formatter = LatitudeFormatter()

# [left, bottom, width, height]

ax = [
    plt.axes([0.04,0.795,0.22,0.145],projection=proj),
    plt.axes([0.285,0.795,0.22,0.145],projection=proj),
    plt.axes([0.53,0.795,0.22,0.145],projection=proj),
    plt.axes([0.775,0.795,0.22,0.145],projection=proj),
    plt.axes([0.04,0.555,0.22,0.145],projection=proj),
    plt.axes([0.285,0.555,0.22,0.145],projection=proj),
    plt.axes([0.53,0.555,0.22,0.145],projection=proj),
    plt.axes([0.775,0.555,0.22,0.145],projection=proj),
    plt.axes([0.04,0.315,0.22,0.145],projection=proj),
    plt.axes([0.285,0.315,0.22,0.145],projection=proj),
    plt.axes([0.53,0.315,0.22,0.145],projection=proj),
    plt.axes([0.775,0.315,0.22,0.145],projection=proj),
    plt.axes([0.04,0.075,0.22,0.145],projection=proj),
    plt.axes([0.285,0.075,0.22,0.145],projection=proj),
    plt.axes([0.53,0.075,0.22,0.145],projection=proj),
    plt.axes([0.775,0.075,0.22,0.145],projection=proj)
]

# [left, bottom, width, height]

ax_cbar = [
    plt.axes([0.035,0.76,0.22,0.011]),
    plt.axes([0.280,0.76,0.22,0.011]),
    plt.axes([0.525,0.76,0.22,0.011]),
    plt.axes([0.770,0.76,0.22,0.011]),
    plt.axes([0.035,0.52,0.22,0.011]),
    plt.axes([0.280,0.52,0.22,0.011]),
    plt.axes([0.525,0.52,0.22,0.011]),
    plt.axes([0.770,0.52,0.22,0.011]),
    plt.axes([0.035,0.28,0.22,0.011]),
    plt.axes([0.280,0.28,0.22,0.011]),
    plt.axes([0.525,0.28,0.22,0.011]),
    plt.axes([0.770,0.28,0.22,0.011]),
    plt.axes([0.035,0.04,0.22,0.011]),
    plt.axes([0.280,0.04,0.22,0.011]),
    plt.axes([0.525,0.04,0.22,0.011]),
    plt.axes([0.770,0.04,0.22,0.011]),
]

plt.text(0.160,0.97,'Low Resolution',ha='center',size=16,transform=fig.transFigure)
plt.text(0.400,0.97,'High Resolution',ha='center',size=16,transform=fig.transFigure)
plt.text(0.640,0.97,'HighRes - LowRes',ha='center',size=16,transform=fig.transFigure)
plt.text(0.880,0.97,'Observation',ha='center',size=16,transform=fig.transFigure)

cmap = [[ 'RdBu_r', 'RdBu_r', 'bwr', 'RdYlBu_r' ],
        [ 'RdBu_r', 'RdBu_r', 'bwr', 'RdYlBu_r' ],
        [ 'RdBu_r', 'RdBu_r', 'bwr', 'RdYlBu_r' ],
        [ 'RdBu_r', 'RdBu_r', 'bwr', 'twilight_shifted' ]]

#        [ 'RdYlBu_r', 'RdYlBu_r', 'bwr', 'RdYlBu_r' ],
#        [ 'twilight_shifted', 'twilight_shifted', 'bwr', 'twilight_shifted' ]]

unit = ['[$^\circ$C]', '[cm]', '[m]', '[$\mathrm{W}\,\mathrm{m}^{-2}$]']
var = [ 'exp1', 'exp2', 'exp2-1', 'obs' ]


panel = 0
for item in range(4):

    if (item == 0):
        DS = DS_sst
        bounds1 = [-2.0, -1.5, -1.0, -0.7, -0.4, -0.1, 0.1, 0.4, 0.7, 1.0, 1.5, 2.0]
        bounds2 = [-1.0, -0.7, -0.4, -0.3, -0.2, -0.1, 0.1, 0.2, 0.3, 0.4, 0.7, 1.0]
        bounds3 = np.arange(-1,30.1,1)
        ticks_bounds3 = [0, 5, 10, 15, 20, 25, 30] 

    if (item == 1):
        DS = DS_zos * 100
        #bounds1 = [-1.0, -0.7, -0.5, -0.3, -0.2, -0.1, -0.05, 0., 0.05, 0.1, 0.2, 0.3, 0.5, 0.7, 1.0 ]
        #bounds2 = [-0.3, -0.25, -0.2, -0.15, -0.1, -0.05, -0.02, 0., 0.02, 0.05, 0.1, 0.15, 0.2, 0.25, 0.3]
        #bounds3 = np.arange(-0.8,1.201,0.1)
        #ticks_bounds3 = np.arange(-0.8,1.201,0.2)
        bounds1 = [-50, -30, -20, -10, -5, 0., 5, 10, 20, 30, 50]
        bounds2 = [-30, -25, -20, -15, -10, -5, -2, 0, 2, 5, 10, 15, 20, 25, 30]
        bounds3 = np.arange(-80,120.1,10)
        ticks_bounds3 = np.arange(-80,120.1,20)

    if (item == 2):
        DS = DS_mld
        #bounds1 = np.array([0,10,20,30,50,75,100,125,150,200,250,300,350,400,500])
        #ticks_bounds1 = np.array([0,10,20,30,50,75,100,125,150,200,250,300,350,400,500])
        bounds1 =       [-200, -150, -100, -50, -20, -10, 10, 20, 50, 100, 150, 200]
        ticks_bounds1 = [-200, -150, -100, -50, -20, -10, 10, 20, 50, 100, 150, 200]
        bounds2 =       [-200, -100, -50, -30, -20, -10, -5, 5, 10, 20, 30, 50, 100, 200]
        ticks_bounds2 = [-200, -100, -50, -30, -20, -10, -5, 5, 10, 20, 30, 50, 100, 200]
        bounds3 = [0,10,20,30,50,75,100,125,150,200,250,300,350,400,500]
        ticks_bounds3=[0,10,20,30,50,75,100,125,150,200,250,300,350,400,500]

    if (item == 3):
        DS = DS_qtt
        #bounds1 = [-200.0, -150.0, -100.0, -50.0, -25.0, -10.0, 0, 10.0, 25.0, 50.0, 100.0, 150.0, 200.0]
        bounds1 = [-100.0, -50.0, -30.0, -20.0, -10.0, -5.0,  0,  5.0, 10.0, 20.0, 30.0, 50.0, 100.0]
        bounds2 = [-50.0, -30.0, -20.0, -15.0, -10.0, -5.0, -2.0, 0, 2.0, 5.0, 10.0, 15.0, 20.0, 30.0, 50.0]
        bounds3 = [-200.0, -150.0, -100.0, -50.0, -25.0, -10.0, 0, 10.0, 25.0, 50.0, 100.0, 150.0, 200.0]
        ticks_bounds3 = [-200.0, -150.0, -100.0, -50.0, -25.0, -10.0, 0, 10.0, 25.0, 50.0, 100.0, 150.0, 200.0]


    for nv_out in range(4):

        if (nv_out == 0 or nv_out == 1):
            bounds = bounds1
            ticks_bounds = bounds1
            da = DS[var[nv_out]]
            msktmp = np.where( np.isnan(da.values), 0.0, 1.0 )
            if (item == 0):
                msktmp = msktmp * da1.values
                datmp = np.where( np.isnan(da.values), 0.0, da.values )
                tmp1 = (datmp * datmp * area * msktmp * maskpjp).sum()
                tmp2 = (area * msktmp * maskpjp).sum()
                tmp3 = (datmp * area * msktmp * maskpjp).sum()
            if (item == 1):
                datmp = np.where( np.isnan(da.values), 0.0, da.values )
                tmp1 = (datmp * datmp * area * msktmp * maskcmems * maskpjp).sum()
                tmp2 = (area * msktmp * maskcmems * maskpjp).sum()
                tmp3 = (datmp * area * msktmp * maskcmems * maskpjp).sum()
            if (item == 2):
                datmp = np.where( np.isnan(da.values), 0.0, da.values )
                tmp1 = (datmp * datmp * area * maskpac).sum()
                tmp2 = (area * msktmp * maskpac).sum()
                tmp3 = (datmp * area * msktmp * maskpac).sum()
            if (item == 3):
                datmp = np.where( np.isnan(da.values), 0.0, da.values )
                tmp1 = (datmp * datmp * area * maskpjp).sum()
                tmp2 = (area * msktmp * maskpjp).sum()
                tmp3 = (datmp * area * msktmp * maskpjp).sum()
            rmse = np.sqrt(tmp1/tmp2)
            mead = tmp3/tmp2
            if (item == 0):
                title_panel = title_top[item][nv_out] + '\n   rmse= ' + '{:.3f}'.format(rmse) + '$^\circ$C,   mean bias = ' + '{:.3f}'.format(mead) + '$^\circ$C'
            if (item == 1):
                title_panel = title_top[item][nv_out] + '\n   rmse= ' + '{:.2f}'.format(rmse) + ' cm,   mean bias = ' + '{:.2f}'.format(mead) + ' cm'
            if (item == 2):
                title_panel = title_top[item][nv_out] + '\n   rmse= ' + '{:.1f}'.format(rmse) + ' m,   mean bias = ' + '{:.1f}'.format(mead) + ' m '
            if (item == 3):
                title_panel = title_top[item][nv_out] + '\n   rmse= ' + '{:.1f}'.format(rmse) + ' W m$^{-2}$,   mean bias = ' + '{:.1f}'.format(mead) + ' W m$^{-2}$'

        elif (nv_out == 2):
            bounds = bounds2
            ticks_bounds = bounds2
            da = DS[var[nv_out]]
            msktmp = np.where( np.isnan(da.values), 0.0, 1.0 )
            if (item == 0):
                datmp = np.where( np.isnan(da.values), 0.0, da.values )
                tmp1 = (datmp * datmp * area * msktmp * maskpjp).sum()
                tmp2 = (area * msktmp * maskpjp).sum()
            if (item == 1):
                datmp = np.where( np.isnan(da.values), 0.0, da.values )
                tmp1 = (datmp * datmp * area * msktmp * maskpjp).sum()
                tmp2 = (area * msktmp * maskpjp).sum()
            if (item == 2):
                datmp = np.where( np.isnan(da.values), 0.0, da.values )
                tmp1 = (datmp * datmp * area * msktmp * maskpac).sum()
                tmp2 = (area * msktmp * maskpac).sum()
            if (item == 3):
                datmp = np.where( np.isnan(da.values), 0.0, da.values )
                tmp1 = (datmp * datmp * area * msktmp * maskpjp).sum()
                tmp2 = (area * msktmp * maskpjp).sum()
            rmsd = np.sqrt(tmp1/tmp2)
            title_panel = title_top[item][nv_out]
            #if (item == 0):
            #    title_panel = title_top[item][nv_out] + '\n   rmsd = ' + '{:.3f}'.format(rmsd) + '$^\circ$C'
            #if (item == 1):
            #    title_panel = title_top[item][nv_out] + '\n   rmsd= ' + '{:.2f}'.format(rmsd) + ' cm '
            #if (item == 2):
            #    title_panel = title_top[item][nv_out] + '\n   rmsd = ' + '{:.1f}'.format(rmsd) + ' m'
            #if (item == 3):
            #    title_panel = title_top[item][nv_out] + '\n   rmsd = ' + '{:.1f}'.format(rmsd) + ' W m$^{-2}$'

        else:
            bounds = bounds3
            ticks_bounds = ticks_bounds3
            title_panel = title_top[item][nv_out]

        da = DS[var[nv_out]]
        print(item,nv_out,title_panel)

        im = da.plot(ax=ax[panel],cmap=cmap[item][nv_out],
                     levels=bounds,
                     extend='both',
                     add_colorbar=False,
                     transform=ccrs.PlateCarree())

        units = unit[item]

        #cbar_kwargs = { 'orientation': 'vertical',
        #                'spacing': 'uniform',
        #                'label': '[$^\circ$C]',
        #                'shrink': 0.8,
        #                'ticks': ticks_bounds, },

        cbar = plt.colorbar(im,orientation='horizontal',spacing='uniform',
                            ticks=ticks_bounds, cax=ax_cbar[panel])
        cbar.set_label(label=units,fontsize=6,x=0.95,labelpad=0)
        cbar.ax.tick_params(labelsize=6)

        ax[panel].coastlines()
        #ax[panel].set_xticks(np.arange(-180,180.1,60),crs=ccrs.PlateCarree())
        #ax[panel].set_yticks(np.arange(-90,90.1,30),crs=ccrs.PlateCarree())
        ax[panel].set_xticks(np.arange(120,285.1,30),crs=ccrs.PlateCarree())
        ax[panel].set_yticks(np.arange(-10,60.1,10),crs=ccrs.PlateCarree())
        ax[panel].set_extent([99,286,-16,66],crs=ccrs.PlateCarree())
        ax[panel].xaxis.set_major_formatter(lon_formatter)
        ax[panel].yaxis.set_major_formatter(lat_formatter)
        ax[panel].set_aspect(1.2)
        ax[panel].set_xlabel('')
        ax[panel].set_ylabel('')
        ax[panel].set_title(title_panel,{'fontsize':8, 'verticalalignment':'top', 'linespacing':0.85})
        #ax[panel].set_title(title_panel,{'fontsize':8, 'verticalalignment':'center'})
        #ax[panel].set_title(title_panel,{'fontsize':8, 'multialignment':'center'})
        ax[panel].tick_params(labelsize=7)
        ax[panel].patch.set_facecolor('lightgray')
        
        panel += 1


#plt.subplots_adjust(left=0.04,right=0.98,bottom=0.06,top=0.96,hspace=0.25,wspace=0.16)

outfile = './fig/SST_SSH_MLD_QTOT_bias_compare_v3'
outpdf = outfile+'.pdf'
outpng = outfile+'.png'
plt.savefig(outpng, bbox_inches='tight', pad_inches=0.05)
plt.savefig(outpdf, bbox_inches='tight', pad_inches=0.05)

plt.show()
