# -*- coding: utf-8 -*-
import sys
import json
import numpy as np
import xarray as xr
import matplotlib.pyplot as plt
import datetime
import netCDF4
from netCDF4 import Dataset
from matplotlib.cm import ScalarMappable
from matplotlib.colors import Normalize

########
#ro = 1036.0
ro_scl = 1024.5

if (len(sys.argv) < 2):
    print ('Usage: ' + sys.argv[0] + ' start_year end_year longitude')
    sys.exit()

styr = int(sys.argv[1])
edyr = int(sys.argv[2])
lon_draw = float(sys.argv[3])

title = [[
 '(a) DIC bias relative to GLODAPv2',
 '(b) DIC bias relative to GLODAPv2',
 '(c) DIC difference (HighRes - Lowres)',
 '(d) DIC GLODAPv2 (normalized to yr 2002)' ],
         [
 '(e) Alk bias relative to GLODAPv2',
 '(f) Alk bias relative to GLODAPv2',
 '(g) Alk difference (HighRes - Lowres)',
 '(h) Alk GLODAPv2' ],
         [
 r'(i) NO$_3$ bias relative to WOA13v2',
 r'(j) NO$_3$ bias relative to WOA13v2',
 r'(k) NO$_3$ difference (HighRes - Lowres)',
 r'(l) NO$_3$ WOA13v2' ], ]

xlim = [ [-20, 65], [-20, 65], [-20, 65], [-20, 65] ]
xtlim = [ [-20, 60], [-20, 60], [-20, 60], [-20, 60] ]
#xlim = [ [9, 61], [9, 61], [9, 61], [9, 61] ]
#xtlim = [ [10, 60], [10, 60], [10, 60], [10, 60] ]

#####
timey = np.empty((edyr+1-styr),dtype='object')
for yr in range(styr,edyr+1):
    timey[yr-styr] = datetime.datetime(yr,1,1)

########
path_glb100km='../data/glb100km/'
path_np10km='../data/np10km/'
path_ref='../data/refdata/'
########

path_glo = path_ref + 'GLODAPv2/'
file_tco2_glodap = 'GLODAPv2.2016b.TCO2.nc'

#path_glo = '/NasB2019_1/hotspot2_a0206/htsujino/gondola_100/exp/run-20210919o2/result_org/ANALY/snap/y1653/'
#file_tco2_glodap = 'rs_DIC_woa1x1.16530101000000'

#path_g100 = '/storage1/C401/htsujino/gondola_100/run-20210607o2/ANALY/annual/'

path_g100 = path_glb100km + 'run-20211104o2/ANALY/annual/'
file_dic_g100_base = 'hs_DIC_woa1x1.'

path_fill= path_np10km + 'hst_ann-glb_fill/'
file_dic_fill_base = 'hs_DIC_woa1x1.'

tv = -9.00e33

file_tco2 = path_glo + file_tco2_glodap
print('Reading GLODAPv2 from '+ file_tco2)
nctco2 = Dataset(file_tco2,'r')
nx = len(nctco2.dimensions['lon'])
ny = len(nctco2.dimensions['lat'])
nz = len(nctco2.dimensions['depth_surface'])
#nz = len(nctco2.dimensions['depth'])
longlo = nctco2.variables['lon'][:]
latglo = nctco2.variables['lat'][:]
depglo = nctco2.variables['Depth'][:]
#depglo = nctco2.variables['depth'][:]
tco2_glo = nctco2.variables['TCO2'][:,:,:]
#tco2_glo = nctco2.variables['dissic'][0,:,:,:]
nctco2.close()
tco2_glo = np.where(tco2_glo < -990, np.nan, tco2_glo*ro_scl*1e-6) 
#tco2_glo = np.where(tco2_glo < tv, np.nan, tco2_glo*1e3) 

longlo_ = np.where(longlo>360,longlo-360,longlo)

DS_glodic = xr.Dataset({'obs': (['depth','lat','lon'], tco2_glo), },
                coords = {'depth': depglo, 'lat': latglo, 'lon': longlo_ } )
DS_glodic = DS_glodic.roll(lon=20, roll_coords=True)

##############

path_glo = path_ref + 'GLODAPv2/'
file_talk_glodap = 'GLODAPv2.2016b.TAlk.nc'

#path_glo = '/NasB2019_1/hotspot2_a0206/htsujino/gondola_100/exp/run-20210919o2/result_org/ANALY/snap/y1653/'
#file_talk_glodap = 'rs_Alk_woa1x1.16530101000000'

#path_g100 = '/storage1/C401/htsujino/gondola_100/run-20210607o2/ANALY/annual/'
#path_g100 = '/NasB2019_1/hotspot2_a0206/htsujino/OMIP2-like-BGC/GONDOLA_100_BGC/exp/run-20180525j/result_org/ANALY/annual/'
path_g100 = path_glb100km + 'run-20211104o2/ANALY/annual/'
path_fill = path_np10km + 'hst_ann-glb_fill/'

file_talk_g100_base = 'hs_Alk_woa1x1.'
file_talk_fill_base = 'hs_Alk_woa1x1.'

tv = -9.00e33

file_talk = path_glo + file_talk_glodap
print('Reading GLODAPv2 from '+ file_talk)
nctalk = Dataset(file_talk,'r')
nx = len(nctalk.dimensions['lon'])
ny = len(nctalk.dimensions['lat'])
nz = len(nctalk.dimensions['depth_surface'])
#nz = len(nctalk.dimensions['depth'])
longlo = nctalk.variables['lon'][:]
latglo = nctalk.variables['lat'][:]
depglo = nctalk.variables['Depth'][:]
#depglo = nctalk.variables['depth'][:]
talk_glo = nctalk.variables['TAlk'][:,:,:]
#talk_glo = nctalk.variables['talk'][0,:,:,:]
nctalk.close()
talk_glo = np.where(talk_glo < -990, np.nan, talk_glo*ro_scl*1e-6) 
#talk_glo = np.where(talk_glo < tv, np.nan, talk_glo*1e3) 

longlo_ = np.where(longlo>360,longlo-360,longlo)
DS_gloalk = xr.Dataset({'obs': (['depth','lat','lon'], talk_glo), },
                coords = {'depth': depglo, 'lat': latglo, 'lon': longlo_ } )
DS_gloalk = DS_gloalk.roll(lon=20, roll_coords=True)

########
# WOA

path_woa=path_ref + 'WOA13v2/1deg_L102/'
file_no3_woa = 'woa13_all_n00_01.nc'
file_no3 = path_woa + file_no3_woa
print('Reading WOA from '+ file_no3)
ncno3 = Dataset(file_no3,'r')
nx = len(ncno3.dimensions['lon'])
ny = len(ncno3.dimensions['lat'])
nz = len(ncno3.dimensions['depth'])
lonwoa = ncno3.variables['lon'][:].astype(np.float_)
latwoa = ncno3.variables['lat'][:].astype(np.float_)
depwoa = ncno3.variables['depth'][:].astype(np.float_)
no3_woa = np.array(np.zeros((nz,ny,nx)),dtype=np.float32)
no3_woa[:,:,:] = ncno3.variables['n_an'][0,:,:,:].astype(np.float32)
ncno3.close()
no3_woa = np.where(no3_woa > 9.9e36, np.nan, no3_woa) 

DS_woano3 = xr.Dataset({'obs': (['depth','lat','lon'], no3_woa), },
                coords = {'depth': depwoa, 'lat': latwoa, 'lon': lonwoa } )
DS_woano3 = DS_woano3.assign_coords(lon=('lon', np.where( DS_woano3.lon < 0, DS_woano3.lon + 360, DS_woano3.lon )))
DS_woano3 = DS_woano3.roll(lon=-180, roll_coords=True)

########
# model

suffix='{year:04}'.format(year=styr)
file_dic = path_g100 + file_dic_g100_base + suffix
print('Reading metadata from '+ file_dic)
ncdic = Dataset(file_dic,'r')
nx = len(ncdic.dimensions['lon'])
ny = len(ncdic.dimensions['lat'])
nz = len(ncdic.dimensions['depth'])
lonmod = ncdic.variables['lon'][:].astype(np.float_)
latmod = ncdic.variables['lat'][:].astype(np.float_)
depmod = ncdic.variables['depth'][:].astype(np.float_)
ncdic.close()

ix=0
londist=abs(lonmod[ix]-lon_draw)
for i in range(1,nx):
   if (abs(lonmod[i]-lon_draw) < londist):
       ix = i
       londist = abs(lonmod[i]-lon_draw)

print("draw along ", lonmod[ix])

dzmod = np.array(np.empty((nz)),dtype=np.float64)
dzmod[0] = 2.0 * depmod[0]
for k in range(1,nz):
    dzmod[k] = 2.0 * ((depmod[k] - depmod[k-1]) - 0.5 * dzmod[k-1])

print("dz ",dzmod)

########
# DIC

dic_g100_all = np.array(np.zeros((edyr-styr+1,nz,ny,nx)),dtype=np.float32)
dic_fill_all = np.array(np.zeros((edyr-styr+1,nz,ny,nx)),dtype=np.float32)

nyr = 0

for yr in range(styr,edyr+1):

    suffix='{year:04}'.format(year=yr)

    file_dic_g100 = path_g100 + file_dic_g100_base + suffix
    print('Reading from '+ file_dic_g100)
    ncdic = Dataset(file_dic_g100,'r')
    dic_glb = ncdic.variables['dissic'][0,:,:,:].astype(np.float_)
    ncdic.close()
    dic_g100_all[nyr,0:nz,0:ny,0:nx] = np.where(dic_glb[0:nz,0:ny,0:nx] < tv, np.nan, dic_glb[0:nz,0:ny,0:nx]) 

    #-------------------------------------------

    file_dic_fill = path_fill + 'y' + str(yr) + '/' + file_dic_fill_base + suffix
    print('Reading from '+ file_dic_fill)
    ncdic = Dataset(file_dic_fill,'r')
    dic_glb = ncdic.variables['dissic'][0,:,:,:].astype(np.float_)
    ncdic.close()
    dic_fill_all[nyr,0:nz-1,0:ny,0:nx] = np.where(dic_glb[0:nz-1,0:ny,0:nx] < tv, np.nan, dic_glb[0:nz-1,0:ny,0:nx])

    nyr = nyr + 1

#--------------------------------------------------------

DS_dic = xr.Dataset({'exp1': (['time','depth','lat','lon'], dic_g100_all),
                     'exp2': (['time','depth','lat','lon'], dic_fill_all),
                     'exp2-1': (['time','depth','lat','lon'], dic_fill_all-dic_g100_all), },
                    coords = {'time': timey, 'depth': depmod, 'lat': latmod, 'lon': lonmod } )

########################

talk_g100_all = np.array(np.zeros((edyr-styr+1,nz,ny,nx)),dtype=np.float32)
talk_fill_all = np.array(np.zeros((edyr-styr+1,nz,ny,nx)),dtype=np.float32)

nyr = 0

for yr in range(styr,edyr+1):

    suffix='{year:04}'.format(year=yr)

    file_talk_g100 = path_g100 + file_talk_g100_base + suffix
    print('Reading from '+ file_talk_g100)
    nctalk = Dataset(file_talk_g100,'r')
    talk_glb = nctalk.variables['talk'][0,:,:,:].astype(np.float_)
    nctalk.close()
    talk_g100_all[nyr,0:nz,0:ny,0:nx] = np.where(talk_glb[0:nz,0:ny,0:nx] < tv, np.nan, talk_glb[0:nz,0:ny,0:nx]) 

    #-------------------------------------------

    file_talk_fill = path_fill + 'y' + str(yr) + '/' + file_talk_fill_base + suffix
    print('Reading from '+ file_talk_fill)
    nctalk = Dataset(file_talk_fill,'r')
    talk_glb = nctalk.variables['talk'][0,:,:,:].astype(np.float_)
    nctalk.close()
    talk_fill_all[nyr,0:nz-1,0:ny,0:nx] = np.where(talk_glb[0:nz-1,0:ny,0:nx] < tv, np.nan, talk_glb[0:nz-1,0:ny,0:nx])

    nyr = nyr + 1

#--------------------------------------------------------

DS_alk = xr.Dataset({'exp1': (['time','depth','lat','lon'], talk_g100_all),
                     'exp2': (['time','depth','lat','lon'], talk_fill_all),
                     'exp2-1': (['time','depth','lat','lon'], talk_fill_all-talk_g100_all), },
                coords = {'time': timey, 'depth': depmod, 'lat': latmod, 'lon': lonmod } )

############################################################################################################################
no3_g100_all = np.array(np.zeros((edyr-styr+1,nz,ny,nx)),dtype=np.float32)
no3_fill_all = np.array(np.zeros((edyr-styr+1,nz,ny,nx)),dtype=np.float32)

nyr = 0

file_no3_g100_base = 'hs_NO3_woa1x1.'
file_no3_fill_base = 'hs_NO3_woa1x1.'

for yr in range(styr,edyr+1):

    suffix='{year:04}'.format(year=yr)

    file_no3_g100 = path_g100 + file_no3_g100_base + suffix
    print('Reading from '+ file_no3_g100)
    ncno3 = Dataset(file_no3_g100,'r')
    no3_glb = ncno3.variables['no3'][0,:,:,:].astype(np.float_)
    ncno3.close()
    no3_g100_all[nyr,0:nz,0:ny,0:nx] = np.where(no3_glb[0:nz,0:ny,0:nx] < tv, np.nan, no3_glb[0:nz,0:ny,0:nx]*1e3) 

    #-------------------------------------------

    file_no3_g100 = path_fill + 'y' + str(yr) + '/' + file_no3_fill_base + suffix
    print('Reading from '+ file_no3_g100)
    ncno3 = Dataset(file_no3_g100,'r')
    no3_glb = ncno3.variables['no3'][0,:,:,:].astype(np.float_)
    ncno3.close()
    no3_fill_all[nyr,0:nz-1,0:ny,0:nx] = np.where(no3_glb[0:nz-1,0:ny,0:nx] < tv, np.nan, no3_glb[0:nz-1,0:ny,0:nx]*1e3)

    nyr = nyr + 1

#--------------------------------------------------------

DS_no3 = xr.Dataset({'exp1': (['time','depth','lat','lon'], no3_g100_all),
                     'exp2': (['time','depth','lat','lon'], no3_fill_all),
                     'exp2-1': (['time','depth','lat','lon'], no3_fill_all-no3_g100_all), },
                coords = {'time': timey, 'depth': depmod, 'lat': latmod, 'lon': lonmod } )


############################################################################################################################
# Draw figures

suptitle = r'Longitude = {:.1f}'.format(lonmod[ix]) + '$^{\circ}$E' + ' (' + str(styr) + '-' + str(edyr) + ' mean)'

fig = plt.figure(figsize=(11,7.5))
fig.suptitle( suptitle, fontsize=14, y=0.985)

# [left, bottom, width, height]
ax = [
    plt.axes([0.04,0.825,0.20,0.07]),
    plt.axes([0.04,0.705,0.20,0.11]),
    plt.axes([0.29,0.825,0.20,0.07]),
    plt.axes([0.29,0.705,0.20,0.11]),
    plt.axes([0.54,0.825,0.20,0.07]),
    plt.axes([0.54,0.705,0.20,0.11]),
    plt.axes([0.79,0.825,0.20,0.07]),
    plt.axes([0.79,0.705,0.20,0.11]),
    plt.axes([0.04,0.520,0.20,0.07]),
    plt.axes([0.04,0.400,0.20,0.11]),
    plt.axes([0.29,0.520,0.20,0.07]),
    plt.axes([0.29,0.400,0.20,0.11]),
    plt.axes([0.54,0.520,0.20,0.07]),
    plt.axes([0.54,0.400,0.20,0.11]),
    plt.axes([0.79,0.520,0.20,0.07]),
    plt.axes([0.79,0.400,0.20,0.11]),
    plt.axes([0.04,0.215,0.20,0.07]),
    plt.axes([0.04,0.095,0.20,0.11]),
    plt.axes([0.29,0.215,0.20,0.07]),
    plt.axes([0.29,0.095,0.20,0.11]),
    plt.axes([0.54,0.215,0.20,0.07]),
    plt.axes([0.54,0.095,0.20,0.11]),
    plt.axes([0.79,0.215,0.20,0.07]),
    plt.axes([0.79,0.095,0.20,0.11]),
]

# [left, bottom, width, height]
ax_cbar = [
    plt.axes([0.03,0.665,0.21,0.0115]),
    plt.axes([0.28,0.665,0.21,0.0115]),
    plt.axes([0.53,0.665,0.21,0.0115]),
    plt.axes([0.78,0.665,0.21,0.0115]),
    plt.axes([0.03,0.360,0.21,0.0115]),
    plt.axes([0.28,0.360,0.21,0.0115]),
    plt.axes([0.53,0.360,0.21,0.0115]),
    plt.axes([0.78,0.360,0.21,0.0115]),
    plt.axes([0.03,0.056,0.21,0.0115]),
    plt.axes([0.28,0.056,0.21,0.0115]),
    plt.axes([0.53,0.055,0.21,0.0115]),
    plt.axes([0.78,0.055,0.21,0.0115]),
]


plt.text(0.150,0.93,'Low Resolution',ha='center',size=14,transform=fig.transFigure)
plt.text(0.395,0.93,'High Resolution',ha='center',size=14,transform=fig.transFigure)
plt.text(0.640,0.93,'HighRes - LowRes',ha='center',size=14,transform=fig.transFigure)
plt.text(0.885,0.93,'Observation',ha='center',size=14,transform=fig.transFigure)

cmap = [[ 'RdBu_r', 'RdBu_r', 'bwr', 'terrain' ],
        [ 'RdBu_r', 'RdBu_r', 'bwr', 'terrain' ],
        [ 'RdBu_r', 'RdBu_r', 'bwr', 'terrain' ],]

units = [r'[$\mathrm{mol}\, \mathrm{m}^{-3}$]',r'[$\mathrm{mol}\, \mathrm{m}^{-3}$]',r'[$\mathrm{\mu mol}\, \mathrm{l}^{-1}$]']

var = [ 'exp1', 'exp2', 'exp2-1', 'obs' ]
 
panel = 0
cbars = 0
for item in range(3):
    if (item == 0):
        DS_mod=DS_dic
        DS_obs=DS_glodic
        bounds1 = np.linspace(1.950,2.450,num=21)
        tbounds1 = np.linspace(1.950,2.450,num=11)
        bounds2 = np.linspace(-0.225,0.225,num=19)
        tbounds2 = np.linspace(-0.200,0.200,num=9)
        bounds3 = np.linspace(-0.060,0.060,num=25)
        tbounds3 = np.linspace(-0.060,0.060,num=7)

    if (item == 1):
        DS_mod=DS_alk
        DS_obs=DS_gloalk
        bounds1 = np.linspace(2.200,2.500,num=16)
        tbounds1 = np.linspace(2.200,2.500,num=6)
        bounds2 = np.linspace(-0.100,0.100,num=21)
        tbounds2 = np.linspace(-0.100,0.100,num=6)
        bounds3 = np.linspace(-0.025,0.025,num=11)
        tbounds3 = np.linspace(-0.020,0.020,num=5)

    if (item == 2):
        DS_mod=DS_no3
        DS_obs=DS_woano3
        bounds1 = [0.0, 0.1, 0.2, 1.0, 2.0, 5.0, 10.0, 15.0, 20.0, 25.0, 30.0, 35.0, 40.0, 45.0, 50.0]
        tbounds1 = [0.0, 0.2, 2.0, 10.0, 20.0, 30.0, 40.0, 50.0]
        bounds3 = [-20.0, -10.0, -5.0, -2.0, -1.0, -0.5, -0.1, 0.0, 0.1, 0.5, 1.0, 2.0, 5.0, 10.0, 20.0]
        tbounds3 = [-10.0, -2.0, -0.5, 0.0, 0.5, 2.0, 10.0]
        bounds2 = bounds3
        tbounds2 = tbounds3

    for nv_out in range(4):

        if (nv_out == 0):
            da1 = DS_mod[var[nv_out]].isel(lon=ix).sel(depth=slice(0,1050)).sel(time=slice(str(styr),str(edyr))).mean(dim='time',skipna=False) - DS_obs['obs'].isel(lon=ix).interp(depth=depmod).sel(depth=slice(0,1050))
            da2 = DS_mod[var[nv_out]].isel(lon=ix).sel(depth=slice(0,6500)).sel(time=slice(str(styr),str(edyr))).mean(dim='time',skipna=False) - DS_obs['obs'].isel(lon=ix).interp(depth=depmod).sel(depth=slice(0,6500))

            tmp = DS_mod[var[nv_out]].isel(lon=ix).sel(lat=slice(12,60)).sel(depth=slice(0,1000)).sel(time=slice(str(styr),str(edyr))).mean(dim='time',skipna=False) - DS_obs['obs'].isel(lon=ix).sel(lat=slice(12,60)).interp(depth=depmod).sel(depth=slice(0,1000))
            nza, nya = tmp.shape
            dztmp = np.tile(dzmod[0:nza],nya).reshape(nya,nza).T
            #print(tmp.values.shape,dztmp.shape)
            #print(dztmp)
            #print(tmp.values)
            msktmp = np.where( np.isnan(tmp.values), 0.0, 1.0 )
            datmp = np.where( np.isnan(tmp.values), 0.0, tmp.values )
            tmp1 = (datmp * datmp * msktmp * dztmp).sum()
            tmp2 = (msktmp * dztmp).sum()
            tmp3 = (datmp * msktmp * dztmp).sum()
            rmse = np.sqrt(tmp1/tmp2)
            mead = tmp3/tmp2
            title_tmp = title[item][nv_out] + '\n   rmse= ' + '{:.3f}'.format(rmse) +',  mean bias = ' + '{:.3f}'.format(mead) + units[item]
            bounds = bounds2
            ticks_bounds = tbounds2
        elif (nv_out == 1):
            da1 = DS_mod[var[nv_out]].isel(lon=ix).sel(depth=slice(0,1050)).sel(time=slice(str(styr),str(edyr))).mean(dim='time',skipna=False) - DS_obs['obs'].isel(lon=ix).interp(depth=depmod).sel(depth=slice(0,1050))
            da2 = DS_mod[var[nv_out]].isel(lon=ix).sel(depth=slice(0,6500)).sel(time=slice(str(styr),str(edyr))).mean(dim='time',skipna=False) - DS_obs['obs'].isel(lon=ix).interp(depth=depmod).sel(depth=slice(0,6500))
            tmp = DS_mod[var[nv_out]].isel(lon=ix).sel(lat=slice(12,60)).sel(depth=slice(0,1000)).sel(time=slice(str(styr),str(edyr))).mean(dim='time',skipna=False) - DS_obs['obs'].isel(lon=ix).sel(lat=slice(12,60)).interp(depth=depmod).sel(depth=slice(0,1000))
            nza, nya = tmp.shape
            dztmp = np.tile(dzmod[0:nza],nya).reshape(nya,nza).T
            #print(tmp.values.shape,dztmp.shape)
            #print(dztmp)
            #print(tmp.values)
            msktmp = np.where( np.isnan(tmp.values), 0.0, 1.0 )
            datmp = np.where( np.isnan(tmp.values), 0.0, tmp.values )
            tmp1 = (datmp * datmp * msktmp * dztmp).sum()
            tmp2 = (msktmp * dztmp).sum()
            tmp3 = (datmp * msktmp * dztmp).sum()
            rmse = np.sqrt(tmp1/tmp2)
            mead = tmp3/tmp2
            title_tmp = title[item][nv_out] + '\n   rmse= ' + '{:.3f}'.format(rmse) +',  mean bias = ' + '{:.3f}'.format(mead) + units[item]
            bounds = bounds2
            ticks_bounds = tbounds2
        elif (nv_out == 2):
            da1 = DS_mod[var[nv_out]].isel(lon=ix).sel(depth=slice(0,1050)).sel(time=slice(str(styr),str(edyr))).mean(dim='time',skipna=False)
            da2 = DS_mod[var[nv_out]].isel(lon=ix).sel(depth=slice(0,6500)).sel(time=slice(str(styr),str(edyr))).mean(dim='time',skipna=False)
            title_tmp = title[item][nv_out]
            bounds = bounds3
            ticks_bounds = tbounds3
        else:
            da1 = DS_obs[var[nv_out]].isel(lon=ix).sel(depth=slice(0,1050))
            da2 = DS_obs[var[nv_out]].isel(lon=ix).sel(depth=slice(0,6500))
            title_tmp = title[item][nv_out]
            bounds = bounds1
            ticks_bounds = tbounds1

        da1.plot(ax=ax[panel],cmap=cmap[item][nv_out],
                 levels=bounds,
                 extend='both',
                 add_colorbar=False,)

        ax[panel].grid(linestyle=':')
        #ax[panel].set_xlabel('latitude',fontsize=7,labelpad=0)
        ax[panel].set_xlabel('')
        ax[panel].set_ylabel('depth',fontsize=7,labelpad=0)
        ax[panel].set_title(title_tmp,{'fontsize':8,'verticalalignment':'top','linespacing':0.8})
        #ax[panel].set_title(title[item][nv_out],{'fontsize':8,'verticalalignment':'center'})
        #ax[panel].set_title(title[item][nv_out],{'fontsize':8,'verticalalignment':'top'})
        #ax[panel].set_title(title[item][nv_out],{'fontsize':8})
        ax[panel].invert_yaxis()
        ax[panel].set_xlim(xlim[nv_out][0],xlim[nv_out][1])
        ax[panel].set_xticks(np.arange(xtlim[nv_out][0],xtlim[nv_out][1]+0.1,10))
        ax[panel].set_ylim(1050,0)
        ax[panel].set_yticks(np.arange(0,1050,200))
        ax[panel].tick_params(labelsize=7)
        ax[panel].tick_params(axis='x',labelbottom=False)
        ax[panel].set_facecolor('lightgray')

        panel += 1

        im = da2.plot(ax=ax[panel],cmap=cmap[item][nv_out],
                     levels=bounds,
                     extend='both',
                     add_colorbar=False,)
        
        cbar = plt.colorbar(im,orientation='horizontal',spacing='uniform',
                            ticks=ticks_bounds, cax=ax_cbar[cbars])
        cbar.set_label(label=units[item],fontsize=6,x=0.95,labelpad=0)
        cbar.ax.tick_params(labelsize=6)

        ax[panel].grid(linestyle=':')
        #ax[panel].set_xlabel('latitude',fontsize=7,labelpad=0)
        ax[panel].set_xlabel('')
        ax[panel].set_ylabel('depth',fontsize=7,labelpad=0)
        #ax[panel].set_title(title[item][nv_out],{'fontsize':8,'verticalalignment':'top'})
        ax[panel].set_title('')
        ax[panel].invert_yaxis()
        ax[panel].set_xlim(xlim[nv_out][0],xlim[nv_out][1])
        ax[panel].set_xticks(np.arange(xtlim[nv_out][0],xtlim[nv_out][1]+0.1,10))
        ax[panel].set_ylim(6350,0)
        ax[panel].set_yticks(np.arange(0,6001,1000))
        ax[panel].tick_params(labelsize=7)
        ax[panel].set_facecolor('lightgray')

        panel+=1
        cbars+=1

#plt.subplots_adjust(left=0.07,right=0.98,bottom=0.05,top=0.92,wspace=0.16,hspace=0.15)

outpdf = './fig/TCO2_Alk_NO3_' + str(int(lon_draw)) + 'section_v3.pdf'
outpng = './fig/TCO2_Alk_NO3_' + str(int(lon_draw)) + 'section_v3.png'
plt.savefig(outpdf, bbox_inches='tight', pad_inches=0.1)
plt.savefig(outpng, bbox_inches='tight', pad_inches=0.1)
print("figures are saved to " + outpng + " and " + outpdf)

plt.show()
