# -*- coding: utf-8 -*-
import sys
import json
import numpy as np
import netCDF4
import xarray as xr
import cartopy.crs as ccrs
import matplotlib as mpl
import matplotlib.pyplot as plt
from cartopy.mpl.ticker import LongitudeFormatter, LatitudeFormatter
import datetime

########

if (len(sys.argv) < 3):
    print ('Usage: ' + sys.argv[0] + ' exp1 exp2 ')
    sys.exit()

ystr_pco=1990
yend_pco=2018
nyr_pco=yend_pco-ystr_pco+1

#ystr_int = 1987
#yend_int = 2017
ystr_int = 2003
yend_int = 2018
nyr_int = yend_int - ystr_int + 1

exp1=sys.argv[1]
exp2=sys.argv[2]

#suptitle = 'pCO2 (ave. from '+str(ystr)+' to '+str(yend)+')'
outfile = './fig/pCO2_INTPP_comp_v2'

####################################################################
path_ref='../data/refdata/'
path_np10km='../data/np10km/'
path_glb100km='../data/glb100km/'
####################################################################
# Mask to select Pacific Ocean
#
print("Loading WOA13v2 mask")

basinfile = path_ref + 'WOA13v2/1deg_L33/const/WOA13_1x1_mask.nc'

ncbas = netCDF4.Dataset(basinfile,'r')
basin = ncbas.variables['basin_mask'][:,:]
lonm = ncbas['lon'][:]
latm = ncbas['lat'][:]
ncbas.close()

nx = len(lonm)
ny = len(latm)

maskpac = np.array(np.empty((ny,nx)),dtype=np.int64)
maskpac[:,:] = 0
maskpjp = np.array(np.empty((ny,nx)),dtype=np.int64)
maskpjp[:,:] = 0

for j in range(ny):
    if (latm[j] > 12.0 and latm[j] < 63.0):
        for i in range(nx):
            if (basin[j,i] == 2):
                maskpac[j,i] = 1
                maskpjp[j,i] = 1
            if (basin[j,i] == 12):
                maskpjp[j,i] = 1

print("Loading PCMDI-AMIP-SST area")

arefile = path_ref + 'PCMDI-SST/areacello_input4MIPs_SSTsAndSeaIce_CMIP_PCMDI-AMIP-1-1-6_gn.nc'
ncare = netCDF4.Dataset(arefile,'r')
area = ncare.variables['areacello'][:,:]
ncare.close()

####################################################################
# pCO2
# Ref data

print( "Loading JMA data" )

path_jma = path_ref + "JMA_CO2flux"
time_ref = np.empty(nyr_pco*12,dtype='object')
reffile = path_jma + "/JMA_co2map_1990.nc"
ncref = netCDF4.Dataset(reffile,'r')
lon = ncref.variables['lon'][:]
lat = ncref.variables['lat'][:]
nx = len(ncref.dimensions['lon'])
ny = len(ncref.dimensions['lat'])
ncref.close()

spco2_ref = np.array(np.empty((nyr_pco*12,ny,nx)),dtype=np.float64)
spco2_tmp = np.array(np.empty((12,ny,nx)),dtype=np.float64)

for yr in range(ystr_pco,yend_pco+1):
    for mon in range(1,13):
        time_ref[(yr-ystr_pco)*12+mon-1] = datetime.datetime(yr,mon,1)

    reffile = path_jma + "/JMA_co2map_" + str(yr) + ".nc"
    ncref = netCDF4.Dataset(reffile,'r')
    spco2_tmp = ncref.variables['pCO2s'][:,:,:]
    ncref.close()
    smon = (yr-ystr_pco)*12
    emon = (yr-ystr_pco+1)*12
    spco2_ref[smon:emon,:,:] = spco2_tmp[0:12,:,:]

spco2_ref = np.where(spco2_ref < -990, np.NaN, spco2_ref)

DS_ref = xr.Dataset( {'spco2': (['time','lat','lon'], spco2_ref),},
                      coords = { 'time': time_ref,'lat': lat,'lon': lon, } )

DS_ref_ann = DS_ref.resample(time='1YS').mean()

#print(DS_ref_ann.spco2)

####################################################################

timey_pco = np.empty(yend_pco-ystr_pco+1,dtype='object')
for yr in range(ystr_pco,yend_pco+1):
    timey_pco[yr-ystr_pco] = datetime.datetime(yr,1,1)

spco2_tmp = np.array(np.empty((ny,nx)),dtype=np.float64)

####################################################################
# GLB100km

print( "Loading GLB100km data" )


path_g100 = path_glb100km + 'run-20211104o2/ANALY/annual'
spco2_g100 = np.array(np.empty((nyr_pco,ny,nx)),dtype=np.float64)
basename_g100 = 'hs_pCO2_woa1x1'

for yr in range(ystr_pco,yend_pco+1):
    suffix = '.' + '{:04}'.format(yr)
    spco2_file = path_g100 + '/' + basename_g100 + suffix
    ncg100 = netCDF4.Dataset(spco2_file,'r')
    lon = ncg100.variables['lon'][:]
    lat = ncg100.variables['lat'][:]
    spco2_tmp[:,:] = ncg100.variables['spco2'][0,:,:]

    spco2_g100[yr-ystr_pco,:,:] = spco2_tmp[:,:]

spco2_g100 = np.where(spco2_g100 < -9.0e33, np.NaN, spco2_g100 / 1013.25e2 * 1e6) # Pa --> uatm
DS_g100_ann = xr.Dataset( {'spco2': (['time','lat','lon'], spco2_g100),},
                          coords = { 'time': timey_pco,'lat': lat,'lon': lon, } )

####################################################################
# NP10km

print( "Loading NP10km data" )

path_fill=path_np10km + 'hst_ann-glb_fill'
spco2_fill = np.array(np.empty((nyr_pco,ny,nx)),dtype=np.float64)
basename_fill = 'hs_pCO2_woa1x1'

for yr in range(ystr_pco,yend_pco+1):
    suffix = '.' + '{:04}'.format(yr)
    spco2_file = path_fill + '/y' + '{:04}'.format(yr) + '/' + basename_fill + suffix
    ncfill = netCDF4.Dataset(spco2_file,'r')
    lon = ncfill.variables['lon'][:]
    lat = ncfill.variables['lat'][:]
    spco2_tmp[:,:] = ncfill.variables['spco2'][0,:,:]
    spco2_fill[yr-ystr_pco,:,:] = spco2_tmp[:,:]

spco2_fill = np.where(spco2_fill < -9.0e33, np.NaN, spco2_fill / 1013.25e2 * 1e6)
DS_fill_ann = xr.Dataset( {'spco2': (['time','lat','lon'], spco2_fill),},
                          coords = { 'time': timey_pco,'lat': lat,'lon': lon, } )

print(DS_fill_ann)

####################################################################

bias = np.empty( (4,180,360) )

da0 = DS_ref_ann.spco2.sel(time=slice(str(ystr_pco),str(yend_pco))).mean(dim='time',skipna=False)
da1 = DS_g100_ann.spco2.sel(time=slice(str(ystr_pco),str(yend_pco))).mean(dim='time',skipna=False)
da2 = DS_fill_ann.spco2.sel(time=slice(str(ystr_pco),str(yend_pco))).mean(dim='time',skipna=False)

DS_pco = xr.Dataset( {'exp1': (['lat','lon'], da1.values-da0.values),
                      'exp2': (['lat','lon'], da2.values-da0.values),
                      'exp2-1': (['lat','lon'], da2.values - da1.values),
                      'obs':   (['lat','lon'], da0.values), },
                     coords = { 'lat': np.linspace(-89.5,89.5,num=180), 
                                'lon': np.linspace(0.5,359.5,num=360), } )

###########################################################################
# Net Primary Production
# Ref data

path_osu = path_ref+"NPP_Satellite_OSU"
time_ref_int = np.empty(nyr_int*12,dtype='object')

print( "Loading NPP Satellite OSU data" )

reffile = path_osu + "/StandardVGPM/netCDF_1x1/vgpm_1x1.2003.nc"
ncref = netCDF4.Dataset(reffile,'r')
loni = ncref.variables['lon'][:]
lati = ncref.variables['lat'][:]
nxi = len(ncref.dimensions['lon'])
nyi = len(ncref.dimensions['lat'])
ncref.close()

intpp_standard_ref = np.array(np.empty((nyr_int*12,nyi,nxi)),dtype=np.float64)
intpp_eppley_ref = np.array(np.empty((nyr_int*12,nyi,nxi)),dtype=np.float64)
intpp_cbpm_ref = np.array(np.empty((nyr_int*12,nyi,nxi)),dtype=np.float64)

intpp_tmp = np.array(np.empty((12,nyi,nxi)),dtype=np.float64)

for yr in range(ystr_int,yend_int+1):
    for mon in range(1,13):
        time_ref_int[(yr-ystr_int)*12+mon-1] = datetime.datetime(yr,mon,1)

    smon = (yr-ystr_int)*12
    emon = (yr-ystr_int+1)*12

    # Standard
    reffile = path_osu + "/StandardVGPM/netCDF_1x1/vgpm_1x1." + str(yr) + ".nc"
    ncref = netCDF4.Dataset(reffile,'r')
    intpp_tmp = ncref.variables['npp'][:,:,:]
    ncref.close()
    intpp_standard_ref[smon:emon,:,:] = intpp_tmp[0:12,:,:]

    # Eppley
    reffile = path_osu + "/Eppley-VGPM/netCDF_1x1/eppley_1x1." + str(yr) + ".nc"
    ncref = netCDF4.Dataset(reffile,'r')
    intpp_tmp = ncref.variables['npp'][:,:,:]
    ncref.close()
    intpp_eppley_ref[smon:emon,:,:] = intpp_tmp[0:12,:,:]

    # CbPM
    reffile = path_osu + "/updatedCbPM/netCDF_1x1/cbpm_1x1." + str(yr) + ".nc"
    ncref = netCDF4.Dataset(reffile,'r')
    intpp_tmp = ncref.variables['npp'][:,:,:]
    ncref.close()
    intpp_cbpm_ref[smon:emon,:,:] = intpp_tmp[0:12,:,:]

intpp_standard_ref = np.where(intpp_standard_ref < -9990, np.NaN, intpp_standard_ref*365.25*1e-3/12)
intpp_eppley_ref = np.where(intpp_eppley_ref < -9990, np.NaN, intpp_eppley_ref*365.25*1e-3/12)
intpp_cbpm_ref = np.where(intpp_cbpm_ref < -9990, np.NaN, intpp_cbpm_ref*365.25*1e-3/12)

DS_intpp_ref = xr.Dataset( {'npp': (['time','lat','lon'], (intpp_standard_ref+intpp_eppley_ref+intpp_cbpm_ref)/3),},
                  coords = {'time': time_ref_int,'lat': lati,'lon': loni, } )
#DS_intpp_ref = xr.Dataset( {'npp': (['time','lat','lon'], intpp_eppley_ref),},
#                  coords = {'time': time_ref_int,'lat': lati,'lon': loni, } )

DS_intpp_ref_ann = DS_intpp_ref.resample(time='1YS').mean()

timey_int = np.empty(yend_int-ystr_int+1,dtype='object')
for yr in range(ystr_int,yend_int+1):
    timey_int[yr-ystr_int] = datetime.datetime(yr,1,1)

####################################
# GLB100km

print( "Loading GLB100km data" )

path_g100 = path_glb100km + 'run-20211104o2/ANALY/annual'
basename_g100 = 'hs_pprdc_woa1x1'

suffix = '.' + '{:04}'.format(ystr_int)
intpp_file = path_g100 + '/' + basename_g100 + suffix
ncg100 = netCDF4.Dataset(intpp_file,'r')
lon = ncg100.variables['lon'][:]
lat = ncg100.variables['lat'][:]
nx = len(ncg100.dimensions['lon'])
ny = len(ncg100.dimensions['lat'])
ncg100.close()

print(str(nx),str(ny))

intpp_tmp = np.array(np.empty((ny,nx)),dtype=np.float64)
intpp_g100 = np.array(np.empty((nyr_int,ny,nx)),dtype=np.float64)

for yr in range(ystr_int,yend_int+1):
    suffix = '.' + '{:04}'.format(yr)
    intpp_file = path_g100 + '/' + basename_g100 + suffix
    ncg100 = netCDF4.Dataset(intpp_file,'r')
    intpp_tmp[:,:] = ncg100.variables['intpp'][0,:,:] # mol/m2/sec --> mol/m2/yr
    ncg100.close()

    intpp_g100[yr-ystr_int,:,:] = intpp_tmp[:,:]

intpp_g100 = np.where(intpp_g100 < -9.0e33, np.NaN, intpp_g100 * 365.25 * 86400)
DS_g100_int = xr.Dataset( {'intpp': (['time','lat','lon'], intpp_g100),},
                          coords = { 'time': timey_int,'lat': lat,'lon': lon, } )

####################################################################
# NP10km

print( "Loading NP10km data" )

path_fill=path_np10km + 'hst_ann-glb_fill'
intpp_fill = np.array(np.empty((nyr_int,ny,nx)),dtype=np.float64)
basename_fill = 'hs_pprdc_woa1x1'
for yr in range(ystr_int,yend_int+1):
    suffix = '.' + '{:04}'.format(yr)
    intpp_file = path_fill + '/y' + '{:04}'.format(yr) + '/' + basename_fill + suffix
    ncfill = netCDF4.Dataset(intpp_file,'r')
    lon = ncfill.variables['lon'][:]
    lat = ncfill.variables['lat'][:]
    intpp_tmp[:,:] = ncfill.variables['intpp'][0,:,:]
    intpp_fill[yr-ystr_int,:,:] = intpp_tmp[:,:]
    ncfill.close()

intpp_fill = np.where(intpp_fill < -9.0e33, np.NaN, intpp_fill * 365.25 * 86400 )
DS_fill_int = xr.Dataset( {'intpp': (['time','lat','lon'], intpp_fill),},
                          coords = { 'time': timey_int,'lat': lat,'lon': lon, } )

print(DS_fill_int)

####################################################################

bias = np.empty( (4,180,360) )

da0 = DS_intpp_ref_ann.npp.sel(time=slice(str(ystr_int),str(yend_int))).mean(dim='time',skipna=False)
da1 = DS_g100_int.intpp.sel(time=slice(str(ystr_int),str(yend_int))).mean(dim='time',skipna=False)
da2 = DS_fill_int.intpp.sel(time=slice(str(ystr_int),str(yend_int))).mean(dim='time',skipna=False)

DS_int = xr.Dataset( {'exp1': (['lat','lon'], da1.values),
                      'exp2': (['lat','lon'], da2.values),
                      'exp2-1': (['lat','lon'], da2.values-da1.values),
                      'obs':   (['lat','lon'], da0.values), },
                     coords = { 'lat': np.linspace(-89.5,89.5,num=180), 
                                'lon': np.linspace(0.5,359.5,num=360), } )

###########################################################################

print( 'Start drawing' )
fig = plt.figure(figsize=(11,4.5))
#fig.suptitle( suptitle, fontsize=18 )

proj = ccrs.PlateCarree(central_longitude=-140.)
lon_formatter = LongitudeFormatter(zero_direction_label=True)
lat_formatter = LatitudeFormatter()

# [left, bottom, width, height]
ax = [
    plt.axes([0.05,0.56,0.20,0.38],projection=proj),
    plt.axes([0.29,0.56,0.20,0.38],projection=proj),
    plt.axes([0.53,0.56,0.20,0.38],projection=proj),
    plt.axes([0.77,0.56,0.20,0.38],projection=proj),
    plt.axes([0.05,0.12,0.20,0.38],projection=proj),
    plt.axes([0.29,0.12,0.20,0.38],projection=proj),
    plt.axes([0.53,0.12,0.20,0.38],projection=proj),
    plt.axes([0.77,0.12,0.20,0.38],projection=proj),
]

# [left, bottom, width, height]
ax_cbar = [
    plt.axes([0.03,0.54,0.23,0.012]),
    plt.axes([0.27,0.54,0.23,0.012]),
    plt.axes([0.51,0.54,0.23,0.012]),
    plt.axes([0.75,0.54,0.23,0.012]),
    plt.axes([0.03,0.10,0.23,0.012]),
    plt.axes([0.27,0.10,0.23,0.012]),
    plt.axes([0.51,0.10,0.23,0.012]),
    plt.axes([0.75,0.10,0.23,0.012]),
]

plt.text(0.160,0.955,'Low Resolution',ha='center',size=16,transform=fig.transFigure)
plt.text(0.400,0.955,'High Resolution',ha='center',size=16,transform=fig.transFigure)
plt.text(0.640,0.955,'HighRes - LowRes',ha='center',size=16,transform=fig.transFigure)
plt.text(0.880,0.955,'Observation',ha='center',size=16,transform=fig.transFigure)


cmap = [[ 'RdBu_r', 'RdBu_r', 'bwr', 'BrBG_r' ],
        [ 'YlGnBu_r', 'YlGnBu_r', 'bwr', 'YlGnBu_r' ]]
ext  = [[ 'both', 'both', 'both', 'both' ],
        [ 'max', 'max', 'both', 'max' ]]

unit = [ 'uatm', '$\mathrm{molC}\, \mathrm{m}^{-2}\,  \mathrm{yr}^{-1}$' ]
var = [ 'exp1', 'exp2', 'exp2-1', 'obs' ]

title_top = [[ 
    r"(a) pCO$_2$ bias relative to JMA-MLR", 
    r"(b) pCO$_2$ bias relative to JMA-MLR",
    r"(c) pCO$_2$ difference (HighRes - LowRes)" +"\n (1990-2018 mean)",
    r"(d) pCO$_2$ (1990-2018 mean)" + "\n JMA-MLR (Iida et al 2021)"],
             [
     "(e) INTPP (2003-2018 mean)",
     "(f) INTPP (2003-2018 mean)",
     "(g) INTPP difference (HighRes - LowRes)" +"\n (2003-2018 mean)",
     "(h) INTPP (2003-2018 mean)" + "\n mean of 3 MODIS-based OSU products" ],]

panel = 0
for item in range(2):

    if (item == 0):
        DS = DS_pco
        bounds1 = [-100.0, -50.0, -30.0, -20.0, -10.0, -5.0, 5.0, 10.0, 20.0, 30.0, 50.0, 100.0 ]
        tick_bounds1 = [-100.0, -50.0, -30.0, -20.0, -10.0, -5.0, 5.0, 10.0, 20.0, 30.0, 50.0, 100.0 ]
        bounds2 = [-30.0, -20.0, -15.0, -10.0, -7.0, -4.0, -2.0, 2.0, 4.0, 7.0, 10.0, 15.0, 20.0, 30.0]
        tick_bounds2 = bounds2
        bounds3 = np.arange(300,501,10)
        tick_bounds3 = np.arange(300,501,20)

    if (item == 1):
        DS = DS_int
        bounds1 = [0.0, 0.5, 1.0, 2.0, 4.0, 6.0, 8.0, 10.0, 15.0, 20.0, 30.0, 50.0]
        tick_bounds1 = [0.0, 0.5, 1.0, 2.0, 4.0, 6.0, 8.0, 10.0, 15.0, 20.0, 30.0, 50.0]
        #bounds1 = [-20.0, -10.0, -5.0, -1.0, -0.5, 0.5, 1.0, 5.0, 10.0, 20.0]  # for bias
        #tick_bounds1 = [-20.0, -10.0, -5.0, -1.0, -0.5, 0.5, 1.0, 5.0, 10.0, 20.0] # for bias
        bounds2 = [-10.0, -5.0, -2.0, -1.0, -0.2, -0.1, 0.1, 0.2, 1.0, 2.0, 5.0, 10.0]
        tick_bounds2 = [-10.0, -5.0, -2.0, -1.0, -0.2, -0.1, 0.1, 0.2, 1.0, 2.0, 5.0, 10.0]
        #bounds2 = [-5.0, -2.0, -1.0, -0.5, -0.2, -0.1, 0.1, 0.2, 0.5, 1.0, 2.0, 5.0, 10.0]
        #tick_bounds2 = [-5.0, -2.0, -1.0, -0.5, -0.2, -0.1, 0.1, 0.2, 0.5, 1.0, 2.0, 5.0, 10.0]
        bounds3 = [0.0, 1.0, 2.0, 4.0, 6.0, 8.0, 10.0, 12.0, 15.0, 20.0, 30.0, 50.0]
        tick_bounds3 = [0.0, 1.0, 2.0, 4.0, 6.0, 8.0, 10.0, 12.0, 15.0, 20.0, 30.0, 50.0]

    for nv_out in range(4):

        print("Drawing panel "+ str(panel))

        if (nv_out == 0 or nv_out == 1):
            bounds = bounds1
            ticks_bounds = tick_bounds1
            da = DS[var[nv_out]]
            units = unit[item]
            if (item == 0):
                msktmp = np.where( np.isnan(da.values), 0.0, 1.0 )
                datmp = np.where( np.isnan(da.values), 0.0, da.values )
                tmp1 = (datmp * datmp * area * msktmp * maskpjp).sum()
                tmp2 = (area * msktmp * maskpjp).sum()
                tmp3 = (datmp * area * msktmp * maskpjp).sum()
                rmse = np.sqrt(tmp1/tmp2)
                mead = tmp3/tmp2
                title = title_top[item][nv_out] + '\n   rmse= ' + '{:.1f}'.format(rmse) + ' ' + units + '   mean bias = ' + '{:.1f}'.format(mead) + ' '+ units
            else:
                title = title_top[item][nv_out]

        elif (nv_out == 2):
            bounds = bounds2
            ticks_bounds = tick_bounds2
            da = DS[var[nv_out]]
            title = title_top[item][nv_out]
            units = unit[item]
        else:
            bounds = bounds3
            ticks_bounds = tick_bounds3
            da = DS[var[nv_out]]
            title = title_top[item][nv_out]
            units = unit[item]


        im = da.plot(ax=ax[panel],cmap=cmap[item][nv_out],
                     levels=bounds,
                     extend=ext[item][nv_out],
                     add_colorbar=False,
                     transform=ccrs.PlateCarree())

        cbar = plt.colorbar(im,orientation='horizontal',spacing='uniform',
                            ticks=ticks_bounds, cax=ax_cbar[panel])
        cbar.set_label(label=units,fontsize=6,x=0.95,labelpad=0)
        cbar.ax.tick_params(labelsize=6)
        
        ax[panel].coastlines()
        ax[panel].set_xticks(np.arange(120,285.1,30),crs=ccrs.PlateCarree())
        ax[panel].set_yticks(np.arange(-10,60.1,10),crs=ccrs.PlateCarree())
        ax[panel].set_extent([99,286,-16,66],crs=ccrs.PlateCarree())
        ax[panel].xaxis.set_major_formatter(lon_formatter)
        ax[panel].yaxis.set_major_formatter(lat_formatter)
        ax[panel].set_aspect(1.2)
        ax[panel].set_xlabel('')
        ax[panel].set_ylabel('')
        ax[panel].set_title(title,{'fontsize':8, 'verticalalignment':'top', 'linespacing':0.85})
        ax[panel].tick_params(labelsize=7)
        ax[panel].patch.set_facecolor('lightgray')
        
        if (panel == 2 or panel == 6):
            ax[panel].vlines(x=[164.5],ymin=-15.0,ymax=63.0, colors="black",linestyle="dashed",linewidth=0.5,transform=ccrs.PlateCarree())
        panel += 1

#plt.subplots_adjust(left=0.04,right=0.98,bottom=0.06,top=0.85,wspace=0.16,hspace=0.15)

outpdf = outfile+'.pdf'
outpng = outfile+'.png'

plt.savefig(outpng, bbox_inches='tight', pad_inches=0.1)
plt.savefig(outpdf, bbox_inches='tight', pad_inches=0.1)

print("figure is saved to " + outpng + " and " + outpdf)

plt.show()
