# -*- coding: utf-8 -*-

import sys
import json
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

#-----------------------------------------------------------------------------
#if (len(sys.argv) < 2):
#    print ('Usage: ' + sys.argv[0] + ' region')
#    print ([ "(1) Tropics", "(2) SW-Subtropical", "(3) NW-Subtropical", "(4) E-Subtropical", "(5) Subarctic", "(6) JapanSea", "(7) OkhotskSea" ])
#
#    sys.exit()
#
#region = int(sys.argv[1]) - 1
#-----------------------------------------------------------------------------

mipcols = ["darkred", "darkblue"]
miparcs = ["orange", "lightblue"]

title_list = [ "(a) Tropics",
               "(b) SW-Subtropical", 
               "(c) NW-Subtropical", 
               "(d) E-Subtropical", 
               "(e) Subarctic",
               "(f) Japan Sea", 
               "(g) Okhotsk sea",
               ]

item_list = [ "Tropics", "SW-Subtropical", "NW-Subtropical", "E-Subtropical", "Subarctic", 
              "JapanSea", "OkhotskSea" ]

glb_list = [ "GLB-TRP", "GLB-SWST", "GLB-NWST", "GLB-EST", "GLB-SAR", "GLB-JPS", "GLB-OKH" ]
np_list = [ "NP-TRP", "NP-SWST", "NP-NWST", "NP-EST", "NP-SAR", "NP-JPS", "NP-OKH" ]

range_list = [ [-8.0, 20.0], [-5.0, 6.0], [-5.0, 6.0], [-5.0, 6.0], [-5.0, 6.0], [-1.0, 1.0], [-1.0, 1.0]]

plot_list = [ "LowRes", "HighRes" ]

rem_dict_glb = dict(zip(item_list,glb_list))
rem_dict_np = dict(zip(item_list,np_list))

##########################################
path_glb100km='../data/glb100km/'
path_np10km='../data/np10km/'
##########################################

infl1 = path_glb100km + "run-20211104o2/csv/Pac3_Volume_conversion_lyr_GLB100km_1987-2017.csv"
df1=pd.read_csv(infl1,index_col=0)
df1.rename(columns=rem_dict_glb,inplace=True)


infl4 = path_np10km + "csv/Pac3_Volume_conversion_lyr_NP10km_1987-2017.csv"
df4=pd.read_csv(infl4,index_col=0)
df4.rename(columns=rem_dict_np,inplace=True)

dfa=pd.concat([df1,df4],axis=1)

print(dfa)
#----------------------

fig = plt.figure(figsize=(11,6))
fig.suptitle('Volume conversion rate summary (1987-2017)',fontsize=18)
#fig.suptitle('Volume conversion rate summary (1987-2017) for '+item_list[region])

ax = [ plt.subplot(2,4,1),
       plt.subplot(2,4,2),
       plt.subplot(2,4,3),
       plt.subplot(2,4,4),
       plt.subplot(2,4,5),
       plt.subplot(2,4,6),
       plt.subplot(2,4,7) ]

nax=0
nreg=0
for item in item_list:

    dfb=dfa[[glb_list[nreg],np_list[nreg]]].copy(deep=True)
    rem_dict_bar = dict(zip([glb_list[nreg],np_list[nreg]],plot_list))
    dfb.rename(columns=rem_dict_bar,inplace=True)
    dfb.iloc[0:10].plot.bar(ax=ax[nax],subplots=False)
    ax[nax].set_title(item,fontsize=12)
    ax[nax].set_xlabel('')
    ax[nax].set_ylabel(r'[$\mathrm{Sv}$]',fontsize=10)
    ax[nax].set_xticklabels(dfb.index.values[0:10],rotation=45,fontsize=8)
    ax[nax].set_ylim(range_list[nreg][0],range_list[nreg][1])
    nreg+=1
    nax+=1

plt.subplots_adjust(top=0.90,bottom=0.08,left=0.05,right=0.98,hspace=0.33,wspace=0.38)
plt.savefig('fig/Volume_conversion_rate_summary_pac3_all.png')
plt.savefig('fig/Volume_conversion_rate_summary_pac3_all.pdf')
plt.show()
